<?php
/**
 * Projekt: WS-Reporter
 *          Spezifikation einer Datenquelle in Text-Datei exportieren
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    23.02.2023
 */
include 'includes/defender.inc';
require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
global  $query;
require_once './phpsec/createDBH.inc';

if (session_status() == PHP_SESSION_NONE)  session_start();

anmeldungPruefen();
adminRechteVerlangen();

$query = null;
$table = null;
if (array_key_exists('WSReportQuery', $_SESSION)){
    $query = $_SESSION['WSReportQuery'];
}
session_abort();

if (array_key_exists('table', $_REQUEST)){
    $table = $_REQUEST['table'];
}

if ($table != null && ($query == null
    || array_key_exists( 'reload',$_REQUEST)
    || strcmp($table, $query->class_table) != 0)){
        $query = new Query($table);
        require_once './includes/loadClassDef.inc';
        loadClassDef($query);
}
    
header("Content-Type: text/plain; charset=UTF-8");
header("Content-Disposition: attachment; filename=\"".$query->class_table.'-'.date_create('now')->format('Y-m-d').".wsrc\"");

ksort($query->metaData);
$metaData = $query->metaData;

foreach ($metaData as $metaRecord){
    if ( !isValid($metaRecord->selectFilterTable)) $metaRecord->getSelectOptions();
    $metaRecord->setTable($query->class_table);
    //    $class_table = "ET_".$class_table;
}

$logSql = new SqlAbfrage('WSR');
$logSql->sql = 'select userStored, timeStored, userReleased, timeReleased from wsrAdminQuery';
$logSql->addSel('table_name = ', $query->class_table);
$stmt = $logSql->executeE(false);
$z = $stmt->fetchObject();

echo "<WS-Report Class Version 1.2>\n";
echo "<classTable>".$query->class_table."\n";
echo "<queryVersion>0.7\n";
echo "<defaultTitle>".$query->defaultTitle."\n";
echo "<outputTitlePrefix>".$query->outputTitlePrefix."\n";
if (isValid($z->userStored)) echo '<userStored>'.$z->userStored."\n";
if (isValid($z->timeStored)) echo '<timeStored>'.$z->timeStored."\n";
if (isValid($z->userReleased)) echo '<userReleased>'.$z->userReleased."\n";
if (isValid($z->timeReleased)) echo '<timeReleased>'.$z->timeReleased."\n";
echo "<reportCSS>".$query->reportCSS."\n";
echo "<reportHoverMode>".$query->reportHoverMode."\n";
echo "<dwh>".$query->dwh."\n";
echo "<physTable>".$query->phys_table."\n";
echo "<tableDescription>\n";
echo $query->table_description."\n";
echo "<endTableDescription>\n";
echo "<tableHints>\n";
echo $query->table_hints."\n";
echo "<endTableHints>\n";

if ($query->qTranslations != null && count($query->qTranslations) > 0){
    foreach ($query->qTranslations as $tr){
        if (isValid($tr)){
            echo '<tableDescriptionTranslated>'.$tr."\n";
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select kommentar from queryComment';
            $sql->addSel('table_name = ',$query->class_table);
            $sql->addSel('qLanguage = ',$tr);
            $sql->sql .= ' order by SortNr';
            $stmt = $sql->executeE(true);
            if ($stmt !== false)
                while ($z = $stmt->fetchObject())
                    echo $z->kommentar;                
            echo "\n<endTableDescriptionTranslated>\n";
            
            echo '<tableHintsTranslated>'.$tr."\n";
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select kommentar from queryHints';
            $sql->addSel('table_name = ',$query->class_table);
            $sql->addSel('qLanguage = ',$tr);
            $sql->sql .= ' order by SortNr';
            $stmt = $sql->executeE(true);
            if ($stmt !== false)
                while ($z = $stmt->fetchObject())
                    echo $z->kommentar;
                    echo "\n<endTableHintsTranslated>\n";
        }
    }
}


echo "<queryType>".$query->queryType."\n";
if ($query->dimKey != null) echo "<queryDimKey>".$query->dimKey."\n";
if ($query->hatWerteAuswahl) echo "<hatWerteAuswahl>\n";
    
if ($query->histType > 0) {
    echo "<queryHistType>".$query->histType."\n";
    echo "<queryHistOption>".$query->historyOption."\n";
    if (isValid($query->histAb)) echo "<queryHistAb>".$query->histAb."\n";
    if (isValid($query->histBis)) echo "<queryHistBis>".$query->histBis."\n";
    if (isValid($query->histStichtag)) echo "<queryHistStichtag>".$query->histStichtag."\n";
    if (isValid($query->historyDate)) echo "<queryHistDate>".$query->historyDate."\n";    
}


if (isValid($query->qLanguage)) echo "<qLanguage>".$query->qLanguage."\n";
if (count($query->qTranslations) > 0){
    echo "<qTranslations>".$query->qTranslations[0];
    for ($i = 1; $i < count($query->qTranslations); $i ++) echo ' '.$query->qTranslations[$i];
    echo "\n";
}



foreach ($metaData as $metaRecord){
    $metaRecord->export();
}

foreach ($query->dimensions as $dimNr => $dim){
    if (false) $dim = new Dimension($z);
    echo "<dimension>".$dimNr."\n";
    echo "<dimensionName>".$dim->dimName."\n";
    echo "<dimensionPhysTable>".$dim->phys_table."\n";
    echo "<dimensionKey>".$dim->dimKey."\n";
    echo "<dimensionType>".$dim->dimType."\n";
    echo "<dimensionSelectVar>".$dim->dimSelectVar."\n";
    echo "<dimensionBaseDimNr>".$dim->baseDimNr."\n";
    echo "<dimensionBaseKey>".$dim->baseKey."\n";
    if ($dim->histType > 0){
        echo "<dimensionHistType>".$dim->histType."\n";
        echo "<dimensionHistOption>".$dim->historyOption."\n";
        if (isValid($dim->histAbSelect)) echo "<dimensionHistAb>".$dim->histAbSelect."\n";
        if (isValid($dim->histBisSelect)) echo "<dimensionHistBis>".$dim->histBisSelect."\n";
        if (isValid($dim->histStichtagSelect)) echo "<dimensionHistStichtag>".$dim->histStichtagSelect ."\n";
        if (isValid($dim->historyDate)) echo "<dimensionHistDate>".$dim->historyDate."\n";        
        if ($dim->dimIsFilter) echo "<dimensionDimIsFilter>\n";
    }
    echo "</dimension>\n";
}

echo "<rowVars>".count($query->rowVars)."\n";

for ($i = 0; $i < count($query->rowVars); $i++) {
    echo "<rowVar>".$query->rowVars[$i]."\n";
}
echo "<endRowVars>\n";

echo "<valueVars>".count($query->valueVars)."\n";

for ($i = 0; $i < count($query->valueVars); $i++) {
    echo "<valueVar>".$query->valueVars[$i]."\n";
}
echo "<endValueVars>\n";

echo "<filterVars>".count($query->filterVars)."\n";
for ($i = 0; $i < count($query->filterVars); $i++) {
    echo "<filterVar>".$query->filterVars[$i]."\n";
}
echo "<endFilterVars>\n";

for ($i = 0; $i < count($query->defaultFilter); $i++) {
    $df = $query->defaultFilter[$i];
    echo "<defaultFilter>".$df->column_name."\n";
    if ($df->fs_von != null) echo "<defaultFilterVon>".str_replace(array("\n","\r","\t"),' ',$df->fs_von)."\n";
    if ($df->fs_bis != null) echo "<defaultFilterBis>".str_replace(array("\n","\r","\t"),' ',$df->fs_bis)."\n";
    echo "<endDefaultFilter>\n";
}


// Die Dimensionen müssen eingelesen sein, bevor mit addQueries gearbeitet
// werden kann. Wenn die mit gleichnamiger Klasse initialisiert werden,
// müssen die Dimensionen in der Basisklasse angelegt sein
for ($q = 0; $q < $query->addQueries->size(); $q++) {
    $ad = $query->addQueries->getPos($q);
    echo "<addQuery>".$ad->class_table."\n";
    $cvs = $ad->calcVars;
    for ($i=0; $i < count($cvs); $i++) {
        $vn = $cvs[$i];
        echo "<calculatedVar>".$vn."\n";
        $mr = $ad->getQuery()->metaData[$vn];
        $mr->export();
    }
    echo "</addQuery>\n";		
}


if (! isValid($query->qLanguage) || count($query->qTranslations) <= 0) return;

/**
 * Übersetzungen exportieren
 */

echo "\n<WS-Report Translations Version 1.0>\n";
// require_once './includes/qTranslate.inc';

$sql = new SqlAbfrage('WSR');

$sql->sql = "select distinct q.type, q.text, t.* ";

$sql->sql .= "from (select 1 as sort, ".qTranslate::QT_LABEL." as type, attribute as text from queryAttribute";
$sql->addSel('table_name = ', $table);
$sql->addWhere('SatzTyp in (1,2) ');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQLTABLE." as type, attribute as text from queryAttribute";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);
$sql->addWhere('SatzTyp = 4 ');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, column_label as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_TEXT." as type, column_desc as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);
/*
 $sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQL." as type,";
 $sql->sql .= 'case when dimNr > 0 then replace(qSelectVar,concat("d",dimNr,"."),"dim.") else qSelectVar end ';
 $sql->sql .= " as text from queryColumnMetadata";
 $sql->firstWhere = true;
 $sql->addSel('table_name = ', $table);
 */
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQLDISP." as type, ";
$sql->sql .= 'case when dimNr > 0 then replace(qDisplayVar,concat("d",dimNr,"."),"dim.") else qDisplayVar end ';
$sql->sql .= " as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQL." as type, s.label as text "
    ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
$sql->firstWhere = true;
$sql->addWhere("m.qFilterTyp = 7 and m.column_name = s.column_name and m.table_name = s.table_name ");
$sql->addSel('m.table_name = ', $table);


$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, s.value as text "
    ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
$sql->firstWhere = true;
$sql->addWhere("m.qFilterTyp = 7 and m.column_name = s.column_name and m.table_name = s.table_name ");
$sql->addSel('m.table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, s.label as text "
    ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
    $sql->firstWhere = true;
    $sql->addWhere("m.qFilterTyp != 7 and m.column_name = s.column_name and m.table_name = s.table_name ");
    $sql->addSel('m.table_name = ', $table);
    
$sql->sql .="\n) q join queryTranslations t on t.".$query->qLanguage." = trim(q.text) "; // and q.type = t.qLanguageType";
$sql->sql .="\n and (q.type != ".qTranslate::QT_SQLTABLE.' or t.qLanguageType = '.qTranslate::QT_SQLTABLE.')';

$sql->sql .="\nwhere q.text is not null and q.text != ''";
$sql->sql .="\norder by type, q.text";

$stmt = $sql->execute();
if ($stmt === false){
    echo '<pre>';
    var_dump($sql);
    var_dump($sql->errorInfo());
    die();
}

echo '<translateFrom>'.$query->qLanguage."\n";
while ($z = $stmt->fetch(PDO::FETCH_ASSOC)){
    echo '<'.$query->qLanguage.'>'.$z['qLanguageType']."\n";
    echo $z[$query->qLanguage]."\n";
    echo '</'.$query->qLanguage.">\n";
    foreach ($query->qTranslations as $t)
        if (isValid($t))
        if (isValid($z[$t])){
            echo '<'.$t.">\n";
            echo $z[$t]."\n";
            echo '</'.$t.">\n";            
    }
    echo "<tReady>\n";    
}
echo "\n</WS-Report Translations>\n";
?>


 ?>