<?php
/**
 * Projekt: WS-Reporter
 *          Alle Klassen-Deklarationen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    3.1.2023
 *
 */
// echo ("Hier werden die Klassen deklariert.<br>");
require_once './includes/funktionen.inc';
require_once './includes/qTranslate.inc';
require_once './includes/wsrTranslate.inc';

/**
 * 
 * @param string $valName
 */
function getSourceTable($valName){
    if (substr($valName,1,1) == '='
        || substr($valName,1,1) == ':') return substr($valName,0,1);
    return 'base';
}

class DisplayOptions{
    const ZAHL = 1;
    const VBAR = 10;
    const HBAR = 20;
    const ZAHLVBAR = 11;
    const ZAHLHBAR = 21;
    
    const klartext = array(
        self::ZAHL => "nur Zahlen",
        self::VBAR => "vert. Balken",
        self::HBAR => "hor. Balken",
        self::ZAHLVBAR => "Zahlen und vert. Balken",
        self::ZAHLHBAR => "Zahlen und hor. Balken"
    );
    public static function getKlartext($option){
        if (isValid($option) && $option > 0) return WSRTranslate::translate(self::klartext[$option]);
        return '';
    }
}

class HistoryOptions{
    const HSUEBERNEHMEN = 0;
    const HSAKTUELL = 1;
    const HSSTICHTAGBASE = 2;
    const HSSTICHTAGSELBST = 3;
    const HSVJENDE = 4;
    const HSVVJENDE = 5;
    const HSDATUM = 6;
    const HSOHNEAUSWAHL = 7;
    const HSDELAKT = 11;
    const HSDELBIS = 12;
    const HSDELAKTJB = 13;
    const HSDELBISJB = 14;
    
    
    const klartext = array(
        self::HSUEBERNEHMEN => "übernehmen",
        self::HSAKTUELL => "aktueller Stand",
        self::HSSTICHTAGBASE => "Stichtag Basisquelle",
        self::HSSTICHTAGSELBST => "Stichtag der Quelle",
        self::HSVJENDE => "Ende Vorjahr",
        self::HSVVJENDE => "Ende Vor-Vorjahr",
        self::HSDATUM => "Datum eingeben",
        self::HSOHNEAUSWAHL => "keine Auswahl",
        self::HSDELAKT => "aktive Einträge",
        self::HSDELBIS => "gelöschte Einträge",
        self::HSDELAKTJB => "aktiv am Jahresbeginn",
        self::HSDELBISJB => "gelöscht am Jahresbeginn"
    );
    public static function getKlartext($option){
        if ($option == null) return '-';
        if (isValid($option)) {
            if ( array_key_exists($option, HistoryOptions::klartext)) return WSRTranslate::translate(HistoryOptions::klartext[$option]);
            else {
                echo 'HistoryOptions::getKlartext() von ';
                var_dump($option);
            }
        }
        return '';
    }
}



class SortOptions{
    const NOSORTVAL = 0;
    const SORTLASTGROUP = 1;
    const SORTTABLE = 2;
    const SORTSPECIAL = 3;
    const SORTSPECIALSZ = 4;
    
    const klartext = array(
        self::NOSORTVAL => "keine Sortierung",
        self::SORTLASTGROUP => "nur in letzter Gruppe",
        self::SORTTABLE => "ganze Tabelle (einfach)",
        self::SORTSPECIAL => "alle Gruppen in der Tabelle",
        self::SORTSPECIALSZ => "alle Gruppen nach Summenzeilen"
    );
    public static function getKlartext($option){
        if (isValid($option) && $option >= 0) return WSRTranslate::translate(self::klartext[$option]);
        return '';
    }
}

class Sort{
    const NONE = 0;
    const AUF = 1;
    const AB = 3;
    const SORTINTERN = 1;
    const SORTANZEIGE = 2;
    const SORTINTERNDESC = 3;
    const SORTANZEIGEDESC = 4;
    
    const klartext = array(
        self::NONE => "nicht sortieren",
        self::SORTINTERN => "aufsteigend",
        self::SORTANZEIGE => "nach Anzeige",
        self::SORTINTERNDESC => "absteigend",
        self::SORTANZEIGEDESC => "nach Anzeige absteigend"
    );
    
    public static function getKlartext($option){
        if (isValid($option) && $option >= 0) return WSRTranslate::translate(self::klartext[$option]);
        return '';
    }
}

class ChartOptions{
    const KEINESUMMEN = 0;
    const NURSUMMEN = 1;
    const IMMER = 2;
    const SKALIEREGRUPPE = 10;
    const SKALIEREWERT = 11;
    const SKALIEREZEILE = 12;
    const SKALIERESPALTE = 13;
    const SKALIERETABELLE = 14;
    const klartext = array(
        self::KEINESUMMEN => "keine Summen",
        self::NURSUMMEN => "nur Summen",
        self::IMMER => "immer",
        self::SKALIEREGRUPPE => "Gruppe",
        self::SKALIEREWERT   => "Wert",
        self::SKALIEREZEILE => "Zeile",
        self::SKALIERESPALTE => "Spalte",
        self::SKALIERETABELLE => "Tabelle"
    );
    public function getKlartext($option){
        if (isValid($option) && $option >= 0) return WSRTranslate::translate(self::klartext[$option]);
        return '';
    }
}

class Dimension{
    const SIMPLEDIM = 1;
    const ADDEDDIM = 2;
    public $dimNr = 1;
    public $baseDimNr = -1;
    public $dimType = 0;
    public $dimName = null;
    public $phys_table = null;
    public $dimKey = null;
    public $baseKey = null;
    public $dimSelectVar = null;
    public $histType = Query::KEINE;
    public $historyOption = HistoryOptions::HSUEBERNEHMEN;
    public $historyDate = null;
    public $historyQueryDate = null;
    public $histAbSelect = null;
    public $histBisSelect = null;
    public $histStichtagSelect = null;
    
    public $dimIsFilter = false; // Wenn true wirkt die Dimension auch als Filter, wenn daraus nichts selektiert wurde
    public $use = 0;  // Benutzung im Report wird in createReport() ermittelt
    
    function __construct($z){  // eingelesener Datensatz
        if ($z == null) return;
        $this->dimName = $z->qDimName;
        $this->dimNr = $z->SortNr;
        $this->baseDimNr = $z->baseDimNr;
        $this->phys_table = $z->qDimTable;
        $this->dimType = $z->qDimType;
        $this->dimKey = $z->qDimKey;
        $this->baseKey = $z->qBaseKey;
        $this->dimSelectVar = $z->qSelectVar;
        $this->histType = $z->qHistType;
        $this->historyOption = $z->qHistoryOption;
        $this->historyDate = $z->qHistoryDate;
        $this->dimIsFilter = $z->qDimIsFilter;
        $this->histAbSelect  = $z->qGueltigAb;
        $this->histBisSelect  = $z->qGueltigBis;
        $this->histStichtagSelect  = $z->qStichtag;
        
    }      
    
    public function clone(){
        $newDim = new Dimension(null);
        
        $newDim->dimNr = $this->dimNr;
        $newDim->baseDimNr = $this->baseDimNr;
        $newDim->dimType = $this->dimType;
        $newDim->dimName = $this->dimName;
        $newDim->phys_table = $this->phys_table;
        $newDim->dimKey = $this->dimKey;
        $newDim->baseKey = $this->baseKey;
        $newDim->dimSelectVar = $this->dimSelectVar;
        $newDim->histType = $this->histType;
        $newDim->historyOption = $this->historyOption;
        $newDim->historyDate = $this->historyDate;
        $newDim->historyQueryDate = $this->historyQueryDate;
        $newDim->histAbSelect = $this->histAbSelect;
        $newDim->histBisSelect = $this->histBisSelect;
        $newDim->histStichtagSelect = $this->histStichtagSelect;
        
        $newDim->dimIsFilter = $this->dimIsFilter; // Wenn true wirkt die Dimension auch als Filter, wenn daraus nichts selektiert wurde
        return $newDim;
    }
    
    
    public function getKlartextHistoryOption() {
        return HistoryOptions::getKlartext($this->historyOption);
    }
    
    public function save(){
        global $dbh, $query;
        require_once './phpsec/createDBH.inc';
        static $insertStatement = null ;
        if ($insertStatement == null){
            $insertStatement = $dbh->prepare(
                "replace into  queryDimensions set "
                ." table_name = :table_name"
                .", qDimName = :dimName"
                .", SortNr = :dimNr"
                .", baseDimNr = :baseDimNr"
                .", qDimTable = :phys_table"
                .", qDimType = :dimType"
                .", qDimKey = :dimKey"
                .", qBaseKey = :baseKey"
                .", qSelectVar = :dimSelectVar"
                .", qHistType = :histType"
                .", qHistoryOption = :historyOption"
                .", qHistoryDate = :historyDate"
                .", qDimIsFilter = :dimIsFilter"
                .", qGueltigAb = :histAbSelect"
                .", qGueltigBis = :histBisSelect"
                .", qStichtag = :histStichtagSelect"
                );
        }
        if ($insertStatement === false){
            http_response_code(425);
            echo "FEHLER Dim-InsertStatement:\n";
            var_dump($dbh->errorInfo());
            return;
            
        }
        
        $a=array(
            "table_name" => $query->class_table
            , "phys_table" => $this->phys_table
            , "dimName" => $this->dimName
            , "dimNr" => $this->dimNr
            , "baseDimNr" => $this->baseDimNr
            , "dimType" => $this->dimType
            , "dimKey" => $this->dimKey
            , "baseKey" => $this->baseKey
            , "dimSelectVar" => $this->dimSelectVar
            , "histType" => $this->histType
            , "historyOption" => $this->historyOption
            , "historyDate" => $this->historyDate
            , "dimIsFilter" => 0 // $this->dimIsFilter
            , "histAbSelect" => $this->histAbSelect
            , "histBisSelect" => $this->histBisSelect
            , "histStichtagSelect" => $this->histStichtagSelect
        );
        $erg = $insertStatement->execute($a);
        
        if ($erg === false){
            http_response_code(425);
            var_dump($insertStatement->errorInfo());
            var_dump($insertStatement);
            var_dump($a);
            return;
        }
        $insertStatement->closeCursor(); // ???
        return $erg;
    }

    
    public function getReqDim(){
        global $query;
        $reqDim = array();
            $dim = $this;
            $ld = -2;
            while ($dim->baseDimNr >= 0){
                $ld = $dim->baseDimNr;
                if ($dim->baseDimNr >= $this->dimNr){
                    error_log('Fehler bei Verkettung der Dimensionen');
                    error_log('Die Dimensionen Nr. '.$dim->baseDimNr.' ist größer als die eigene Nr.: '.$this->dimNr);
                    http_response_code(425);
                    echo "Fehler bei Verkettung der Dimensionen\n";
                    echo 'Die Dimensionen Nr. '.$dim->baseDimNr.' ist größer als die eigene Nr.: '.$this->dimNr;
                    die();                    
                }
                    
                if ($dim->baseDimNr >= 0) $reqDim[] = $dim->baseDimNr;
                
                if (! array_key_exists($dim->baseDimNr, $query->dimensions)
                    || $query->dimensions[$dim->baseDimNr] == null){
                    error_log('Fehler bei Verkettung der Dimensionen');
                    error_log('Die Dimensionen Nr. '.$dim->baseDimNr.' gibt es nicht.');
                    error_log('Aufgerufen aus der Dimensionen '.$dim->dimNr.': '.$dim->dimName.'.');
                    error_log (print_r($query->dimensions,true));
                    http_response_code(425);
                    echo "Fehler bei Verkettung der Dimensionen:\n";
                    echo 'Die Dimensionen Nr. '.$dim->baseDimNr." gibt es nicht.\n";
                    echo 'Aufgerufen aus der Dimensionen '.$dim->dimNr.': '.$dim->dimName.'.';
                    die();                    
                }
                $dim = $query->dimensions[$dim->baseDimNr];
                if ($ld == $dim->baseDimNr || $dim == null ){
                    error_log('Endlosschleife bei Dim.Nr. '.$ld);
                    error_log (print_r($query->dimensions,true));
                    http_response_code(425);
                    echo 'Endlosschleife bei Dim.Nr. '.$ld;
                    die();
                }
            }
        return $reqDim;
    }
    /**
     * 
     * @param MetaRecord $mr
     * @param int $where 1='Row',2='Value' oder 3='Filter'
     */
    public function insertVar($mr,$where,$reqDim){
        global $query;
        if ($reqDim === -2) $reqDim = $this->getReqDim();
        $name = $mr->getName();
        if (array_key_exists($name, $query->metaData)) {
            $name = ''.$this->dimNr.$name;
        }
        switch ($where){
            case 1: $vec = &$query->rowVars;
            break;
            case 2: $vec = &$query->valueVars;
            break;
            case 3: $vec = &$query->filterVars;
            break;
            default:
                http_response_code(425);
                echo 'FEHLER bei Dimension::insertVar: where='.$where.' ist ungültig';
                die();
                break;
        }
        
        if (! in_array($name, $vec)
            && ! array_key_exists($name, $query->metaData)){
                $vec[] = $name;
                $mr->dimNr = $this->dimNr;
                if ($where == 1){
                    if (strpos($mr->selectVar, 'dim.') === false) $mr->selectVar = 'dim.'.$mr->selectVar;
                    if (isValid($mr->displayVar) && (strpos($mr->displayVar, 'dim.') === false))
                        $mr->displayVar = 'dim.'.$mr->displayVar;
                }
                $mr->selectVar = str_replace('dim.', 'd'.$this->dimNr.'.', $mr->selectVar);
                $mr->displayVar = str_replace('dim.', 'd'.$this->dimNr.'.', $mr->displayVar);
                $mr->selectFilterVar = str_replace('dim.', 'd'.$this->dimNr.'.', $mr->selectFilterVar);
                $mr->selectFilterVar2 = str_replace('dim.', 'd'.$this->dimNr.'.', $mr->selectFilterVar2);
                $mr->dimIsSet = true;
                $mr->reqDim = $reqDim;
                // Hier muss noch der Name aus der Dimension stehen!
                if ($mr->hasMetadataSelectOptions()) {
                    $mr->readSelections();
                    if ($mr->filterType == MetaRecord::SONDERFILTERSEL){
                        $mr->setSelectOptionsDimnr($this->dimNr);
                    }
                }
                $mr->setTable($query->class_table);
                $mr->setColumnName($name);
                $query->metaData[$name] = $mr;
        }else{
            http_response_code(425);
            echo $name.' ist schon eingetragen.';
            die();
        }
    }
    
    public function initAddedDim($key){
        global $query;
        require_once './includes/loadClassDef.inc';
        $dimQuery = new Query($this->dimName);
        loadSimpleClassDef($dimQuery);
        $this->baseDimNr = -1;
        $this->phys_table = $dimQuery->phys_table;
        $this->dimType = Dimension::ADDEDDIM;
        $this->histType = $dimQuery->histType;        
        $this->historyOption = $dimQuery->historyOption;
        $this->histAbSelect = null;
        
        if (isValid($dimQuery->histAb)){
            $mr = $dimQuery->getReportMetaRecord($dimQuery->histAb);
            if ($mr != null) {
                $sv = $mr->selectVar;
                if (strpos($sv, 'dim.') === false) $sv = 'd'.$this->dimNr.'.'. $sv;
                else $sv = str_replace('dim.', 'd'.$this->dimNr.'.', $sv);
                $this->histAbSelect = $sv;
            }
        }
        $this->histBisSelect = null;
        if (isValid($dimQuery->histBis)){
            $mr = $dimQuery->getReportMetaRecord($dimQuery->histBis);
            if ($mr != null) {
                $sv = $mr->selectVar;
                if (strpos($sv, 'dim.') === false) $sv = 'd'.$this->dimNr.'.'. $sv;
                else $sv = str_replace('dim.', 'd'.$this->dimNr.'.', $sv);
                $this->histBisSelect = $sv;
            }
        }
        $this->histStichtagSelect = null;
        if (isValid($dimQuery->histStichtag)){
            $mr = $dimQuery->getReportMetaRecord($dimQuery->histStichtag);
            if ($mr != null) {
                $sv = $mr->selectVar;
                if (strpos($sv, 'dim.') === false) $sv = 'd'.$this->dimNr.'.'. $sv;
                else $sv = str_replace('dim.', 'd'.$this->dimNr.'.', $sv);
                $this->histStichtagSelect = $sv;
            }
        }
        
        $dimKey = $dimQuery->dimKey;
        $this->dimKey = $dimKey;
        $mr = $dimQuery->getReportMetaRecord($dimKey);
        $dimSelectVar = $mr->selectVar;
        if (strpos($dimSelectVar, 'dim.') === false) $dimSelectVar = 'dim.'.trim($dimSelectVar," \n");
        $this->dimSelectVar = str_replace('dim.', 'd'.$this->dimNr.'.', $dimSelectVar);
        if (in_array($key, $query->rowVars)){
            $this->baseKey = $key;
            $baseKey = $key;
            $this->baseDimNr = $query->getReportMetaRecord($baseKey)->dimNr;
            $dim = $this;
            $ld = -2;
            $reqDim = $this->getReqDim();
        }else{
            error_log('FEHLER: Kann '.$key.' in der Basisquelle nicht finden.');
            return 'FEHLER: Kann '.$key.' in der Basisquelle nicht finden.';
        }
        
        foreach ($dimQuery->rowVars as $name){
            if ($name != $dimKey){
                $mr = $dimQuery->getReportMetaRecord($name);
                $this->insertVar($mr,1,$reqDim);
            }
        }
        foreach ($dimQuery->valueVars as $name){
            if ($name != $dimKey){
                $mr = $dimQuery->getReportMetaRecord($name);
                $this->insertVar($mr,2,$reqDim);
            }
        }
        foreach ($dimQuery->filterVars as $name){
            if ($name != $dimKey){
                $mr = $dimQuery->getReportMetaRecord($name);
                $this->insertVar($mr,3,$reqDim);
            }
        }
    }
    
    public function initSimpleDim(){
        require_once './includes/loadClassDef.inc';
        $dimQuery = new Query($this->dimName);
        loadSimpleClassDef($dimQuery);
        $this->initAddedDim($dimQuery->dimKey);        
    }
    
}

class DefaultFilter{
    public $column_name = null;
    public $fs_von = null;
    public $fs_bis = null;
    
    public function __construct($name) {
        $this->column_name = $name;
    }
    
    public function getName() {
        return $this->column_name;
    }
    
}



class MetaRecord {
// FilterType    
    const NONE = 0;
    const SELECT = 1;
    const TEXT = 2;
    const VON_BIS = 3;
    const VON_BIS_SELECT = 4;
    const SONDERFILTER = 5;  // Wird ausgewählt und hat keine Werte
    const TEXTEXAKT = 6;
    const SONDERFILTERSEL = 7;  // Ein Name aber mehrere Optionen auswählbar     
    const TEXTMITLABEL = 8;   // Text und Label aus Tabelle, es wird aber keine Auswahlliste aufgebaut
    const TEXTLIKE = 9;   // Text mit LIKE-Filter 
    const LIST = 10;   // Text mit LIST-Filter
    const WINDOWFILTER = 11; // Wird beim Aufbau des Reports erzeugt.
    const WERTELISTE = 101; // Pseudo-Filter für Werte-Liste zum Ersetzen in Reports
    
    const klartextFilter = array(
        self::NONE => "",
        self::SELECT => "Auswahl",
        self::LIST => "Liste",
        self::TEXT => "Text (enthält)",
        self::VON_BIS => "von bis",
        self::VON_BIS_SELECT => "von bis Auswahl",
        self::SONDERFILTER => "Sonderfilter",  // Wird ausgewählt und hat keine Werte
        self::TEXTEXAKT => "Text (exakt)",
        self::TEXTLIKE => "Text mit Muster-Auswahl",
        self::TEXTMITLABEL => "Text mit Label",
        self::SONDERFILTERSEL => "Sonderfilter Auswahl",  // Ein Name aber mehrere Optionen auswählbar
        self::WINDOWFILTER => "Anzeige-Filter",  // Ein Name aber mehrere Optionen auswählbar
        self::HISTDEL => "Hist.: Gelöscht am"  // Sonderfilter für gelöscht_am-Werte
//        self::HISTSTICHTAG => "Hist.: Stichtag"  // Sonderfilter: Setzt Stichtags-Wert ??? 
        
    );
    public static function getKlartextFilter($option){
        if (isValid($option) && $option > 0) return WSRTranslate::translate(self::klartextFilter[$option]);
        return '';
    }
    public function getMyKlartextFilter(){
        return WSRTranslate::translate(self::klartextFilter[$this->filterType]);
    }
    
    
// Type
    const INT = 1;
    const DECIMAL = 2;
    const STRING = 3;
    const DATE = 4;
    const TIME = 5;
    const DATETIME = 6;
    const DATETIMES = 7;
    const TIMES = 8;
    const WEBLINK = 11;   // Sonderfall: Select-SQL wird übersetzt
    const GELD = 12;
    
    const HISTAB = 21;   // gueltig_ab-Wert in Historisierung
    const HISTBIS = 22;   // gueltig_bis-Wert in Historisierung
    const HISTDEL = 23;   // geloescht_am-Wert in Historisierung
    const HISTSTICHTAG = 24;   // Stichtag in Historisierung
    

    const klartextType = array(
        self::INT => "Ganzzahl",
        self::DECIMAL => "Dezimalzahl",
        self::GELD => "Geld",
        self::STRING => "Text",
        self::WEBLINK => "Web-Link",
        self::DATE => "Datum",
        self::TIME => "Zeit",
        self::TIMES => "Zeit mit Sekunden",
        self::DATETIME => "Datum und Zeit",
        self::DATETIMES => "Datum und Zeit mit Sek.",
        self::HISTAB => "Hist.: Gültig ab",
        self::HISTBIS => "Hist.: Gültig bis",
        self::HISTDEL => "Hist.: Gelöscht am",
        self::HISTSTICHTAG => "Hist.: Stichtag",
    );
    public static function getKlartextType($option){
        if (isValid($option) && $option > 0) return WSRTranslate::translate(self::klartextType[$option]);
        return '';
    }

    public function getMyKlartextType(){
        return self::klartextType[$this->type];
    }
    
    
    /* Wird aktuell noch nicht verarbeitet
    const INROWCOLUMN = 1;
    const INVALUES = 2;
    const FILTERONLY = 3;
    public $posInquery = MetaRecord::INROWCOLUMN;
    */

    
    public function getMyKlartextFilterSort(){
        // wird in Sort::getKlartext übersetzt
        return Sort::getKlartext($this->filterSort);
    }
    
    /*
     * Name der class_table aus der Query.
     * Das hilft nichts, wenn das Query-Objekt der verknüpften Quelle benötigt wird.
     */
    private $table_name = null;
    /*
     * Das ist jetzt ein sehr schlampiger Hack für die Anwendung der Filter bei readSelectOptions():
     * Grundsätzlich werden die Filter nur benötigt, wenn anschließend eine Auswahlliste angezeigt werden soll.
     * Aber dann müssen die Filter verwendet werden, die in der jeweiligen Query gelten. 
     * Deshalb benötigt readSelectTable() die Information, ob es sich um die base-Query oder eine Joined-Query
     * handelt. Dazu setzt enterFilterSpec() hier das kennzeichen der Query oder 'base'.
     * getSelectOptions() setzt dieses Kennzeichen wieder zurück. 
     */
    public $queryOfSelect = null; 
    /*
     * Name der Variablen (Zeile, Wert, Spalte, Filter)
     */
    private $column_name = null;
    
    public $type = self::STRING;
    public $htmlData = "th";
    public $label = "";
    public $description = "";
    public $hints = "";
    public $decimals = 0;
    public $align = "right"; // oder left oder center?
    /**
     * Kennzeichen Eingabefeld
     */
    public $enterValue = false;
    public $enterValueSize = 15;
    public $qSetVarname = null;
    
    public $filterType = self::NONE;
    /*
     * Wenn es in der Datenquelle mindestens einen "reqired" Filter
     * gibt, muss in der Abfrage mindestens einer dieser Filter
     * gesetzt sein.
     * Muss noch implementiert werden.
     */
    public $filterRequired = false;
    public $filterRequiredGroup = 0;
    
    private $selectOptions = null;
    public $optionsPointer = -1;
    public $selectVar = null;
    public $displayVar = null;
    public $selectFilterVar = null;
    public $selectFilterTable=null;
    public $selectFilterVar2 = null;
    public $selectFilterTable2=null;
    public $filterSort = Sort::SORTINTERN;
    public $chartColor = "#6666ff";
    public $chartNegColor = "#FF3333";
    public $chartGroup = null;
    public $chartMin = null;
    public $chartMax = null;
    
    /**
     * Informationen für Zeilen und Filter aus Dimensionen 
     * @var $dimNr integer Nummer der zugehörigen Dimension. -1 wenn aus base
     */
    public $dimNr = -1;  // Nummer der Dimension in $query
    public $reqDim = array();  // Zusätzlich benötigte Dimensionen zur Ermittlung
    public $dimIsSet = false; // Kennzeichen, dass
    public $hidden = false; // Kennzeichen zum Verbergen bei der Report-Erstellung
    
    public function init($table,$column){
        $this->table_name = $table;
        $this->column_name = $column;
        $this->label = $column;
        $this->selectVar = $column;
//        $this->displayVar = $column;
    }

    /**
     * Formatiert den Wert gemäß den Metadaten Typ und Decimals
     * @param string $value
     * @return string
     */
    public function formatValue($value){
        if (!isValid($value)) return $value;
        if ($this->filterType == self::WINDOWFILTER) return $value;
        switch ($this->type) {
            case self::HISTSTICHTAG:
            case self::INT:
            case self::STRING: return $value;
            case self::WEBLINK: return qTranslate::translate($value);
            case self::DECIMAL:
            case self::GELD:
            case self::WINDOWFILTER:
                $newValue = WSRFormats::formatDecimal($value,$this->decimals);
                break;
            case self::DATE:
            case self::HISTAB:
            case self::HISTBIS:
            case self::HISTDEL:
                $d = DateTime::createFromFormat('Y-m-d',$value);
                if ($d === false) {
                    echo 'Ungültiges Datum für '.$this->column_name.': '.$value;
                    die();
                }
                $newValue = WSRFormats::formatDate($d);
                break;
            case self::TIME:
                $d = DateTime::createFromFormat('H:i:s',$value);
                if ($d === false) {
                    echo 'Ungültige Zeit für '.$this->column_name.': '.$value;
                    die();
                }
                $newValue = WSRFormats::formatTime($d);
                break;
            case self::TIMES:
                $d = DateTime::createFromFormat('H:i:s',$value);
                if ($d === false) {
                    echo 'Ungültige Zeit für '.$this->column_name.': '.$value;
                    die();
                }
                $newValue = WSRFormats::formatTimeS($d);
                break;
                
            case self::DATETIME:
                $d = DateTime::createFromFormat('Y-m-d H:i:s',$value);
                if ($d === false) {
                    echo 'Ungültiges Datum/Zeit-Format für '.$this->column_name.': '.$value;
                    die();
                }
                $newValue = WSRFormats::formatDateTime($d);
                break;
            case self::DATETIMES:
                $d = DateTime::createFromFormat('Y-m-d H:i:s',$value);
                if ($d === false) {
                    echo 'Ungültiges Datum/Zeit-Format für '.$this->column_name.': '.$value;
                    die();
                }
                $newValue = WSRFormats::formatDateTimeS($d);
                break;
                
            default:
                return $value;
                break;
        }
//        error_log('MR-Format '.$this->column_name.' ergibt '.$value.' -> '.$newValue);
        if ($newValue === false) return $value;
        return $newValue;
    }
    
    /**
     * Benutzer-Eingaben für Filter-Werte konvertieren:
     * Vor Version 0.132 (Report-Version == 1) waren Zahlen und Datums-Werte
     * der Benutzer im Eingabe-Format (Deutsch) gespeichert. 
     * Seit Version 0.132 (mit Übersetzungen) werden die Eingaben direkt 
     * bei der Eingabe aus der aktuellen Sprache in das Datenbank-Format konvertiert.
     * 
     * Dazu muss der Typ, Zahl oder Datum/Zeit, und der Filter-Typ, manuelle Eingabe,
     * geprüft werden. Bei Auswahllisten wird immer schon das Datenbank-Format übergeben.
     * @param string $value
     * @return string
     */
    public function reformatValue($value){
        if (!isValid($value)) return $value;
        switch ($this->filterType) {
            case self::SONDERFILTER:
            case self::SONDERFILTERSEL:
            case self::SELECT:
            case self::VON_BIS_SELECT:
            case self::LIST:
            case self::WERTELISTE:
            case self::HISTDEL:
            case self::WINDOWFILTER:
                // An dieser Stelle ist der Typ des Window-Filters unbekannt
                return $value;
            break;
            
            default:
                ;
            break;
        }
        switch ($this->type) {
            case self::HISTSTICHTAG:
            case self::WEBLINK:
            case self::STRING: return $value;
            case self::INT:
            case self::DECIMAL:
            case self::GELD:
            case self::WINDOWFILTER:
                $newValue = WSRFormats::stdNumber($value);                
            break;    
            case self::DATE:
            case self::HISTAB:
            case self::HISTBIS:
            case self::HISTDEL:
                $newValue = WSRFormats::stdDate($value);
                break;
            case self::TIME:
            case self::TIMES:
                $newValue = WSRFormats::stdTime($value);
                break;
            
            case self::DATETIME:
            case self::DATETIMES:
                $newValue = WSRFormats::stdDateTime($value);
                break;
            
            default:
                return $value;
            break;
        }
        if ($newValue === false) return $value;
        return $newValue;
    }
    
    
    public function import($file) {
        global $queryVersion;
        while (($zeile = fgets($file)) !== false) {
            $zeile = trim($zeile);            
            if (strpos($zeile,"<classTable>") === 0 && strlen($zeile) > strlen("<classTable>")
                ) $this->table_name = substr($zeile, strlen("<classTable>"));
            
            if (strpos($zeile,"<name>") === 0 && strlen($zeile) > strlen("<name>")){
                $this->column_name = substr($zeile, strlen("<name>"));
            }
            if (strpos($zeile,"<type>") === 0 && strlen($zeile) > strlen("<type>")){
                $this->type = substr($zeile, strlen("<type>"));
            }
            if (strpos($zeile,"<label>") === 0 && strlen($zeile) > strlen("<label>")){
                $this->label = substr($zeile, strlen("<label>"));
            }
            if (strpos($zeile,"<description>") === 0 ){
                $this->description ="";
                while (($zeile = fgets($file)) !== false && (strpos($zeile, "<endDescription>") === false)) {                   
                    $this->description.=$zeile;
                }
                $this->description = trim($this->description);
            }
            if (strpos($zeile,"<hints>") === 0 ){
                $this->hints ="";
                while (($zeile = fgets($file)) !== false && (strpos($zeile, "<endHints>") === false)) {
                    $this->hints.=$zeile;
                }
                $this->hints = trim($this->hints);
            }
            
            if (strpos($zeile,"<decimals>") === 0 && strlen($zeile) > strlen("<decimals>")){
                $this->decimals = substr($zeile, strlen("<decimals>"));
            }
            if (strpos($zeile,"<filterType>") === 0 && strlen($zeile) > strlen("<filterType>")){
                $this->filterType = substr($zeile, strlen("<filterType>"));
            }
            if (strpos($zeile,"<filterRequired>") === 0 ) {
                $this->filterRequired = true;
                if (strlen($zeile) > strlen("<filterRequired>"))
                    $this->filterRequiredGroup = (int) substr($zeile, strlen("<filterRequired>"));
                    else $this->filterRequiredGroup = 1;
            }
            if (strpos($zeile,"<selectVar>") === 0 && strlen($zeile) > strlen("<selectVar>")){
                $this->selectVar = substr($zeile, strlen("<selectVar>"));
            }
            if (strpos($zeile,"<displayVar>") === 0 && strlen($zeile) > strlen("<displayVar>")){
                $this->displayVar = substr($zeile, strlen("<displayVar>"));
            }
            if ($queryVersion == '0.1'
                && ($this->displayVar == $this->selectVar
                    || $this->displayVar == $this->column_name
                    )) $this->displayVar = null;                                
            if (strpos($zeile,"<setVarname>") === 0 && strlen($zeile) > strlen("<setVarname>")){
                $this->qSetVarname = substr($zeile, strlen("<setVarname>"));
            }
            if (strpos($zeile,"<enterValue>") === 0 ) $this->enterValue = true;
            if (strpos($zeile,"<enterValueSize>") === 0 && strlen($zeile) > strlen("<enterValueSize>")){
                $this->enterValueSize = (int) substr($zeile, strlen("<enterValueSize>"));
            }
            
            if (strpos($zeile,"<selectFilterTable>") === 0 && strlen($zeile) > strlen("<selectFilterTable>")){
                $this->selectFilterTable = substr($zeile, strlen("<selectFilterTable>"));
            }
            if (strpos($zeile,"<selectFilterSort>") === 0 && strlen($zeile) > strlen("<selectFilterSort>")){
                $this->filterSort = 1 * substr($zeile, strlen("<selectFilterSort>"));
            }
            if (strpos($zeile,"<selectFilterTable2>") === 0 && strlen($zeile) > strlen("<selectFilterTable2>")){
                $this->selectFilterTable2 = substr($zeile, strlen("<selectFilterTable2>"));
            }
            if (strpos($zeile,"<selectFilterVar>") === 0 && strlen($zeile) > strlen("<selectFilterVar>")){
                $this->selectFilterVar = substr($zeile, strlen("<selectFilterVar>"));
            }
            if (strpos($zeile,"<selectFilterVar2>") === 0 && strlen($zeile) > strlen("<selectFilterVar2>")){
                $this->selectFilterVar2 = substr($zeile, strlen("<selectFilterVar2>"));
            }
            if (strpos($zeile,"<htmlData>") === 0 && strlen($zeile) > strlen("<htmlData>")){
                $this->htmlData = substr($zeile, strlen("<htmlData>"));
            }
            if (strpos($zeile,"<align>") === 0 && strlen($zeile) > strlen("<align>")){
                $this->align = substr($zeile, strlen("<align>"));
            }
            if (strpos($zeile,"<chartColor>") === 0 && strlen($zeile) > strlen("<chartColor>")){
                $this->chartColor = substr($zeile, strlen("<chartColor>"));
            }
            if (strpos($zeile,"<chartNegColor>") === 0 && strlen($zeile) > strlen("<chartNegColor>")){
                $this->chartNegColor = substr($zeile, strlen("<chartNegColor>"));
            }
            if (strpos($zeile,"<chartGroup>") === 0 && strlen($zeile) > strlen("<chartGroup>")){
                $this->chartGroup = substr($zeile, strlen("<chartGroup>"));
            }
            if (strpos($zeile,"<chartMin>") === 0 && strlen($zeile) > strlen("<chartMin>")){
                $this->chartMin = (float) substr($zeile, strlen("<chartMin>"));
            }
            if (strpos($zeile,"<chartMax>") === 0 && strlen($zeile) > strlen("<chartMax>")){
                $this->chartMax = (float) substr($zeile, strlen("<chartMax>"));
            }
            if (strpos($zeile,"<MetaSelections>") === 0 ){
                $this->selectOptions = array();
                $ms = 0;
                while (($zeile = fgets($file)) !== false && (strpos($zeile, "<EndMetaSelections>") === false)) {
                    $zeile = trim($zeile);
                    if (strpos($zeile,"<MetaSelectOption>") === 0){
                        $this->selectOptions[$ms][0] = substr($zeile, strlen("<MetaSelectOption>"));
                    }
                    elseif (strpos($zeile,"<MetaSelectLabel>") === 0){
                        $this->selectOptions[$ms][1] = substr($zeile, strlen("<MetaSelectLabel>"));
                        $ms ++;
                    }
                }
            }
            if (strpos($zeile,"<hidden>") === 0 ) $this->hidden = true;
            if (strpos($zeile,"<DimNr>") === 0 ) $this->dimNr = 1* substr($zeile, strlen("<dimNr>"));
            if (strpos($zeile,"<ReqDimNr>") === 0 ) $this->reqDim[] = 1* substr($zeile, strlen("<ReqDimNr>"));

            if (strpos($zeile,"<EndMetaRecord>") === 0 ) {
                return $this->table_name;
            }
		}
		return null;
	}     
// ENDE: importMetaData($file)
    
 	public function export()  {
 	    global $query;
 	    printf("<MetaRecord>\n");
 	    printf("<name>%s\n",$this->column_name);
 	    printf("<type>%d\n",$this->type);
 	    printf("<label>%s\n",wsrDecode($this->label));
 	    if (isValid($this->description)) {
 	        printf("<description>\n");
 	        printf("%s\n",wsrDecode($this->description));
 	        printf("<endDescription>\n");
 	    }
 	    if (isValid($this->hints)) {
 	        printf("<hints>\n");
 	        printf("%s\n",wsrDecode($this->hints));
 	        printf("<endHints>\n");
 	    }
 	    printf("<decimals>%d\n",$this->decimals);
 	    printf("<filterType>%d\n",$this->filterType);
 	    if ($this->filterRequired)printf("<filterRequired>%d\n",$this->filterRequiredGroup);
 	    if ($this->enterValue)printf("<enterValue>\n");
 	    if (isValid($this->enterValueSize)) printf("<enterValueSize>%s\n",$this->enterValueSize);
 	    if (isValid($this->qSetVarname))printf("<setVarname>%s\n",$this->qSetVarname);
 	    if (isValid($this->selectVar))printf("<selectVar>%s\n",str_replace(array("\r","\n","\t"),' ',$this->selectVar));
 	    if (isValid($this->displayVar))printf("<displayVar>%s\n",str_replace(array("\r","\n","\t"),' ',$this->displayVar));
 	    if (isValid($this->selectFilterTable))printf("<selectFilterTable>%s\n",$this->selectFilterTable);
 	    printf("<selectFilterSort>%d\n",$this->filterSort);
 	    if (isValid($this->selectFilterTable2))printf("<selectFilterTable2>%s\n",$this->selectFilterTable2);
 	    if (isValid($this->selectFilterVar))printf("<selectFilterVar>%s\n",$this->selectFilterVar);
 	    if (isValid($this->selectFilterVar2)) printf("<selectFilterVar2>%s\n",$this->selectFilterVar2);
 	    printf("<htmlData>%s\n",$this->htmlData);
 	    printf("<align>%s\n",$this->align);
 	    printf("<chartColor>%s\n",$this->chartColor);
 	    printf("<chartNegColor>%s\n",$this->chartNegColor);
 	    if (isValid($this->chartGroup)) printf("<chartGroup>%s\n",$this->chartGroup);
 	    if (isValid($this->chartMin)) printf("<chartMin>%s\n",$this->chartMin);
 	    if (isValid($this->chartMax)) printf("<chartMax>%s\n",$this->chartMax);
 	    
 	    if ((($this->filterType == self::SELECT
 	        || $this->filterType == self::VON_BIS_SELECT)
 	        && ! isValid($this->selectFilterTable))
 	        || $this->filterType == self::SONDERFILTERSEL
 	        || $this->filterType == self::HISTDEL
 	        || $this->filterType == self::WERTELISTE
 	        ){
 	        $so = $this->getSelectOptions();
 	        
 	        printf("<MetaSelections>%d\n",count($so));
 	        
 	        for ($ms = 0; $ms < count($so); $ms ++) {
 	            printf("<MetaSelectOption>%s\n",str_replace(array("\r","\n","\t"),' ',$this->selectOptions[$ms][0]));
 	            printf("<MetaSelectLabel>%s\n",str_replace(array("\r","\n","\t"),' ',$this->selectOptions[$ms][1])); 	            
 	        }
 	        printf("<EndMetaSelections>\n");
 	    }
 	    if ($this->hidden) printf("<hidden>\n"); 	    
 	    if ($this->dimNr > 0){
 	        printf("<DimNr>%d\n",$this->dimNr);
 	    }
 	    foreach ($this->reqDim as $rDim)
 	        if (array_key_exists($rDim, $query->dimensions))
 	            printf("<ReqDimNr>%d\n",$rDim);
		printf("<EndMetaRecord>\n");	
		return true;
	}
	// ENDE:  exportMetaData()

	public function saveForTable($table,$SpaltenTyp){
	    $myTable = $this->table_name;
	    if ($this->selectOptions == null) $this->getSelectOptions();
	    $this->table_name = $table;
	    $erg = $this->save($SpaltenTyp, 0);
	    $this->table_name = $myTable;
	    return $erg;
	}
	    
	
	public function save($SpaltenTyp, $SortNr){
        global $dbh, $query;
        require_once './phpsec/createDBH.inc';
	    if (!isValid($this->selectFilterTable) 
	        && $this->selectOptions == null
	        && $this->dimNr < 0){
	        $this->getSelectOptions();
	    }
	    static $insertStatement = null ;
	    if ($insertStatement == null){
	        $insertStatement = $dbh->prepare(
	            "replace into  queryColumnMetadata set "
	            ." table_name = :table_name"
	            .", column_name = :column_name"
	            .", SpaltenTyp = :SpaltenTyp"
	            .", SortNr = :SortNr"
	            .", column_label = :column_label"
	            .", column_desc = :column_desc"
	            .", column_hints = :column_hints"
	            .", qColType = :qColType"
                .", qDecimals = :qDecimals"
	            .", qSelectVar = :qSelectVar"
	            .", qDisplayVar = :qDisplayVar"
	            .", qFilterTyp = :qFilterTyp"
	            .", qSelectFilterTable = :qSelectFilterTable"
	            .", filterSort = :filterSort"
	            .", qSelectFilterVar = :qSelectFilterVar"
	            .", qSelectFilterTable2 = :qSelectFilterTable2"
	            .", qSelectFilterVar2 = :qSelectFilterVar2"
	            .", qHtmlData = :qHtmlData"
	            .", qAlign = :qAlign"
	            .", qChartGruppe = :qChartGruppe"
	            .", qChartColor = :qChartColor"
	            .", qChartNegColor = :qChartNegColor"
	            .", qChartMin = :qChartMin"
	            .", qChartMax = :qChartMax"
	            .", enterValue = :enterValue"
	            .", enterValueSize = :enterValueSize"
	            .", qSetVarname = :qSetVarname"
	            .", filterRequiredGroup = :filterRequiredGroup"
	            .", dimNr = :dimNr"
	            .", reqDimNr1 = :reqDimNr1"
	            .", reqDimNr2 = :reqDimNr2"
	            .", reqDimNr3 = :reqDimNr3"
	            .", reqDimNr4 = :reqDimNr4"
	            .", reqDimNr5 = :reqDimNr5"
	            .", hidden = :hidden"
	            );
	    }
	    if ($insertStatement === false){
	        http_response_code(425);
	        echo "FEHLER InsertStatement:\n";
	        var_dump($dbh->errorInfo());
	        return;
	        
	    }
	    $this->description = wsrDecode($this->description);
	    if (strlen($this->description) > 1020) $this->description = substr($this->description, 0,1020);
	    $this->hints = wsrDecode($this->hints);
	    if (strlen($this->hints) > 1020) $this->hints = substr($this->hints, 0,1020);
	    
	    if (!isValid($this->selectVar)) $this->selectVar = null;
	    if (!isValid($this->displayVar)) $this->displayVar = null;
	    if (!isValid($this->selectFilterTable)) $this->selectFilterTable = null;
	    if (!isValid($this->selectFilterTable2)) $this->selectFilterTable2 = null;
	    if (!isValid($this->selectFilterVar)) $this->selectFilterVar = null;
	    if (!isValid($this->selectFilterVar2)) $this->selectFilterVar2 = null;
	    
	    $a=array(
	        "table_name" => $this->table_name
	        , "column_name" => $this->column_name
	        , "SpaltenTyp" => $SpaltenTyp
	        , "SortNr" => $SortNr
	        , "column_label" => wsrDecode($this->label)
	        , "column_desc" => $this->description
	        , "column_hints" => $this->hints
	        , "qColType" => $this->type
	        , "qDecimals" => $this->decimals
	        , "qSelectVar" => str_replace("\r", '', $this->selectVar)
	        , "qDisplayVar" => str_replace("\r", '', $this->displayVar)
	        , "qFilterTyp" => 1* $this->filterType
	        , "qSelectFilterTable" => $this->selectFilterTable
	        , "filterSort" => $this->filterSort
	        , "qSelectFilterVar" => $this->selectFilterVar
	        , "qSelectFilterTable2" => $this->selectFilterTable2
	        , "qSelectFilterVar2" => $this->selectFilterVar2
	        , "qHtmlData" => $this->htmlData
	        , "qAlign" => $this->align
	        , "qChartGruppe" => $this->chartGroup
	        , "qChartColor" => $this->chartColor
	        , "qChartNegColor" => $this->chartNegColor
	        , "qChartMin" => $this->chartMin
	        , "qChartMax" => $this->chartMax
	        , "enterValue" => 0
	        , "enterValueSize" => $this->enterValueSize
	        , "qSetVarname" => $this->qSetVarname
	        , "filterRequiredGroup" => 0
	        , "dimNr" => 1*$this->dimNr
	        , "reqDimNr1" => -1
	        , "reqDimNr2" => -1
	        , "reqDimNr3" => -1
	        , "reqDimNr4" => -1
	        , "reqDimNr5" => -1
	        , "hidden" => 0
	    );
	    if ($this->enterValue) $a['enterValue'] = 1;
	    if ($this->filterRequired) $a['filterRequiredGroup'] = $this->filterRequiredGroup;
	    if ($SpaltenTyp >= 100) $a['column_name'] = ($SpaltenTyp - 100).":".$this->column_name;
	    if ($this->hidden) $a['hidden'] = 1;
	    $i = 1;
	    foreach ($this->reqDim as $rDim){
	        if ($i < 6 && array_key_exists($rDim, $query->dimensions)) {
	            $a['reqDimNr'.$i] = $rDim;
	            $i++;
	        }
	    }
	    $erg = $insertStatement->execute($a);
        
        if ($erg === false){
            http_response_code(425);
            var_dump($insertStatement->errorInfo());
            var_dump($insertStatement);
            var_dump($a);
            return;
            
        }
        $insertStatement->closeCursor(); // ???
        
        if ($this->filterType > 0 )
        if (($this->selectOptions != null && count($this->selectOptions) > 0)
            || ($this->getSelectOptions() != null && count($this->getSelectOptions()) > 0)){
            $this->saveSelections();
            
        }else{
            $this->selectOptions = null;
        }
// FEHLERBEHANDLUNG FEHLT        
        return $erg;
	}
	
	function saveSelections(){
	    global $dbh;
	    if ($this->selectOptions == null) return;
	    if (isValid($this->selectFilterTable)) return;	 
// Auch Auswahllisten aus Dimensionen werden in der Abfrage-Query übernommen	    if ($this->dimNr >= 0) return;  
	    $deleteStatement = $dbh->prepare(
	        "delete from  queryColumnMetadataSelectOptions where "
	        ." table_name = :table_name and column_name = :column_name");
	    $deleteStatement->execute(array(
	        ":table_name" => $this->table_name
	        , ":column_name" => $this->column_name));
	    
	    static $insertStatement = null;
	    if ($insertStatement == null){
	        $insertStatement = $dbh->prepare(
	            "insert into  queryColumnMetadataSelectOptions set "
	            ." table_name = :table_name"
	            .", column_name = :column_name"
	            .", SortNr = :SortNr"
	            .", value = :value"
	            .", label = :label"
	          );
	    }
	    foreach ($this->selectOptions as $SortNr => $eintrag){
	        if (strlen($eintrag[0]) > 250) {
	            echo 'FEHLER: Eintrag '.$eintrag[0]."\nwird abgeschnitten.\n";
	            $eintrag[0] = substr($eintrag[0], 0,250);
	        }
	        if (strlen($eintrag[1]) > 1024) {
	            echo 'FEHLER: Eintrag '.$eintrag[1]."\nwird abgeschnitten.\n";
	            $eintrag[1] = substr($eintrag[1], 0,1024);	        
	        }
	        if (false === $insertStatement->execute(array(
	            ":table_name" => $this->table_name
	            , ":column_name" => $this->column_name
	            , ":SortNr" => $SortNr
	            , ":value" => str_replace("\r", '',$eintrag[0])
	            , ":label" => str_replace("\r", '',$eintrag[1])
	            
	           ))
	            ){
	                if ($insertStatement->errorInfo()[0] == "23000"){
	                    echo 'Fehler: Doppelter Eintrag bei Filterauswahl '.$this->column_name.' Auswahl='.$eintrag[0].' Ausdruck='.$eintrag[1]."\n";
	                }else{
	                    echo "InsertFehler "; 
	                    var_dump($insertStatement->errorInfo());
	                }
	            }
//	            else echo "insert ".$this->column_name." $SortNr <br>\n";
	    }
	    $insertStatement->closeCursor();
 // FEHLERBEHANDLUNG FEHLT
 	    return true;
	}
	
    /**
     * Für die Spracherkennung muss die Query übergeben werden.
     * @param Object $z
     * @param Query $query
     */
	public function fillFrom ($z,$query){
	    global $queryVersion; //  $query ist bei verknüpften Quellen nicht die einzulesende Datenquelle;
	    $this->table_name = $z->table_name;
	    $this->column_name = $z->column_name;
	    if (substr($this->column_name, 1,1) == ":") $this->column_name = substr($this->column_name, 2);
	    elseif (substr($this->column_name, 2,1) == ":") $this->column_name = substr($this->column_name, 3);
	    if (isValid($z->column_label)) $this->label = qTranslate::translateQuery($query, $z->column_label);	    	    
	    else $this->label = $this->column_name;
	    
	    if (isValid($z-> column_desc)) $this->description = qTranslate::translateQuery($query, $z-> column_desc);
	    if (isValid($z-> column_hints)) $this->hints = qTranslate::translateQuery($query, $z-> column_hints);
	    if (isValid($z-> qColType)) $this->type = $z-> qColType;
	    if (isValid($z-> qDecimals)) $this->decimals = $z-> qDecimals;
	    if (isValid($z-> qFilterTyp)) $this->filterType = $z-> qFilterTyp;
	    
	    if ($z->dimNr > 0){
	        if (isValid($z-> qSelectVar)){
//    	        $this->selectVar = str_replace('dim.', 'd'.$z->dimNr.'.', qTranslate::translateQuery($query, 
//    	            str_replace('d'.$z->dimNr.'.', 'dim.', $z-> qSelectVar)));
	            if ($this->filterType == self::SONDERFILTER)
	                $this->selectVar = str_replace('dim.', 'd'.$z->dimNr.'.', qTranslate::translateQuery($query,
    	                str_replace('d'.$z->dimNr.'.', 'dim.', $z-> qSelectVar)));
    	            else  $this->selectVar = $z-> qSelectVar;
	        }
    	    else  $this->selectVar = $this-> column_name;
    	    if (isValid($z-> qDisplayVar)) 
    	        $this->displayVar = str_replace('dim.', 'd'.$z->dimNr.'.', qTranslate::translateQuery($query, 
    	            str_replace('d'.$z->dimNr.'.', 'dim.', $z-> qDisplayVar)));
    	    else $this->displayVar = null;
	    }else{
	        if (isValid($z-> qSelectVar)) {
	            if ($this->filterType == self::SONDERFILTER
	                || $this->type == self::WEBLINK)
	                $this->selectVar = qTranslate::translateQuery($query, $z-> qSelectVar);
	            else $this->selectVar = $z-> qSelectVar; //qTranslate::translateQuery($query, $z-> qSelectVar);
	        }
	        else  $this->selectVar = $this-> column_name;
	        
	        if (isValid($z-> qDisplayVar)) $this->displayVar = qTranslate::translateQuery($query, $z-> qDisplayVar);
	        else $this->displayVar = null;	        
	    }
	    if ($queryVersion == '0.1' 
	        && ($this->displayVar == $this->selectVar
	            || $this->displayVar == $this->column_name
	            )) $this->displayVar = null;
	    if (isValid($z-> qSelectFilterTable)) $this->selectFilterTable = $z-> qSelectFilterTable;
	    if (isValid($z-> qSelectFilterVar)) $this->selectFilterVar = $z-> qSelectFilterVar;
	    $this->filterSort = $z->filterSort;
	    if (isValid($z-> qSelectFilterTable2)) $this->selectFilterTable2 = $z-> qSelectFilterTable2;
	    if (isValid($z-> qSelectFilterVar2)) $this->selectFilterVar2 = $z-> qSelectFilterVar2;
	    if (isValid($z-> qHtmlData)) $this->htmlData = $z-> qHtmlData;
	    if (isValid($z-> qAlign)) $this->align = $z-> qAlign;
	    if (isValid($z-> qChartGruppe)) $this->chartGroup = $z-> qChartGruppe;
	    if (isValid($z-> qChartColor)) $this->chartColor = $z-> qChartColor;
	    if (isValid($z-> qChartNegColor)) $this->chartNegColor = $z-> qChartNegColor;
	    if (isValid($z-> qChartMin)) $this->chartMin = $z-> qChartMin;
	    if (isValid($z-> qChartMax)) $this->chartMax = $z-> qChartMax;
	    if (isValid($z-> qSetVarname)) $this->qSetVarname = $z-> qSetVarname;
	    if ($z->enterValue) $this->enterValue = true;
	    if ($z->enterValueSize > 1) $this->enterValueSize = $z->enterValueSize;
	    if ($z->filterRequiredGroup > 0) {
	        $this->filterRequired = true;
	        $this->filterRequiredGroup = $z->filterRequiredGroup;
	    }
	    $this->dimNr = $z->dimNr;  
	    if ($z->reqDimNr1 > 0 ) $this->reqDim[] = $z->reqDimNr1;
	    if ($z->reqDimNr2 > 0 ) $this->reqDim[] = $z->reqDimNr2;
	    if ($z->reqDimNr3 > 0 ) $this->reqDim[] = $z->reqDimNr3;
	    if ($z->reqDimNr4 > 0 ) $this->reqDim[] = $z->reqDimNr4;
	    if ($z->reqDimNr5 > 0 ) $this->reqDim[] = $z->reqDimNr5;
	    
	    if ($z->hidden) $this->hidden = true;
	    	            
	}
	public function setTable($table){
	    $this->table_name = $table;
	}
	public function setColumnName($column_name){
	    $this->column_name = $column_name;
	}
	
	/**
	 * Prüft, ob der Filter $repFilter in der Datenquelle zur Auswahlliste (ggf. Dimension)
	 * vor diesem Flter steht. 
	 * Wenn ja, muss der gesetzte $repFilter bei der Erstellung der Auswahlliste
	 * für diesen Filter berücksichtigt werden. 
	 * @param Query $selQuery
	 * @param int $selPos
	 * @param RepFilter $repFilter
	 * @return boolean
	 */
	function checkSelectFilter($selQuery, $selPos, $repFilter){
	    global $query;
	    if (false) $repFilter = new RepFilter('');
	    if ($repFilter->isWindowFilter()) return false;
  	    $filterMR = $query->getReportMetaRecord($repFilter->getName());
	    if (false) $filterMR = new MetaRecord();
	    if ($filterMR == null) {
	        error_log("FEHLER: MetarRecord nicht gefunden für ".$repFilter->getName());
	        return false;
	    }
	    if ($repFilter->queryOfSelect() != 'base' && $repFilter->queryOfSelect() != $this->queryOfSelect) return false;
	    if (! array_key_exists($filterMR->getName(), $selQuery->metaData)) return false;
	    if ($selQuery->metaData[$filterMR->getName()]->filterType <= 0 ) return false;
	    
	    $repPos = array_search($filterMR->getName() ,$selQuery->rowVars);
	    if ($repPos >= $selPos) return false;
	    
	    if ($repFilter->queryOfSelect() == 'base'){
	        if($selQuery->repFilter->get($filterMR->getName()) == null){
	            $selQuery->repFilter->put($repFilter);
	            return true;
	        }else return false;
	    }
	       	    
	    $filter = new RepFilter($filterMR->getName());
	    $filter->fs_von = $repFilter->fs_von;
	    $filter->fs_bis = $repFilter->fs_bis;
	    $filter->fs_list = $repFilter->fs_list;
	    $selQuery->repFilter->put($filter);
	    return true;
	}
	
	public function readSelectQuery(){
	    global $query, $testMode, $sourceSQL;
	    require_once './includes/loadClassDef.inc';
	    require_once './includes/createReport.inc';
	    $oldTestMode = false;
	    $oldSourceSql = false;
	    if (isset($testMode)) {
	        $oldTestMode = $testMode;
	        $testMode = false;
	    }
	    
	    if (isset($sourceSQL) && $sourceSQL){
	        $oldSourceSql = true;
	        $sourceSQL = false;
	    }
	    $selQuery = $this->createSelectQuery();
	    if (! $oldTestMode || $oldSourceSql){
	        $selPos = array_search($this->column_name ,$selQuery->rowVars);
	        if ($selPos === false){
	            echo 'FEHLER bei readSelectQuery: Zeile '.$this->column_name
	            .' in Datenquelle '.$selQuery->class_table.' nicht gefunden.';
	            error_log('FEHLER bei readSelectQuery: Zeile '.$this->column_name
	            .' in Datenquelle '.$selQuery->class_table.' nicht gefunden.');
	            return;
	        }
            if ($query->repFilter != null)
            for ($i=0; $i < $query->repFilter->size(); $i++){
                $repFilter = $query->repFilter->getPos($i);
                if ($this->checkSelectFilter($selQuery,$selPos,$repFilter)){
                     error_log('readSelectQuery - add RepFilter '.$repFilter->getName());
                }
            }
            if ($query->addFilter != null)
            for ($i=0; $i < $query->addFilter->size(); $i++){
                $repFilter = $query->addFilter->getPos($i);
                if ($this->checkSelectFilter($selQuery,$selPos,$repFilter)){
                     error_log('readSelectQuery - add AddFilter '.$repFilter->getName());
                }
            }
        }
        
        createSelectionTable($selQuery);
        if (false) $selQuery = new Query('');
//        error_log($this->queryOfSelect.': '.$this->column_name.' aus '.$selQuery->class_table);
        $sql = new SqlAbfrage('DWH');
        $oldQuery = $query;
        $query = $selQuery;
        if ($this->dimNr > 0 && strpos($this->column_name, ''.$this->dimNr) === 0)
            $mr = $selQuery->getReportMetaRecord(substr($this->column_name,strlen(''.$this->dimNr)));
        else $mr = $selQuery->getReportMetaRecord($this->column_name);
        if ($mr == null){
            http_response_code(425);
            echo WSRTranslate::translate('Fehler beim Erstellen der Auswahlliste für').' '.$this->column_name.".\n";
            echo WSRTranslate::translate('Keine Entsprechung in der Datenquelle').' '.$selQuery->class_table;
            die();
        }
        $sql->sql = 'select distinct '.$mr->getSelName().' as value, '
            .$mr->getDispName().' as label from  WSR_Auswahlliste';
        $query = $oldQuery;
        switch ($this->filterSort){
            default:
            case Sort::SORTINTERN: $sql->sql .= ' order by value';
            break;
            case Sort::SORTINTERNDESC: $sql->sql .= ' order by value desc';
            break;
            case Sort::SORTANZEIGE: $sql->sql .= ' order by label';
            break;
            case Sort::SORTANZEIGEDESC: $sql->sql .= ' order by label desc';
            break;                
        }
        $readStatement = $sql->execute( );
        $this->selectOptions = array();
        $sop = 0;
        while (($z = $readStatement->fetchObject()) != null){
            $se = array(0 => $z->value, 1 => $z->label);
            $this->selectOptions[$sop++] = $se;
        }
        $readStatement->closeCursor();
        $testMode = $oldTestMode;
        if (isset($oldSourceSQL) && $oldSourceSQL){
            $sourceSql = true;
        }
        return true;
	}
	
	public function readSelectTable(){
	    global $query, $testMode;
	    if (!isset($testMode)) $testMode = false;
	    $sql = new SqlAbfrage('DWH');
	    $sql->sql = "select distinct ".$this->selectVar." as value,"
	        .$this->getDisplayVar()." as label \nfrom $this->selectFilterTable ";
	    if ($query->queryType == Query::DIMENSION) $sql->sql .= " dim\n";
        else if ($this->dimNr < 0) $sql->sql .= " base\n";
        else  $sql->sql .= " d".$this->dimNr."\n";
                
        $selQuery = $query;
        if ($this->queryOfSelect != null){
            $prefix = '';
            if ($this->queryOfSelect != 'base') {
                $prefix = $this->queryOfSelect.':';
                $selQuery = $query->repTables->get($this->queryOfSelect)->query;
            }
    	    if (isValid($this->selectFilterVar && ! $testMode)){
               if ($query->repFilter->has($prefix.$this->selectFilterVar)){
    	            $filter = $query->repFilter->get($prefix.$this->selectFilterVar);
    	            if ($filter->isSet()){
    	                $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar, $filter, null);	                
    	            }
                }
    	        else if ($query->repFilter->has($this->selectFilterVar)){
    	            $filter = $query->repFilter->get($this->selectFilterVar);
    	            if ($filter->isSet()){
    	                $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar, $filter, null);
    	            }
    	        }
	            
                if (isValid($this->selectFilterVar2)){
                    if ($query->repFilter->has($prefix.$this->selectFilterVar2)){
                        $filter = $query->repFilter->get($prefix.$this->selectFilterVar2);
                        if ($filter->isSet()){
                            if (isValid($this->selectFilterTable2)){
                                $sql->addWhere($this->selectVar." in (select ".$this->selectVar.
                                       " from ".$this->selectFilterTable2);
                                $sql->klammerWeg();
                                if ($selQuery->queryType == Query::DIMENSION) $sql->sql .= " dim\n";
                                else if ($this->dimNr < 0) $sql->sql .= " base\n";
                                else  $sql->sql .= " d".$this->dimNr."\n";
                                
                                $sql->firstWhere = true;
                                $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar2, $filter, null);
                                $sql->sql.= '))';
    	                    }else{
    	                        $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar2, $filter, null);
    	                    }
    	                }
    	            }else if ($query->repFilter->has($this->selectFilterVar2)){
    	                $filter = $query->repFilter->get($this->selectFilterVar2);
    	                if ($filter->isSet()){
    	                    if (isValid($this->selectFilterTable2)){
    	                        $sql->addWhere($this->selectVar." in (select ".$this->selectVar.
    	                            " from ".$this->selectFilterTable2);
    	                        $sql->klammerWeg();
    	                        if ($selQuery->queryType == Query::DIMENSION) $sql->sql .= " dim\n";
    	                        else if ($this->dimNr < 0) $sql->sql .= " base\n";
    	                        else  $sql->sql .= " d".$this->dimNr."\n";
    	                        
    	                        $sql->firstWhere = true;
    	                        $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar2, $filter, null);
    	                        $sql->sql.= '))';
    	                    }else{
    	                        $sql->addFilterSQLForQuery($selQuery,$this->selectFilterVar2, $filter, null);
    	                    }
    	                }
    	            }	            
    	        }
    	    }
        } // if $this->queryOfSelect != null;
        
        switch ($this->filterSort){
            case Sort::SORTANZEIGE: $sql->sql .= "\n order by label";
            break;
            case Sort::SORTANZEIGEDESC: $sql->sql .= "\n order by label desc";
            break;
            case Sort::SORTINTERNDESC: $sql->sql .= "\n order by value desc";
            break;
            default: $sql->sql .= "\n order by value";
                
        }
	        
	    $readStatement = $sql->execute( );
	    if ($readStatement === false){
	        echo "FEHLER in readSelectTable:\n".$sql->errorInfo()[2];
	        error_log("FEHLER in readSelectTable bei:\n".$sql->sql);
	        die();
	    }
	    $this->selectOptions = array();
	    $sop = 0;
	    while (($z = $readStatement->fetchObject()) != null){
	        $se = array(0 => $z->value, 1 => $z->label);
	        $this->selectOptions[$sop++] = $se;
	    }
	    $readStatement->closeCursor();
	    
	    // FEHLERBEHANDLUNG FEHLT
	    return true;
	    
	}
	function readSelections(){
	    require_once './phpsec/createDBH.inc';
	    global $dbh, $query;
	    static $readStatement = null;
	    if ($readStatement == null){
	        $readStatement = $dbh->prepare(
	            "select * from queryColumnMetadataSelectOptions where "
	            ." table_name = :table_name"
	            ." and column_name = :column_name"
	            ." order by SortNr"
	            );
	    }
	    /**
	     * Wenn eine Dimension zugeordnet wird, müssen
	     * diese Optionen eigelesen werden, bevor der 
	     * Tabellen-Name auf die Basis-Tabelle gesetzt wird.
	     */
	    $selTable = $this->table_name;	    
	    $readStatement->execute(array(
	        ":table_name" => $selTable
	        , ":column_name" => $this->column_name
	    )
	        );
	    $this->selectOptions = array();
	    $sop = 0;
	    while (($z = $readStatement->fetchObject()) != null){
	        if ($this->filterType == self::SONDERFILTERSEL)
	            $value = qTranslate::translateQuery($query, $z->value);
	        else $value = $z->value;
	        $label = qTranslate::translateQuery($query, $z->label);
	        $se = array(0 => $value, 1 => $label);
	        $this->selectOptions[$sop++] = $se;
	    }
	    $readStatement->closeCursor();
	    // FEHLERBEHANDLUNG FEHLT
	    return true;
	}
	/*
	 * Im Sonderfilter dim. durch d$nr. ersetzten
	 */
	function setSelectOptionsDimnr($dimNr) {
	    foreach ($this->selectOptions as $n=>$option){
	        $option[1] = str_replace('dim.', 'd'.$dimNr.'.', $option[1]);
	        $this->selectOptions[$n] = $option;
	    }
	}
	
	public function getListOptions(){
	    return $this->selectOptions;
	}
	/**
	 * Auf selectOptions darf nur über Methoden zugegriffen werden,
	 * damit ggf. ein verzögertes Einlesen implementiert werden kann.
	 * @return array mit array([0]=Wert, [1]=Label oder sonderfilter-Code
	 */
	public function getSelectOptions(){
	    global $query;

	    if ($this->filterType == MetaRecord::TEXTMITLABEL) return null;
	    if ($this->filterType == MetaRecord::LIST) {
	        return null;
	    }
	    if ($this->filterType == MetaRecord::HISTDEL) {
	        if ($this->selectOptions == null){
	            $this->readSelections();
	            if (count($this->selectOptions) == 0){
	            $this->selectOptions = array(
	                array (0 => HistoryOptions::HSDELAKT, 1 => HistoryOptions::getKlartext(HistoryOptions::HSDELAKT)),
                    array (0 => HistoryOptions::HSDELBIS, 1 => HistoryOptions::getKlartext(HistoryOptions::HSDELBIS)),
	                array (0 => HistoryOptions::HSDELAKTJB, 1 => HistoryOptions::getKlartext(HistoryOptions::HSDELAKTJB)),
	                array (0 => HistoryOptions::HSDELBISJB, 1 => HistoryOptions::getKlartext(HistoryOptions::HSDELBISJB))
	            );
	            }
	        }
	    }
	    else
	        if (isValid($this->selectFilterTable)) {
	            if (substr($this->selectFilterTable, 0,1) == '#') $this->readSelectQuery();
	            else $this->readSelectTable();
	        }else if ($this->selectOptions == null
	            &&    
	                ($this->filterType == self::SELECT
	            ||  $this->filterType == self::VON_BIS_SELECT
	            ||  $this->filterType == self::SONDERFILTERSEL
	            ||  $this->filterType == self::WERTELISTE
	                    )) {
	                $this->readSelections();
	            }	            
	    
	    return $this->selectOptions;
	}

	public function hasMetadataSelectOptions(){
	    if ($this->filterType == MetaRecord::TEXTMITLABEL) return false;
	    if ($this->filterType == MetaRecord::LIST) return false;
        if (isValid($this->selectFilterTable)) return false;
	    if ( $this->filterType == self::SELECT
	            ||  $this->filterType == self::VON_BIS_SELECT
	            ||  $this->filterType == self::SONDERFILTERSEL
	            ||  $this->filterType == self::WERTELISTE
	        ) return true;	    	    
	    return false;
	}
	
	
	public function setSelectOptions($options){
	    $this->selectOptions = $options;
	    return;
	}

	public function findSelectOption($opt){
//	    if (! isValid($this->displayVar)) return ($opt);
	    if ($this->selectOptions == null && isValid($this->selectFilterTable)){
	        return $this->findSelectLabel($opt);
	    }
	    $options = $this->getSelectOptions();
	    if ($options == null)  return ($opt);
	    for ($i=0; $i < count($options); $i++)
	        if ($options[$i][0] == $opt) return $options[$i][1];
	    return "? $opt"; //null;
	}

	function createSelectQuery(){
	    global $query;
	    if (substr($this->selectFilterTable, 0,1) != '#'){
	        echo 'FEHLER bei createSelectQuery für '.$this->column_name.' Filter-Table ist '.$this->selectFilterTable;
	        die();
	    }
        require_once './includes/loadClassDef.inc';
        require_once './includes/createReport.inc';
        $selQueryTable = $this->selectFilterTable;
        $table = 'base';
        if ($selQueryTable == '#') {
            error_log('createSelectQuery() '.$this->column_name.' dimNr='.$this->dimNr);
            if ($this->dimNr >= 0){
                if (!isValid($this->queryOfSelect) || $this->queryOfSelect == 'base') $dim = $query->dimensions[$this->dimNr];
                else {
                    $dimQuery = $query->repTables->get($this->queryOfSelect)->query;
                    $dim = $dimQuery->dimensions[$this->dimNr];
                }
                if ($dim == null){
                    echo '<br>FEHLER bei createSelectQuery für '.$this->queryOfSelect.'-'.$this->column_name;
                    echo " DimNr= $this->dimNr <pre>";                    
                    die();
                }
                $selQueryTable = $dim->dimName;                
            }
//            else if ($query->queryType == Query::DIMENSION) $selQueryTable = $query->class_table;
            else $selQueryTable = $query->class_table;
        }
        else {
            $selQueryTable = substr($selQueryTable,1);
        }
        
        if ($selQueryTable == $query->class_table){
            $selQuery = $query->cloneSimpleQuery();
            // error_log('Clone Simple Query');
            
        }else{
            $selQuery = new Query($selQueryTable);
            loadClassDef($selQuery);
        }
        $repRow = new RepRow($this->column_name);
        $selQuery->newReport();
        $selQuery->repRows->put($repRow);
        if (! isValid($this->queryOfSelect)) $this->queryOfSelect = 'base';
        if ($selQuery->histType > 0 && $selQuery->historyOption <= 0){
            if ($this->queryOfSelect == 'base') {
                $selQuery->historyOption = $query->historyOption;
                $selQuery->historyDate = $query->historyDate;
            }
            else {
                $tJoin = $query->repTables->get($this->queryOfSelect);
                if (false) $tJoin = new JoinedTable('');
                if ($tJoin == null){
                    echo 'FEHLER in MetaRecord->createSelectQuery() für '.$this->column_name.': Join '.$this->queryOfSelect.' nicht gefunden.';
                    error_log('FEHLER in MetaRecord->createSelectQuery() für '.$this->column_name.': Join '.$this->queryOfSelect.' nicht gefunden.');
                    die();
                }
                if ($tJoin->dimHistoryOption[0] == 0 
                    || $tJoin->dimHistoryOption[0] == HistoryOptions::HSOHNEAUSWAHL
                    || $tJoin->dimHistoryOption[0] == HistoryOptions::HSSTICHTAGBASE
                    ) {
                        $selQuery->historyOption = $query->historyOption;
                        $selQuery->historyDate = $query->historyDate;
                    }
                else {
                    $selQuery->historyOption = $tJoin->dimHistoryOption[0];
                    $selQuery->historyDate = $tJoin->dimHistoryDate[0];
                }
            }
        }
        error_log('Query-Type= '.$selQuery->queryType);
	   return $selQuery;        
	}
	
	public function findSelectLabel($opt){
	    global $query;
	    if (substr($this->selectFilterTable, 0,1) == '#'){
	        $selQuery = $this->createSelectQuery();
	        $repFilter = new RepFilter($this->column_name);
	        $repFilter->fs_von = $opt;
	        $repFilter->fs_bis = $opt;
	        
	        $selQuery->repFilter->put($repFilter);
	        createSelectionTable($selQuery);
	        
	        $sql = new SqlAbfrage('DWH');
	        $oldQuery = $query;
	        $query = $selQuery;
	        $mr = $selQuery->getReportMetaRecord($this->column_name);	        
	        $sql->sql = 'select distinct  '.$mr->getDispName()
	        .' as label from  WSR_Auswahlliste';
	        $query = $oldQuery;
	    }else{
	        if (! isValid($this->displayVar)) return ($opt);
	        $sql = new SqlAbfrage('DWH');
    	    $sql->sql = "select ".$this->displayVar
    	        ." as label \nfrom $this->selectFilterTable ";
    	        if ($query->queryType == Query::DIMENSION) $sql->sql .= " dim\n";
    	        else if ($this->dimNr < 0) $sql->sql .= " base\n";
    	        else  $sql->sql .= " d".$this->dimNr."\n";
    	        
    	    $sql->addSel($this->selectVar.' = ', $opt);
	    }
        $stmt = $sql->execute();
        if ($stmt !== false && $res = $stmt->fetchObject()) {
            $stmt->closeCursor();
            return $res->label;
        }        
	    return 'Label für '.$this->label.': '.$opt.' nicht gefunden';
	}
	
	function getSelectVar(){
	   $v = $this->selectVar;
	   if (isValid($v)) return $v;
	   return $this->column_name;    
	}
	function getDisplayVar(){
	    $v = $this->displayVar;
	    if (isValid($v)) return $v;
	    $v = $this->selectVar;
	    if (isValid($v)) return $v;
	    return $this->column_name;
	}
	public function getName() {
	    return $this->column_name;
	}
	public function getSelName() {
	    global $query;
	    if (in_array($this->column_name, $query->rowVars))
	           return 'S_'.$this->column_name;
	    return $this->column_name;
	}
	public function hasDispVar(){
	    if (isValid($this->displayVar)
	        && $this->displayVar != $this->selectVar) return true;
	    return false;
	}
	public function getDispName() {
	    global $query;
	    if ($this->hasDispVar()) return 'D_'.$this->column_name;
	    if (in_array($this->column_name, $query->rowVars))
	        return 'S_'.$this->column_name;
	   return $this->column_name;
	}
	
	
	/**
	 * Filter referenziert aud Basistabelle 
	 * und darf deshalb nur in Verknüfungen angewandt werden
	 * @return boolean Filter referenziert auf Basis-Tabelle
	 */
	public function baseFilter(){
	    if (stripos($this->selectVar, 'from WSR_base') === false) return false;
	    return true;
	}
} // ENDE MetaRecord


class AddedQuery{
    function __construct($table){
        $this->class_table = $table;
    }
    public $class_table = null;
    private $query = null;
    public $calcVars = array(); // Array mit Variablen-namen
  
    public function getQuery(){
        global $query;
        if ($this->query == null){
            if ($query->class_table == $this->class_table){
                $this->query = $query->cloneSimpleQuery(); // ohne AddQuery-Einträge clonen
            }else{
                $this->query = new Query($this->class_table);
                require_once "./includes/loadClassDef.inc";
                loadSimpleClassDef($this->query);
            }
        }
        return $this->query;
    }
    public function getName() {
        return $this->class_table;
    }
    public function clearActiveQuery(){
        global $query;
        if ($this->class_table == $query->class_table){
// Das führt zum Fehler bei Berechnungen nach dem Join    $this->query = $query->cloneSimpleQuery();
        }
    }
}

class JoinedTable{
    const JOINSPALTEN = 1;
    const JOINWERTE = 2;
    const JOINOUTERWERTE = 3;    
    
    public $prefix = null; //Text für Ausgabe im Output und Anzeige des Labels
    public $kennzeichen = null; // Buchstabe für SQL-Abfrage
    /**
     * @var Query
     */
    public $query = null;
    public $calcvars = null;
    public $joinType = self::JOINWERTE;
    public $dimHistoryOption = array();
    public $dimHistoryDate = array();
    public $dimHistoryQueryDate = array();
    
    function __construct($addQuery){
        global $query;
        $this->query = $addQuery->getQuery()->cloneSimpleQuery();
//        error_log("JoinedTable:".print_r($this->query,true));
        
        $this->calcvars = $addQuery->calcVars;
        $this->prefix = $this->query->outputTitlePrefix;
        $this->kennzeichen = $query->getKennzeichen();
        
        $this->dimHistoryOption[0] = $this->query->historyOption;
        $this->dimHistoryDate[0] = $this->query->historyDate;
        
        foreach ($this->query->dimensions as $dim){
            $this->dimHistoryOption[$dim->dimNr] = $dim->historyOption;
            $this->dimHistoryDate[$dim->dimNr] = $dim->historyDate;
        }
    }
    
//    const zeichen = array("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z" );
    const klartext = array(
        self::JOINSPALTEN => "Spalten",
        self::JOINWERTE => "Werte",
        self::JOINOUTERWERTE => "Werte mit Zeilen"  
    );
    public static function getKlartextType($option){
        if (isValid($option) && $option > 0) return WSRTranslate::translate(self::klartext[$option]);
        return '';
    }
    
    public function getKlartextHistoryOption() {
        return HistoryOptions::getKlartext($this->historyOption);
    }
    
    public function getName() {
        return $this->kennzeichen;
    }
    
    public $deleteStatement = 'delete from reportJoinedTables where reportID = :reportID';
    public static $readStatement = 'select * from reportJoinedTables where reportID = :reportID order by kennzeichen';
    public $insertStatement = <<<EOS
 insert into reportJoinedTables set reportID = :reportID
   , kennzeichen = :kennzeichen
   , table_name = :table_name
    , prefix = :prefix
    , rJoinType = :joinType
    , historyOptions = :historyOptions
EOS;
    
    /**
     * 
     * @param Query $query
     * @return string[]
     */
    public function getInsertArray($query){        
        $array = array();
        $array ['kennzeichen'] = $this->kennzeichen;
        $array['table_name'] = $this->query->class_table;
        $array ['prefix'] = $this->prefix;
        $array ['joinType'] = $this->joinType;
        $historyOptions = '';
        foreach($this->dimHistoryOption as $dimNr => $opt){
            if (isValid($opt) &&  $query->reportHasDim($this->kennzeichen, $dimNr)){
                $historyOptions.= $dimNr.'.'.$opt;
                if (isValid($this->dimHistoryDate[$dimNr])) 
                    $historyOptions.='.'.$this->dimHistoryDate[$dimNr];
                $historyOptions .= ':';
            }
        }
        $array ['historyOptions'] = $historyOptions;
        return $array;
    }
    
    public function read($z){
        $this->kennzeichen = $z->kennzeichen;
        $this->prefix = $z->prefix;
        $this->joinType = $z->rJoinType;
        if ($this->query->class_table != $z->table_name){
            echo "\nFEHLER bei read Joined Table "
                .$this->query->class_table.' passt nicht zu '.$z->table_name;
            die();
        }        
        $histEntries = explode(':',$z->historyOptions);
//        error_log('JT Start '.$z->prefix.':'.print_r($this->dimHistoryOption,true));
//        error_log('JT '.$z->prefix.':'.print_r($histEntries,true));
        foreach ($histEntries as $hEntry){
            $hEntryArray = explode('.',$hEntry);
            if (isValid($hEntryArray[0])){
                if (isValid($hEntryArray[1])){
                    $this->dimHistoryOption[(int)$hEntryArray[0]] = (int) $hEntryArray[1];
//                    error_log ('ergibt '.$hEntryArray[0].' = '.$hEntryArray[1]);
                    if (array_key_exists(2, $hEntryArray)){
                        $this->dimHistoryDate[(int)$hEntryArray[0]] = $hEntryArray[2];
                    }
                }
            }
        }
    }
    
}

class Vector{
    public $var = array();
    public $varPos = array();
    
    public function put($o){
        if (array_key_exists($o->getName(), $this->var)){
            $this->var[$o->getName()] = $o;
            return;
        }
        $this->var[$o->getName()] = $o;
        $this->varPos[] = $o->getName();
    }

    public function putPos($o,$p){
        if (array_key_exists($o->getName(), $this->var)){
            $this->var[$o->getName()] = $o;
            $this->varPos[$p] = $o->getName();
            echo "putPos verschiebe $o->getName() nach $p <br>";
            return;
        }
        $this->var[$o->getName()] = $o;
        $this->varPos[$p] = $o->getName();
    }
    
    
    
    public function get($n){
        if (array_key_exists($n, $this->var)){
            return $this->var[$n];
        }
        return null;
    }
    
    /**
     * Nach namen suchen
     * @param String $n
     * @return boolean
     */
    public function has($n){
        return array_key_exists($n, $this->var);
    }

    /**
     * Position vom Namen suchen
     * @param String $n
     * @return number|boolean
     */
    public function findPos($n){
        for ($p = 0; $p < count($this->varPos); $p ++){
            if ($n == $this->varPos[$p]) return $p;
        }
        return false;    
    }
    

    public function getPos($p){
        if ($p >= 0 && $p < count($this->varPos)){
            $n = $this->varPos[$p];
            if (array_key_exists($n, $this->var)){
                return $this->var[$n];
            }
        }
        return null;
    }
    public function size(){
        return count($this->varPos);
    }
    
    public function rename($pos, $to){
        if (! array_key_exists($pos, $this->varPos)) return;
        if ( array_key_exists($to, $this->var)) return;
        $from = $this->varPos[$pos];
        $this->var[$to] = $this->var[$from];
        $this->var[$to]->rename($to);
        unset ($this->var[$from]);
        $this->varPos[$pos] = $to;
    }
    
    public function remove($n){
        if (! array_key_exists($n, $this->var)) return;
        unset ($this->var[$n]);
        $found = false;
        for ($i = 0; $i < count($this->varPos) -1; $i++){
            if (strcmp($n, $this->varPos[$i]) == 0) $found = true;
            if ($found) $this->varPos[$i] = $this->varPos[$i+1];
        }            
        if ($found) unset($this->varPos[count($this->varPos) -1]);
        $this->varPos = array_values($this->varPos);
    }
    public function removePos($p){
        if ($p < 0 || $p >= count($this->varPos)) return;
        $n = $this->varPos[$p];
        if (! array_key_exists($n, $this->var)) return;
        unset ($this->var[$n]);
        for ($i = $p; $i < count($this->varPos) -1; $i++){
            $this->varPos[$i] = $this->varPos[$i+1];
        }
        unset($this->varPos[count($this->varPos) -1]);
        $this->varPos = array_values($this->varPos);
    }
    
    public function upPos($p){
        if ($p < 1 || $p >= count($this->varPos)) return;
        $on = $this->varPos[$p];
        $this->varPos[$p] = $this->varPos[$p-1] ;
        $this->varPos[$p-1] = $on;
    }
    public function downPos($p){
        if ($p < 0 || $p >= count($this->varPos) -1) return;
        $on = $this->varPos[$p];
        $this->varPos[$p] = $this->varPos[$p+1] ;
        $this->varPos[$p+1] = $on;
    }
    public function movePos($von,$nach){
        $on = $this->varPos[$von];
        if ($von < 0 || $von > $this->size()+1) {
            http_response_code(425);
            echo "FEHLER von = {$von}";
            die();
        }
        if ($nach < 0 || $nach > $this->size()+1) {
            http_response_code(425);
            echo "FEHLER nach = {$nach}";
            die();
        }
        if ($von > $nach){
            for ($i = $von; $i > $nach; $i--) $this->varPos[$i] = $this->varPos[$i -1];
        }else for ($i = $von; $i < $nach; $i++) $this->varPos[$i] = $this->varPos[$i +1];
        $this->varPos[$nach] = $on;        
    }
    
    public function save($reportID){
        global $dbh;
        if (count($this->varPos) < 1) return;
        $statement = $this->var[$this->varPos[0]]->deleteStatement;
        $stmt = $dbh->prepare($statement);
        if ($stmt === false){
            http_response_code(425);
            echo "\nFehler bei prepare von \n".$statement;
            echo "\n".$dbh->errorInfo()[2];
            error_log("Fehler bei prepare von \n".$statement);
            error_log($dbh->errorInfo()[2]);
            die();
        }
        if (false === $stmt->execute(array('reportID' => $reportID))){
            http_response_code(425);
            echo "\nFehler bei execute von \n".$statement;
            echo "\n".$stmt->errorInfo()[2];
            error_log("Fehler bei execute von \n".$statement);
            error_log($stmt->errorInfo()[2]);
            die();            
        }
        $statement = $this->var[$this->varPos[0]]->insertStatement;
        $stmt = $dbh->prepare($statement);
        if ($stmt === false){
            http_response_code(425);
            echo "\nFehler bei prepare von \n".$statement;
            echo "\n".$dbh->errorInfo()[2];
            error_log("Fehler bei prepare von \n".$statement);
            error_log($dbh->errorInfo()[2]);
            die();
        }
        
        for ($i=0; $i < count($this->varPos); $i++){
            $pars = $this->var[$this->varPos[$i]]->getInsertArray();
            $pars['reportID'] = $reportID;
            $pars['SortNr'] = $i;
            if (false === $stmt->execute($pars)){
                http_response_code(425);
                echo "\nFehler bei execute von \n".$statement;
                echo "\n".$stmt->errorInfo()[2];
                error_log("Fehler bei execute von \n".$statement);
                error_log($stmt->errorInfo()[2]);
                error_log(print_r($pars,true));
                error_log(print_r($stmt,true));
                
                die();
            }
        }
    }
}


class Query{
    const DATEN = 0;
    const EINGABE = 1;
    const EINGABE_MIT_SUMMEN = 2;
    const EINGABE_OHNE_SUMMEN = 3;
    const DIMENSION = 11;
    const LISTE = 12;
    
    const klartext = array(
        self::DATEN => "Datenabfrage",
        self::EINGABE => "Erfassung (Standard)",
        self::EINGABE_MIT_SUMMEN => "Erfassung manuell auf allen Ebenen",
        self::EINGABE_OHNE_SUMMEN => "Erfassung ohne Eingabe von Summen",
        self::DIMENSION => "Dimension",
        self::LISTE => "Auswahlliste"
    );
    public static function getTypeText($option){
        if (isValid($option)) return WSRTranslate::translate(self::klartext[$option]);
        return WSRTranslate::translate(self::klartext[0]);
    }

    const KEINE = 0;
    const VONBIS = 1;
    const STICHTAG = 2;
    const STICHTAGE = 3;
    const klartextHist = array(
        self::KEINE => "keine Historisierung",
        self::VONBIS => "von-bis-Intervalle",
        self::STICHTAG => "ein Stichtag je Zeitraum",
        self::STICHTAGE => "mehrere Stichtage (je Zeitraum)"
    );
    public static function getHistText($option){
        if (isValid($option)) return WSRTranslate::translate(self::klartextHist[$option]);
        return WSRTranslate::translate(self::klartextHist[0]);
    }
    
    const FOLDEROFFEN = 1;
    const FOLDERZU = 2;
    const FOLDERZU1 = 3;
    const FOLDERZU2 = 4;
    const FOLDERERSTEZEILE = 5;
    const klarTextFolder = array(
        self::KEINE => "-- nicht aktiv --",
        self::FOLDEROFFEN => "Start aufgeklappt",
        self::FOLDERZU => "Start eingeklappt",
        self::FOLDERZU1 => "Start 1. Stufe auf",
        self::FOLDERZU2 => "Start 2 Stufen auf",
        self::FOLDERERSTEZEILE => "Auf erste Zeile"
    );
    
    public static function getFolderText($option){
        if (isValid($option)) return WSRTranslate::translate(self::klarTextFolder[$option]);
        return WSRTranslate::translate(self::klarTextFolder[0]);
    }
    public function getKlartextHistoryOption() {
        return HistoryOptions::getKlartext($this->historyOption);
    }
    
    
    
    
    
    function __construct($table){
        $this->class_table = $table;
        $this->object_id = time();
        if (array_key_exists('Sprache', $_SESSION)) {
            error_log('Construct Query Sprache = '.print_r($_SESSION['Sprache'],true));
            $this->current = 'de'; // explode(' ',$_SESSION['Sprache'])[0];
        }
//        $this->loadClassDef();  
        if ($table == null){
            $this->class_table = "Dummy-Tabelle";
            $this->addQueries = new Vector();
            $this->metaData = array();
            $this->defaultFilter = array();
            $this->outputTitlePrefix = 'Dummy';
        }
    }    
    public $object_id = null; // Für Query-Editor zur Prüfung der aktiven Query
    // Query-Attribute
    /**
     * Sprache, in der die Datenuelle definiert und gespeichert ist.
     * @var string
     */
    public $qLanguage = 'de';
    /**
     * Sprachen, zu denen diese Datenuelle Übersetzungen hat.
     * @var array
     */
    public $qTranslations = array();
    /**
     * Sprache, in der der Report definiert und gespeichert ist.
     * @var string
     */
    public $rLanguage = 'de';
    /**
     * Sprachen, zu denen dieser Report Übersetzungen hat.
     * @var array
     */
    public $rTranslations = array();
    /**
     * Sprache, in der die Datenquelle aktuell eingelesen wurde.
     * Die Übersetzung erfolgt beim Einlesen
     * @var string
     */
    public $currentLanguage = 'de';
    /**
     * Sprache, in der der Report aktuell eingelesen wurde.
     * Die Übersetzung erfolgt beim Einlesen
     * @var string
     */
    public $currentRLanguage = 'de';
    
    const SPRACHEN = array(
        'de' => 'Deutsch',
        'en' => 'Englisch',
        'fr' => 'Französisch'
    );
    
    public $class_table=null;
    public $dwh='Standard';
    public $phys_table=null;
    public $table_description=null;
    public $table_hints=null;
    public $queryType = Query::DATEN;
    public $histType = Query::KEINE;
    public $historyOption = 0;
    public $historyDate = null; // wird im Report gesetzt    
    public $historyQueryDate = null; // Klausel für Stichtag bei von-bis-abfragen
    
    public $dimKey = null;
    public $histAb = null;
    public $histBis = null;
    public $histStichtag = null;

    /**
     * Zeilen der Datenquelle
     */
    public $rowVars = null;
    /**
     * Werte der Datenquelle
     */
    public $valueVars = null;
    public $filterVars = null;
    public $metaData = array();
    public $addQueries = null;
    public $dimensions = null;
    public $defaultFilter = array();
    public $werteListe = array();
    
    public $selectedDimension = 0;
    public $hatWerteAuswahl = false;
    
    public $outputTitlePrefix = 'WS-Reporter:';
    public $defaultTitle = 'WS Report';
    public $reportCSS = './WSReport.css';
    public $reportHoverMode = 0;
    
    // Muster
    public $muster_tabelle="WS-Report-Muster";
    
    // Report-Attribute
    public $reportID = 0;
    public $title = "";
    public $chart_size = "3em";
    public $display_charts = ChartOptions::KEINESUMMEN;
    public $kommentar = null;
    public $report_hints = null;
    public $fix_labelColumns = false;
    public $values_in_columns = false;
    public $dropSummen = false;
    public $folderMode = Query::KEINE;
    public $valueColumnFolderMode = '';

    public $repWerteBasis = null;
    public $repWerteAuswahl = null;
    public $repWerteAuswahlTitel = 0;
    public $repWerteAuswahlLabel = false;
    public $repWerteAuswahlSelector = false;
    
    public $repRows = null;
    public $repValues = null;
    public $repColumns = null;
    public $repFilter = null;
    public $addFilter = null;
    public $repTables = null;
    public $repReports = null;

    public $repRowsMuster = null;
    public $repValuesMuster = null;
    public $repColumnsMuster = null;
    public $repFilterMuster = null;
    public $addFilterMuster = null;
    
    
    public $loadSaveAddFilter = false;
    public $repGeldFaktor = 1;

    // aus Aufruf
    public $spreadSheet = false;
    public $sort_mode = SortOptions::NOSORTVAL;
    public $sort_value = null;
    public $sort_colGroup = null;
    public $sort_colValue = null;
    public $sort_direction = Sort::NONE;
    
    // Runtime-Werte    
    public $_URL = "";
    public $titleRowSpan = 1;
    public $testMode = false;
    public $titleModified = false;
    public $lastRowVals = null;
    public $displayVars = null;
    public $lastRowSpan = null;
    
    public $query_where_klausel = "";
    
    public $showMuster = 0;
    public $muster = "";
    public $musterListe = array();
    public $musterZiel = 0;
    
    // für Charts
    public $hasCharts = false;
    public $chartValues = array();
    public $chartClors = array();
    
    // für SpaltenAusgabe - repColumns
    public $column_count = 0;
    public $columnDataPos = null;
    public $columnData = null;
    public $columnHeader = null;
    
    // für Query-ClassEditor
    public $qEditSubject = 0;
    public $qEditVar = null;

    // Status nach Import
    public $queryImported = false;
    public $reportImported = false;
    
    public $description_imported = null; // importierte table_description bzw. kommentar in Fremdsprache
    public $hints_imported = null;       // importierte table_hints bzw. report_hints in Fremdsprache
    
    public $importUserCreated = null;
    public $importTimeCreated = null;
    public $importUserReleased = null;
    public $importTimeReleased = null;
    
    public function getKennzeichen(){
        static $zeichen = array("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z" );
        if ($this->repTables == null) $this->repTables = new Vector();
        for ($i=0; $i < count($zeichen); $i++){
            if (! $this->repTables->has($zeichen[$i])) return $zeichen[$i];
        }
        return null;
    }
        
    public function getMetaRecord($bereichNr,$pos){
        $ber = null; // array
        switch ($bereichNr){
            case 1: $ber = $this->rowVars;
            break;
            case 2: $ber = $this->valueVars;
            break;
            case 3: $ber = $this->filterVars;
            break;
        }
        if ($ber != null){
            $metaRecord = $this->metaData[$ber[$pos]];
            if ($metaRecord != null) {
                return $metaRecord;
            }
        }
        echo  "FEHLER getMetaRecord: Pos. $pos in Bereich $bereichNr <br>\n";
        return null;
    }
    
    /**
     * liefert den MetaRecord aus der verknüpften Tabelle
     * bei Werten, Spalten und Filter.
     * Bei Zeilen sind keine verknüpften Namen erlaubt!
     * 
     * @return MetaRecord 
     */
    public function getReportMetaRecord($name){
        global $sql;
        $name = explode('|', $name)[0];
        $sname = $name;
        if (substr($name,1,1) == ':' || substr($name,1,1) == '='){
            $jt = $this->repTables->get(substr($name,0,1));
            if ($jt == null) {
                if ($sql != null) $sql->sql.="\n-- JT ist NULL $sname (in $this->class_table)\n";
                return null;
            }
            if ($jt->query == null) {
                if ($sql != null) $sql->sql.="\n-- JT-Query ist NULL $sname (in $this->class_table)\n";
                return null;
            }
            if (!array_key_exists(substr($name,2), $jt->query->metaData)){
                if ($sql != null) $sql->sql.="\n-- Keine Metadaten für $sname (in $this->class_table)\n";
                return null;
            }
            $mr = $jt->query->metaData[substr($name,2)];
            if ($mr != null) $mr->queryOfSelect = substr($name,0,1);
            return $mr;
        }
        if (substr($name,0,5) == 'base.') $name = substr($name, 5);
        if (substr($name,0,5) == 'join.') {
            echo "getReportMetaRecord(join...) darf nicht vorkommen, weil die Datenquelle dann unbekannt ist";
            die();
            $name = substr($name, 5);
        }
        if (!array_key_exists($name, $this->metaData)) {
//            echo "Keine Metadaten für $sname in $this->class_table<br>";     
            if ($sql != null) $sql->sql.="\n-- Keine Metadaten für $sname (in $this->class_table)\n";
            return null;
        }
        $mr = $this->metaData[$name];
        if ($mr != null) $mr->queryOfSelect = 'base';
        return $mr;
    }
    
    public function listMuster(){
        $returnMuster = array();       
        if ($this->showMuster == 4)
            foreach ($this->musterListe as $name => $label){
                if (! $this->addQueries->has($name))
                    $returnMuster[$name] = $label;
        }
        else
        foreach ($this->musterListe as $name => $label){
            if (! array_key_exists($name, $this->metaData))
                $returnMuster[$name] = $label;
        }
        return $returnMuster;
    }
    public function getModeSortSelected ($mode) {
        if ($mode == $this->sort_mode) return " selected ";
        return "";
    }
    public function getFixLabelColumnsChecked() {
        if ($this->fix_labelColumns) return " checked ";
        return "";
    }
    
    public function newReport(){
        $this->title = $this->defaultTitle;
        $this->repRows= new Vector();
        $this->repValues= new Vector();
        $this->repColumns= new Vector();
        $this->repFilter= new Vector();
        $this->addFilter= new Vector();
        $this->repTables= new Vector();
        $this->repReports= new Vector();
        $this->sort_mode = SortOptions::NOSORTVAL;
        $this->folderMode = Query::KEINE;
        $this->fix_labelColumns = true;
        $this->values_in_columns = false;
        $this->chart_size = "3em";
        $this->display_charts = ChartOptions::IMMER;    
        if (array_key_exists('Sprache', $_SESSION)) $this->rLanguage = explode(' ',$_SESSION['Sprache'])[0];        
        for ($i=0; $i < count($this->defaultFilter); $i++){
            $df = $this->defaultFilter[$i];
            $f = new RepFilter($df->column_name);
            $f->fs_von = $df->fs_von;
            $f->fs_bis = $df->fs_bis;
            $this->repFilter->put($f);
        }
        $this->rLanguage = $this->currentLanguage;
        $this->currentRLanguage = $this->currentLanguage;
        $this->rTranslations = array();
    }
    
    public function addNewDimension($dimName){
        $dim = new Dimension(null);
        if (count($this->dimensions) >= 1) $this->dimensions[] = $dim;
        else $this->dimensions[1] = $dim;
        end($this->dimensions);
        $dim->dimNr = key($this->dimensions);
        $dim->dimName = $dimName;
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'select attribute from queryAttribute ';
        $sql->addWhere('SatzTyp = 8');
        $sql->addSel('table_name = ', $dimName);
        $stmt = $sql->execute();
        if ($z = $stmt->fetchObject()) $dim->dimKey = $z->attribute;
        return $dim;
    }
    
    public function cloneSimpleQuery(){
        $newQuery = new Query($this->class_table);
        $newQuery->dwh = $this->dwh;
        $newQuery->rowVars = $this->rowVars;
        $newQuery->valueVars = $this->valueVars;
        $newQuery->filterVars = $this->filterVars;
        $newQuery->dimensions = $this->cloneDimensions();        
        $newQuery->metaData = $this->metaData;
        $newQuery->addQueries = new Vector();
        $newQuery->outputTitlePrefix = $this->outputTitlePrefix;
        $newQuery->phys_table = $this->phys_table;
        $newQuery->queryType = $this->queryType;
        $newQuery->histType = $this->histType;
        $newQuery->historyOption = $this->historyOption;
        $newQuery->histAb = $this->histAb;
        $newQuery->histBis = $this->histBis;
        $newQuery->histStichtag = $this->histStichtag;
        $newQuery->dimKey = $this->dimKey;
        $newQuery->defaultFilter = $this->defaultFilter;
        $newQuery->table_description = $this->table_description;
        $newQuery->table_hints = $this->table_hints;
        $newQuery->newReport();
        return $newQuery;
    }
    
    function cloneDimensions(){
        $dc = array();
        foreach ($this->dimensions as $nr => $dim){
            $d = $dim->clone();
            $dc[$nr] = $d;
        }
        return $dc;
    }
    
    /**
     * prüft, ob die Dimension in der aktuellen Report-Definition verwendet wird
     * @param integer $dimNr
     */
    function reportHasDim($quelle,$dimNr){
        if ($dimNr <= 0) return true;
        if ($quelle == 'base' || $quelle == null) $prefix = null;
        else $prefix = $quelle.':';
        for ($i = 0; $i < $this->repRows->size(); $i++){
            $name = $this->repRows->getPos($i)->getName();
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && array_key_exists($name, $this->repTables->get($quelle)->query->metaData))
                || ($prefix != null && strpos($name, $prefix) === 0)){
                $mr = $this->getReportMetaRecord($name);
                if ($mr != null && $this->checkDim($mr, $dimNr)) return true;
            }
            
        }
        for ($i = 0; $i < $this->repValues->size(); $i++){
            $name = $this->repValues->getPos($i)->getName();
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && strpos($name, $prefix) === 0)){
                    $mr = $this->getReportMetaRecord($name);
                    if ($mr != null && $this->checkDim($mr, $dimNr)) return true;
            }
        }
        for ($i = 0; $i < $this->repFilter->size(); $i++){
            $name = $this->repFilter->getPos($i)->getName();
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && array_key_exists($name, $this->repTables->get($quelle)->query->metaData))
                || ($prefix != null && strpos($name, $prefix) === 0)){
                    $mr = $this->getReportMetaRecord($name);
                    if ($mr != null
                        && ($this->repFilter->getPos($i)->isSet()
                            || $this->repFilter->getPos($i)->selectorImOutput)
                        && $this->checkDim($mr, $dimNr)) return true;
            }
        }
        for ($i = 0; $i < $this->addFilter->size(); $i++){
            if ($this->addFilter->getPos($i) == null) continue;
            $name = $this->addFilter->getPos($i)->getName();
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && array_key_exists($name, $this->repTables->get($quelle)->query->metaData))
                || ($prefix != null && strpos($name, $prefix) === 0)){
                    $mr = $this->getReportMetaRecord($name);
                    if ($mr != null
                        && $this->addFilter->getPos($i)->isSet()
                        && $this->checkDim($mr, $dimNr)) return true;
            }
        }
        for ($i = 0; $i < $this->repColumns->size(); $i++){
            $name = $this->repColumns->getPos($i)->getName();
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && array_key_exists($name, $this->repTables->get($quelle)->query->metaData))
                || ($prefix != null && strpos($name, $prefix) === 0)){
                    $mr = $this->getReportMetaRecord($name);
                    if ($mr != null && $this->checkDim($mr, $dimNr)) return true;
            }
            $name = $this->repColumns->getPos($i)->val_name;
            if (isValid($name))
            if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                || ($prefix != null && strpos($name, $prefix) === 0)){
                    $mr = $this->getReportMetaRecord($name);
                    if ($mr != null && $this->checkDim($mr, $dimNr)) return true;
            }
            $name = $this->repColumns->getPos($i)->filter;
            if (isValid($name))
                if (($prefix == null && substr($name, 1,1) != ':' && substr($name, 1,1) != '=')
                    || ($prefix != null && array_key_exists($name, $this->repTables->get($quelle)->query->metaData))
                    || ($prefix != null && strpos($name, $prefix) === 0)){
                        $mr = $this->getReportMetaRecord($name);
                        if ($mr != null && $this->checkDim($mr, $dimNr)) return true;
            }
        }                
        return false;
    }    
    
    private function checkDim($mr,$dimNr){
        if (false) $mr = new MetaRecord();
        if ($mr->dimNr == $dimNr) {
            return true;
        }
        foreach ($mr->reqDim as $rNr)
            if ($rNr == $dimNr) {
                return true;
        }
        return false;
    }
    
    /**
     * @return boolean
     */
    public function hasReportShowFilter(){
        for ($i = 0; $i < $this->repFilter->size();$i++){
            $filter = $this->repFilter->getPos($i);            
            if ($filter->isSet() && $filter->isWindowFilter()) return true;
            if ($filter->isSet() && $filter->isValueFilter()) return true;
        }
        return false;
    }
    
    public function reportHasEnterValues(){
        if ($this->queryType == Query::DATEN) return false;
        if ($this->queryType >= Query::DIMENSION) return false;
        for ($i = 0; $i < $this->repValues->size(); $i++){
            $rv = $this->repValues->getPos($i);
            $mr = $this->getReportMetaRecord($rv->getName());
            if ($mr != null && $mr->enterValue) return true;
        }
        for ($i = 0; $i < $this->repColumns->size(); $i++){
            $rc = $this->repColumns->getPos($i);
            if (false) $rc = new RepColumn('');
            if (isValid($rc->val_name)){
                $mr = $this->getReportMetaRecord($rc->val_name);
                if ($mr != null && $mr->enterValue) return true;
            }
        }
        return false;
    }
    
}



Class RepRow { 
    function __construct($name){
        $this->name = $name;
    }
    private $name = null;
    public function getName() {
        return $this->name;
    }
    public function getSelName() {
        return 'S_'.$this->name;        
    }
    
    public $summenZeilen = false;
    public $web_link = "";
    public $web_title = "";
    public $web_target = "";
    public $webCopyFilter = false;
    public $sort = Sort::SORTINTERN;
    public $fixedTableColumns = false;
    
    public $web_link_title = "";
    public $web_link_value = "";
    public $webLinkColumn = false;
    
    public $deleteStatement = 'delete from reportRows where reportID = :reportID';
    public static $readStatement = 'select * from reportRows where reportID = :reportID order by SortNr';
    public $insertStatement = <<<EOS
 insert into reportRows set reportID = :reportID
   , SortNr = :SortNr    
   , column_name = :column_name
    , sort = :sort
    , webLinkColumn = :webLinkColumn
    , web_link_title = :web_link_title
    , web_link_value = :web_link_value
    , web_link = :web_link
    , webCopyFilter = :webCopyFilter
    , web_title = :web_title
    , web_target = :web_target
    , fixedTableColumns = :fixedTableColumns
    , summenZeilen = :summenZeilen;
EOS;    
    
    
    public function getInsertArray(){
        $array = array();
        $array['column_name'] = $this->name;
        $array['sort'] = $this->sort;
        $array['webLinkColumn'] = bool($this->webLinkColumn);
        $array['web_link_title'] = $this->web_link_title;
        $array['web_link_value'] = $this->web_link_value;
        $array['web_link'] = $this->web_link;
        $array['webCopyFilter'] = bool($this->webCopyFilter);
        $array['web_title'] = $this->web_title;
        $array['web_target'] = $this->web_target;
        $array['fixedTableColumns'] = bool($this->fixedTableColumns);
        $array['summenZeilen'] = bool($this->summenZeilen);
        return $array;
    }
    
    public function read($z){
        if (false) $z = new RepRow('');
        $this->name = $z->column_name;
        $this->fixedTableColumns = $z->fixedTableColumns;
        $this->sort = $z->sort;
        $this->summenZeilen = $z->summenZeilen;
        $this->web_link = $z->web_link;
        $this->web_link_title = $z->web_link_title;
        $this->web_link_value = $z->web_link_value;
        $this->web_target = $z->web_target;
        $this->web_title = $z->web_title;
        $this->webCopyFilter = $z->webCopyFilter;
        $this->webLinkColumn = $z->webLinkColumn;
    }
    
}
Class RepValue { 
    function __construct($name){
        $this->name = $name;
    }
    private $name = null;
    
    public $function = 0;
    public $value = null;
    public $abSpalte = 0;
    public $nachSpalten = false;
    
    public $dispVal = DisplayOptions::ZAHL;
    public $skalierung = ChartOptions::SKALIEREGRUPPE;
    public $keineSummen = false;
    public $base = 'base';
    public $label = null;
    public $backgroundColor = "white";
    
    public $deleteStatement = 'delete from reportValues where reportID = :reportID';
    public static $readStatement = 'select * from reportValues where reportID = :reportID order by SortNr';
    public $insertStatement = <<<EOS
 insert into reportValues set reportID = :reportID
   , SortNr = :SortNr
   , column_name = :column_name
    , label = :label
    , keineSummen = :keineSummen
    , backgroundColor = :backgroundColor
    , dispval = :dispVal
    , skalierung = :skalierung
    , base = :base
    , wfFunction = :function
    , wfValue = :value
    , abSpalte = :abSpalte
    , nachSpalten = :nachSpalten
EOS;
    
    const functions = array(
        0 => "(Wert)",
        1 => "kum. Summe",
        2 => "%-Anteil",
        3 => "kum. %-Anteil",
        10 => '&Oslash; Mittelwert',
        11 => 'Diff. zum &Oslash;',
        12 => 'Diff. zum &Oslash; in %',
// Die Implementierung des Medians in der Mariadb ist bei großen Werten kaputt         
/*        14 => 'Median',
        15 => 'Diff. zum Median',
        16 => 'Diff. zum Median in %',
        */
        17 => 'Std-Dev.',
        18 => 'Diff. zum &Oslash; in % STD-Dev.',
        20 => "Rang",
        21 => "Rang &dArr;",
        22 => "Cume_Dist"
    );
    public static function getFunctionText($option){
        if ($option >= 0) return WSRTranslate::translate(self::functions[$option]);
        return WSRTranslate::translate(self::functions['']);
    }
       
    
    const beschreibung = array (
        1 => 'kumulierte Summe'
        , 2 => '%-Anteil'
        , 3 => 'kumulierter %-Anteil'
        , 10 => 'Mittelwert'
        , 11 => 'Differenz zum Mittelwert'
        , 12 => 'Differenz zum Mittelwert in %'
        , 17 => 'Standardabweichung'
        , 18 => 'Differenz zum Mittelwert in % der Standardabweichung'
        , 20 => 'Rangfolge der Werte (aufsteigend)'
        , 21 => 'Rangfolge der Werte (absteigend)'
        , 22 => 'kumulierte Verteilung in % bei aufsteigenden Werten'
    );
    
    public static function getFunctionDesc($option){
        if ($option > 0 && array_key_exists($option, self::beschreibung)) return WSRTranslate::translate(self::beschreibung[$option]);
        return '';
    }
    
    const formatDecs = array(
        2 => 1
        , 3 => 0
        , 12 => 1
        , 16 => 1
        , 18 => 1
        , 20 => 0
        , 21 => 0
        , 22 => 1
    );
    
    public function hasFunctionFormatDecs(){
        return array_key_exists($this->function, self::formatDecs);
    }
    
    public function getFunctionFormatDecs(){
        if (array_key_exists($this->function, self::formatDecs)) return self::formatDecs[$this->function];
        return 0;
    }
    
    public function getFunctionSelect($sortBy){
        global $query;
        $mr = $query->getReportMetaRecord($this->getValueName());
        $select = '';
        if ($mr == null){
            echo '<br>FEHLER: Keine Metadaten für '.$this->getValueName().' bei '.$this->getName();
            error_log('FEHLER: Keine Metadaten für '.$this->getValueName().' bei '.$this->getName());
            die();
        }
            $vn = $this->getValueName();
/*            
            if (substr($vn,1,1) == '='){
                return "\n-- FEHLER bei {$vn}: Anforderung noch nicht implementiert\n";
            }else
                if (substr($vn,1,1) == ':'){
                    $base = substr($vn,0,1).'.';
            }else {
                   $base = 'base.';
            }
            */
            $base = 'base.';
            if ($this->function > 0){
                if ($this->abSpalte > 0){
                    $maxFunctionenLevel = $query->repRows->size();
                    for($ir = $query->repRows->size() -1; $ir > 0 &&  $query->repRows->getPos($ir)->webLinkColumn ; $ir--){
                        $maxFunctionenLevel --;
                    }
//                    if ($query->values_in_columns) $maxFunctionenLevel --;
//                    else 
                        if ($this->nachSpalten) $maxFunctionenLevel --;
                    
                    if ($this->abSpalte > $maxFunctionenLevel) $this->abSpalte = $maxFunctionenLevel;                    
                }
                              
                
                $by = 'base.Stufe, base.colGroup';
                $order = '';
                // , lead.S0 ';
                // $sql->sql .= 'order by lead.S_Gebiet, lead.S1, lead.S_Region ';
                for ($ic=0; $ic < $query->repRows->size(); $ic++){
                    $rRow = $query->repRows->getPos($ic);
                    if ($rRow->webLinkColumn) continue;
                    $rName = 'base.'.$query->getReportMetaRecord($rRow->getName())->getSelName();
                    
                    if ($ic <= $this->abSpalte) $by .= ', base.S'.$ic;
                    else $order .= ', base.S'.$ic;
                    if ($ic < $this->abSpalte) $by .= ', '.$rName;
                    elseif ($ic == $this->abSpalte) $order = ' order by '.$rName;
                    else $order .= ', '.$rName;                   
                }
                //                            $sql->sql .= "\n-- abSpalte = ".$var->abSpalte."\n";
                $select = "case when base.stufe > {$this->abSpalte} ";
                if ($this->nachSpalten) {
                    $select .= " or (base.stufe = {$this->abSpalte} and base.colGroup < 99 and base.S{$this->abSpalte} = 0)";
                }
                else  $select .= " or ( base.colGroup < 99 )";
                $select .= " then\n";
            }
            if (isValid($sortBy)){
                $order = $sortBy;
            }
            if ($query->repColumns->size() > 0){
                
                if ($this->nachSpalten){
                    for ($ic = 0; $ic < $query->repColumns->size(); $ic++){
                        $rc = $query->repColumns->getPos($ic);
                        $mr = $query->getReportMetaRecord($rc->getName());
                        $vn = $mr->getSelName();
                        if (substr($rc->getName(),1,1) == ':'){
                            if ($query->repTables->get(substr($rc->getName(),0,1))->joinType == JoinedTable::JOINSPALTEN){
                                $vn = substr($rc->getName(),0,1).'__'.$vn;
                            }
                        }
                        $by .= ', ';
                        $by .= 'base.'.$vn;
                    }
                }else{
                    if (!isValid($order))  $order = ' order by base.colGroup';                   
                    for ($ic = 0; $ic < $query->repColumns->size(); $ic++){
                        /**
                         * ACHTUNG: wenn eine Spaltenvariable mehrfach verwendet wird, taucht sie auch hier mehrfach auf
                         * @var RepColumn $repCol;
                         */
                        $repCol = $query->repColumns->getPos($ic);                      
                        $mr = $query->getReportMetaRecord($repCol->getName());
                        $vn = $mr->getSelName();
                        $vnd = $mr->getDispName();
                        if (substr($repCol->getName(),1,1) == ':'){
                            if ($query->repTables->get(substr($repCol->getName(),0,1))->joinType == JoinedTable::JOINSPALTEN){
                                $vn = substr($repCol->getName(),0,1).'__'.$vn;
                                $vnd = substr($repCol->getName(),0,1).'__'.$vnd;
                            }
                        }
                        switch ($repCol->sort){
                            case Sort::SORTINTERN: $order .=', '.$vn;
                            break;
                            case Sort::SORTINTERNDESC: $order .=', '.$vn.' desc';
                            break;
                            case Sort::SORTANZEIGE: $order .=', '.$vnd;
                            break;
                            case Sort::SORTANZEIGEDESC: $order .=', '.$vnd.' desc';
                            break;
                            default: $order .=', '.$vn;
                            break;
                        }
                    }
                }
            }           
            
            $base = $base.str_replace('=', '__',str_replace(':', '__', $this->getValueName()));    

        switch ($this->function) {
            case 0:
                 $select = $base.$this->getValueName();
                 break;
            case 1: $select .= "sum(".$base
                    .") over(partition by $by  $order )";
                break;
            case 2: $select .= $base." *100 / sum(".$base
            .") over(partition by $by )";
            break;
            case 3: $select .= "sum(".$base
            .") over(partition by $by  $order ) *100 / sum(".$base
            .") over(partition by $by )";
            break;
            case 10: $select .= 'avg('.$base
            .") over(partition by $by   )";
            break;
            case 11: $select .= $base." - avg(".$base
            .") over(partition by $by )";
            break;
            case 12: $select .= $base." * 100 / avg(".$base
            .") over(partition by $by ) -100";
            break;
            
            case 13: $select .= 'median('.$base
            .") over(partition by $by  )";
            break;
            case 14: $select .= $base." + (median(".$base
            .") over(partition by $by ))-".$base;
            break;
            case 15: $select .= '('.$base." - (median(".$base
            .") over(partition by $by )))";
            break;
            case 16: $select .= $base." * 100 / median(".$base
            .") over(partition by $by ) -100";
            break;
            
            case 17: $select .= 'STDDEV_POP('.$base
            .") over(partition by $by   )";
            break;
            case 18: $select .= '('.$base." - avg(".$base
            .") over(partition by $by )) / STDDEV_POP(".$base
            .") over(partition by $by ) * 100";;
            break;
            case 20: $select .= 'RANK('
                .") over(partition by $by  ORDER BY ".$base.")";
                break;
            case 21: $select .= 'RANK('
                .") over(partition by $by  ORDER BY ".$base." desc)";
                break;
            case 22: $select .= 'CUME_DIST('
                .") over(partition by $by  ORDER BY ".$base.") * 100";
                break;
                
            
       //     PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY star_rating)
            default:
                error_log('getFunctionSelect für '.$this->getName().': ungültige Function '.$this->function);
                return  $base;
        
        }
        if ($this->function > 0){
            $select .= "\nelse null end ";
        }
            error_log('getFunctionSelect für '.$this->getName().': '.$select);
        return $select;
    }
    /*
     *      0 => "(Wert)",
        1 => "kum. Summe",
        2 => "%-Anteil",
        3 => "kum. %-Anteil",
        10 => '&Oslash; Mittelwert',
        11 => 'Diff. zum &Oslash;',
        12 => 'Median',
        13 => 'Diff. zum Median',
        20 => "Rang"
     * 
     * 
        0 => '#value'
        1 => 'sum(#value) over(partition by #by #order)',
        2 => '(100 * #value sum(#value) over(partition by #by )',
        3 => "kum. %-Anteil",
        10 => '&Oslash; Mittelwert',
        11 => 'Diff. zum &Oslash;',
        12 => 'Median',
        13 => 'Diff. zum Median',
        20 => "Rang"
        self::SUM => 
        , self::ASUM => '(sum(#value) over(partition by #by  order by #value #desc))'
        .'/(sum(#value) over(partition by #by ))'
        , self::KSUM => 'sum(#value) over(partition by #by  order by #value #desc)'
        , self::RANG => 'row_number() over(partition by #by order by #value #desc)'
        , self::PCTRANG => 'percent_rank() over(partition by #by order by #value #desc)'
        , self::NTILE => 'ntile(#ntile) over(partition by #by order by #value #desc)'
    );
*/
 
    
    
    
/*

 */    
    public function getName() {
        return $this->name;
    }
    public function getValueName() {
        if (isValid($this->value)
            && $this->function > 0) return $this->value;
        return $this->name;
    }
    public function rename($n) {
        $this->name = $n;
    }
    public function isWindowValue(){
        if (isValid($this->value)
            && $this->function > 0) return true;
        return false;
    }
    
    public function getInsertArray(){
        $array = array();
        $array ['column_name'] = $this->name;
        $array['keineSummen'] = bool($this->keineSummen);
        $array['label'] = $this->label;
        $array['backgroundColor'] = $this->backgroundColor;
        if ($this->base != 'base') $array['base'] = $this->base;
        else  $array['base'] = null;
        $array['dispVal'] = $this->dispVal;
        $array['skalierung'] = $this->skalierung;
        $array ['function'] = $this->function;
        $array ['value'] = $this->value;
        $array ['abSpalte'] = $this->abSpalte;
        if ($this->nachSpalten) $array ['nachSpalten'] = 1;
        else $array ['nachSpalten'] = 0;
        
        return $array;
    }
    
    public function read($z){
        if (false) $z = new RepValue('');
        $this->name = $z->column_name;
        if (isValid($z->base))  $this->base = $z->base;
        else $this->base = 'base';
        $this->keineSummen = $z->keineSummen;
        $this->label = $z->label;
        $this->backgroundColor = $z->backgroundColor;
        $this->dispVal = $z->dispval;
        $this->skalierung = $z->skalierung;
        $this->function = $z->wfFunction;
        $this->value = $z->wfValue;
        $this->abSpalte = $z->abSpalte;
        $this->nachSpalten = $z->nachSpalten;
    }
    
    
}
Class RepColumn { 
    public $column_name = null;
    public $label = "";
    public $val_name = null;
    public $function = 0;
    public $abSpalte = 0;
    public $nachSpalten = false;
    
    public $displayOption = DisplayOptions::ZAHL;
    public $skalierung = ChartOptions::SKALIEREGRUPPE;
    public $filter  = null;
    public $fs_von = null;
    public $fs_bis = null;
    public $fs_list = null;
    public $web_link = null;
    public $web_title = null; // Aktuell nicht benutzt
    public $web_target = null;
    public $reportCopyFilter = false;
    public $reportLinkInValues = false;
    public $sort = Sort::SORTINTERN;
    public $keineSummen = false; // zur Zeit noch nicht bearbeitet oder gespeichert ?
    
    function __construct($name){
        global $query;
        $this->column_name = $name;
        $name = explode('|',$name)[0];
        if (!isValid($name)) return;
        if (substr($name,1,1) == ':'){
            if ($query->repTables->get(substr($name,0,1)) == null ) echo "FEHLER bei RepColumn $name : Tabelle noch nicht geladen\n";
            else
            $this->label = $query->getReportMetaRecord($name)->label;
        }
        else if (array_key_exists($name, $query->metaData))
            $this->label = $query->getReportMetaRecord($name)->label;
        else {
            $this->label = "FEHLER bei ".$name;
            echo "FEHLER bei RepColumn für ".$name;
        }
        if ($this->label == null){
            echo "<pre>DebugMeldung für $name : Joined Table nicht gefunden?";
            var_dump($query->repTables);
            echo '</pre>';
        }
    }
    
    public $deleteStatement = 'delete from reportColumns where reportID = :reportID';
    public static $readStatement = 'select * from reportColumns where reportID = :reportID order by SortNr';
    public $insertStatement = <<<EOS
 insert into reportColumns set reportID = :reportID
   , SortNr = :SortNr
   , column_name = :column_name
   , column_label = :column_label
    , sort = :sort
    , val_name = :val_name
    , filter = :filter
    , fs_von = :fs_von
    , fs_bis = :fs_bis
    , web_link = :web_link
    , web_title = :web_title
    , web_target = :web_target
    , reportCopyFilter = :reportCopyFilter
    , reportLinkInValues = :reportLinkInValues
    , keineSummen = :keineSummen
    , dispval = :dispval
    , skalierung = :skalierung 
    , wfFunction = :function
    , abSpalte = :abSpalte
    , nachSpalten = :nachSpalten

    ;
EOS;
    
    
    public function getInsertArray(){
        $array = array();
        $array['column_name'] = $this->column_name;
        $array['column_label'] = $this->label;
        $array['sort'] = $this->sort;
        $array['val_name'] = $this->val_name;
        if (isValid($this->filter))
            $array['filter'] = $this->filter;
        else
            $array['filter'] = null;
        $array['fs_von'] = $this->fs_von;
        $array['fs_bis'] = $this->fs_bis;
        $array['web_link'] = $this->web_link;
        $array['web_title'] = $this->web_title;
        $array['web_target'] = $this->web_target;
        $array['reportCopyFilter'] = bool($this->reportCopyFilter);
        $array['reportLinkInValues'] = bool($this->reportLinkInValues);
        $array['keineSummen'] = bool($this->keineSummen);
        $array['dispval'] = $this->displayOption;
        $array['skalierung'] = $this->skalierung;
        $array ['function'] = $this->function;
        $array ['abSpalte'] = $this->abSpalte;
        if ($this->nachSpalten) $array ['nachSpalten'] = 1;
        else $array ['nachSpalten'] = 0;
        
        return $array;
    }
    
    public function read($z){
        if (false) $z = new RepColumn('');
        $this->name = $z->column_name;
        $this->label = $z->column_label;
        $this->sort = $z->sort;
        $this->val_name = $z->val_name;
        $this->filter = $z->filter;
        $this->fs_von = $z->fs_von;
        $this->fs_bis = $z->fs_bis;        
        $this->web_link = $z->web_link;
        $this->web_title = $z->web_title;
        $this->web_target = $z->web_target;        
        $this->reportCopyFilter = $z->reportCopyFilter;
        $this->reportLinkInValues = $z->reportLinkInValues;
        $this->keineSummen = $z->keineSummen;
        $this->displayOption = $z->dispval;
        $this->skalierung =$z->skalierung;
        $this->function = $z->wfFunction;
        $this->abSpalte = $z->abSpalte;
        $this->nachSpalten = $z->nachSpalten;
        
    }
    
    public function hasFunctionFormatDecs(){
        return array_key_exists($this->function, RepValue::formatDecs);
    }
    
    public function getFunctionFormatDecs(){
        if (array_key_exists($this->function, RepValue::formatDecs)) return RepValue::formatDecs[$this->function];
        return 0;
    }
    
    
    public function getName(){
        return $this->column_name;
    }
    public function getVarName(){
        return explode('|', $this->column_name)[0];        
    }
    
    public function isSet(){
        global $query;
        $mr = $query->getReportMetaRecord($this->filter);
        if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTER) return true;
        if ($mr != null && $mr->filterType == MetaRecord::WINDOWFILTER) return true;
        if ($mr != null && $mr->filterType == MetaRecord::LIST
            && $this->fs_list != null && count($this->fs_list) > 0) return true;
            return isValid($this->fs_von) || isValid($this->fs_bis);
    }   
}


Class RepFilter { 
    function __construct($name){
        $this->name = $name;
    }
    protected $name = null;
    public function getName() {
        return $this->name;
    }
    public function rename($n) {
        $this->name = $n;
    }
    
    public $fs_von = null;
    public $fs_bis = null;
    public $fs_list = null;
    public $ausschlussListe = false;
    public $abSpalte = 0;
    public $inTitelZeile = 0;
    public $inTitelMitLabel = false;
    public $selectorImOutput = false;
    /**
     * Normalerweise werden Filter aus der Report-Definition 
     * in den verknüpften Aufrufen nicht weitergegeben.
     * Mit diesem Kennzeichen wird der Filter so behandelt,
     * als wäre er im Aufruf des Reports gesetzt. 
     * @var boolean
     */
    public $reportCopyFilter = false;
    /**
     * Kennzeichen, das Filter aus Aufruf NICHT
     * übernommen wird.
     */
    public $doNotTake = false;
    /**
     * Laufzeit-Flag, dass der Filter in der ON-Klausel gesetzt wurde.
     */
    public $erledigt = false;
    
    public $deleteStatement = 'delete from reportFilters where reportID = :reportID';
    public static $readStatement = 'select * from reportFilters where reportID = :reportID order by SortNr';
    public $insertStatement = <<<EOS
 insert into reportFilters set reportID = :reportID
   , SortNr = :SortNr
   , column_name = :column_name
    , fs_von = :fs_von
    , fs_bis = :fs_bis
    , ausschlussListe = :ausschlussListe
    , abSpalte = :abSpalte
    , selectorImOutput = :selectorImOutput
    , inTitelZeile = :inTitelZeile
    , inTitelMitLabel = :inTitelMitLabel
    , reportCopyFilter = :reportCopyFilter
    , doNotTake = :doNotTake 
EOS;
    
    
    public function getInsertArray(){
        $array = array();
        $array ['column_name'] = $this->name;
        $array ['fs_von'] = $this->fs_von;
        $array ['fs_bis'] = $this->fs_bis;
        $array ['ausschlussListe'] = bool($this->ausschlussListe);
        $array ['abSpalte'] = $this->abSpalte;
        $array ['selectorImOutput'] = bool($this->selectorImOutput);
        $array ['inTitelZeile'] = $this->inTitelZeile;
        $array ['inTitelMitLabel'] = bool($this->inTitelMitLabel);
        $array ['reportCopyFilter'] = bool($this->reportCopyFilter);
        $array ['doNotTake'] = bool($this->doNotTake);
        return $array;
    }
    
    public function read($z){
        $this->name = $z->column_name;
        $this->fs_von = $z->fs_von;
        $this->fs_bis = $z->fs_bis;
        $this->ausschlussListe = $z->ausschlussListe;
        $this->abSpalte = $z->abSpalte;
        $this->selectorImOutput = $z->selectorImOutput;
        $this->inTitelZeile = $z->inTitelZeile;
        $this->inTitelMitLabel = $z->inTitelMitLabel;
        $this->reportCopyFilter = $z->reportCopyFilter;
        $this->doNotTake = $z->doNotTake;
    }
    
    
    
    
    // HIER FEHLEN KEINE METHODEN für Where-Klausel und Anzeige, weil sie den MetaRecord erfordern würden.
    /**
     * Zum Vergleich mit gleichnamiger Variablen im Metarecord
     * Das ist zwar großer Blödsinn, geht im Moment aber nicht anders.
     */
    public function queryOfSelect(){
        if (strlen($this->name) < 3) return 'base';
        if (substr($this->name,1,1) == ':') return substr($this->name,0,1);
        if (substr($this->name,1,1) == '=') return substr($this->name,0,1);
        return 'base';
    }
    
    public function isSet(){
        global $query;
        $mr = $query->getReportMetaRecord($this->name);
        if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTER) return true;
        if ($mr != null && $mr->filterType == MetaRecord::WINDOWFILTER) return true;
        if ($mr != null && $mr->filterType == MetaRecord::LIST 
            && $this->fs_list != null && count($this->fs_list) > 0) return true;
        return isValid($this->fs_von) || isValid($this->fs_bis);
    }
    
    public function isJoinedFilter(){
        if (substr($this->name,1,1) == ':' || substr($this->name,1,1) == '=') return true;
        return false;
    }
    
    public function isValueFilter(){
        global $query;
        $mr = $query->getReportMetaRecord($this->name);
        if ($mr == null) return false;
        if ($mr->filterType == MetaRecord::SONDERFILTER) return false;
        if ($mr->filterType == MetaRecord::SONDERFILTERSEL) return false;
        if ($mr->filterType == MetaRecord::LIST) return false;
        if ($this->isJoinedFilter()){
            $jquery = $query->repTables->get(substr($this->name, 0,1))->query;
            if (array_search(substr($this->name,2), $jquery->rowVars) !== false) return false;
            if (array_search(substr($this->name,2), $jquery->filterVars) !== false) return false;            
        }else{        
            if (array_search($this->name, $query->rowVars) !== false) return false;
            if (array_search($this->name, $query->filterVars) !== false) return false;
        }
        return true;
    }
    
    public function isWindowFilter(){        
        return false;
    }
}


class WindowFilter extends RepFilter{
    
    const SUM = 1;
    const KSUM = 2;
    const ASUM = 3;
    const RANG = 21;
    const PCTRANG = 22;
    const NTILE = 23;
    
    const klartext = array (
        self::SUM => 'Summe'
        , self::RANG => 'Rangfolge'
        , self::PCTRANG => 'Anteil in %'
        , self::NTILE => 'Anteil ++'
        , self::KSUM => 'kum. Summe'
        , self::ASUM => 'bis Anteil Summe in %'
    );
    
    const beschreibung = array (
        self::SUM => 'Summe der gewählten Ebene'
        , self::ASUM => "kumulierter maximaler Anteil an der Gruppen-Summe\nbis zu der eine Zeile angezeigt wird"
        , self::KSUM => "kumulierte Summe\nbis zu der eine Zeile angezeigt wird"
        , self::RANG => "Rangfolge\nab der oder bis zu der\neine Zeile angezeigt wird.\nBei gleichen Werten an der Grenze\nist die Auswahl zufällig."
        , self::PCTRANG => "Anteil der Zeilen in %\nab dem oder bis zu dem\neine Zeile angezeigt wird (0,0 bis 100,0)"
        , self::NTILE => "Anteil der Zeilen ab dem oder bis zu dem\neine Zeile angezeigt wird, mit Mindestzahl.\n Beispiel 2/10 : 20%, mindestens 2 Zeilen"
                          ."\n 1/3 : ein Drittel, mindestens eine Zeile."
    );
    
    
    const select = array(
        self::SUM => 'sum(#value) over(partition by #by )'
        , self::ASUM => '(sum(#value) over(partition by #by  order by #value #desc))'
                        .'/(sum(#value) over(partition by #by ))'
        , self::KSUM => 'sum(#value) over(partition by #by  order by #value #desc)'
        , self::RANG => 'row_number() over(partition by #by order by #value #desc)'
        , self::PCTRANG => 'percent_rank() over(partition by #by order by #value #desc)'
        , self::NTILE => 'ntile(#ntile) over(partition by #by order by #value #desc)'
    );
    
    
    public function __construct($name) {
        global $query;
        $this->name = $name;
        $this->fs_bis = ''.self::SUM.'; ;desc';
        if ($query->repRows->size() > 1) $this->abSpalte = $query->repRows->size()-1;
    }
    
    /**
     * hat Sortierung
     * @return boolean
     */
    public function isSortFilter(){
        if ($this->getType() == self::SUM) return false;
        return true;
    }
    public function isWindowFilter(){
        return true;
    }
    public function isValueFilter(){
        return false;
    }
    
    public function isSet(){
        $t = explode(';', $this->fs_bis);
        return isValid($this->fs_von) && count($t) > 1 && isValid($t[1]);
    }
    
    public function getType(){
        if ($this->fs_bis == null) return null;
        $t = explode(';', $this->fs_bis);
        if (isValid($t[0])) return 1 * $t[0];
        return null;        
    }

    public function getWert(){
        if ($this->fs_bis == null) return null;
        $t = explode(';', $this->fs_bis);
        if (isValid($t[1])) return $t[1];
        return null;
    }
    
    public function getDesc(){
        if ($this->fs_bis == null) return false;
        $t = explode(';', $this->fs_bis);
        if (isValid($t[2])) return true;
        return false;
    }
    public function setType($type){
        if ($this->fs_bis == null) $this->fs_bis = '';
        $t = explode(';', $this->fs_bis);
        $this->fs_bis = ''.$type;
        $this->fs_bis .= ';';
//        if (isValid($t[0])) return 1 * $t[0];
        if (count($t) > 1) $this->fs_bis .= $t[1];
        $this->fs_bis .= ';';
        if (count($t) > 2) $this->fs_bis .= $t[2];
    }
    
    public function setWert($wert){
        if ($this->fs_bis == null) $this->fs_bis = '';
        $t = explode(';', $this->fs_bis);
        $this->fs_bis = '';
        if (count($t) > 0) $this->fs_bis .= $t[0];
        $this->fs_bis .= ';';
        $this->fs_bis .= $wert;
        $this->fs_bis .= ';';
        if (count($t) > 2) $this->fs_bis .= $t[2];
    }
    
    public function setDesc($desc){
        if ($this->fs_bis == null) $this->fs_bis = '';
        $t = explode(';', $this->fs_bis);
        $this->fs_bis = '';
        if (count($t) > 0) $this->fs_bis .= $t[0];
        $this->fs_bis .= ';';
        if (count($t) > 1) $this->fs_bis .= $t[1];
        $this->fs_bis .= ';';
        if ($desc) $this->fs_bis .= 'desc';
    }
    
    
    public function getMetaRecord(){
        global $query;
        if ($this->fs_bis == null) return null;
        $mr = new MetaRecord();        
        $mr->init($this->queryOfSelect(),$this->name );
        $mr->description = self::beschreibung[$this->getType()];
        $mr->label = self::klartext[$this->getType()];
        $mr->filterType = MetaRecord::WINDOWFILTER;
        /* ??
        $mrw = $query->getReportMetaRecord($this->wert);
        if ($mrw == null){
            echo 'FEHLER by WindowFilter.getMetaRecord: Wert nicht gefunden '.$this->wert;
            die();
        }
        */
        $sv = $this->getWert();
        if (strlen($sv) >2 && substr($sv,1,1) == ':') $sv = str_replace(':', '.',$sv);
        else $sv = 'base.'.$sv;
        $sv = str_replace('#value', $sv , self::select[$this->getType()]);
        // Getrennt nach Spalten-Gruppen, weil nur 99er-Werte berücksichtigt werden
        if ($this->isSortFilter()) {
            $abSpalte = $query->repRows->size() -1;
            while ($query->repRows->getPos($abSpalte)->webLinkColumn
                && $abSpalte > 1) $abSpalte --;
                
        }
        else {
            $abSpalte = $this->abSpalte;
            if ($abSpalte < 1) $abSpalte = $query->repRows->size()-1;
        }
        $by = 'lead.colGroup';        
        for ($i=0; $i < $abSpalte ; $i++){
            $rr = $query->repRows->getPos($i);
            if (false) $rr = new RepRow(' ');
            if ($rr->webLinkColumn) continue;                
            $by .= ', lead.S'.$i;
            $by .= ', lead.S_'.$rr->getName();
        }
        $by .= ', lead.S'.$i;
        
        if($this->isSortFilter()){
            if ($this->getDesc()) $sv = str_replace('#desc', 'desc', $sv);
            else $sv = str_replace('#desc', '', $sv);
        }
        $sv = str_replace('#by', $by, $sv);
        
        if ($this->getType() == self::NTILE){
            $nt = '10';
            $t = explode('/',$this->fs_von);
            if (count($t) > 1 && isValid($t[1])){
                $nt = $t[1]; 
            }
            $sv = str_replace('#ntile', $nt, $sv);
            
        }
        
        $mr->selectVar = $sv;
        
//        $mr->description .= "\n".$mr->selectVar;
        return $mr;
    }
    
    public function getSelName(){
      return 'WSR_'.substr($this->name,1);   
    }
    
}


// Weitere Reports
Class RepReport {
    function __construct($name){
        $this->name = $name;
    }
    private $name = null;
    public function getName() {
        return $this->name;
    }
    public $kommentar = null;
    public $copyFilter = false;
    
    public $deleteStatement = 'delete from reportReports where reportID = :reportID';
    public static $readStatement = 'select * from reportReports where reportID = :reportID order by SortNr';
    public $insertStatement = <<<EOS
 insert into reportReports set reportID = :reportID
   , SortNr = :SortNr
   , reportTitle = :reportTitle
    , copyFilter = :copyFilter
    , kommentar = :kommentar
EOS;
    
    
    public function getInsertArray(){
        $array = array();
        $array ['reportTitle'] = $this->name;
        $array['copyFilter'] = bool($this->copyFilter);
        $array ['kommentar'] = $this->kommentar;
        return $array;
    }
    
    public function read($z,$query){
        $this->name = qTranslate::translateWebLink($query,$z->reportTitle);
        $this->copyFilter = $z->copyFilter;
        $this->kommentar = qTranslate::translateReport($query,$z->kommentar);
    }
    
}


Class ChartData{
    function __construct($colGroup,$color,$negColor){
        $this->colGroup = $colGroup;
        $this->color = $color;
        $this->negColor = $negColor;
        $this->values = array();
    }
    private $colGroup = null;
    public $color = null;
    public $negColor = null;
    public $values = null;
    public $min = null;
    public $max = null;
    public function getColGroup() {
        return $this->colGroup;
    }
    public function add($id,$wert){
        $this->values[$id] = $wert;
        if ($this->min == null || $this->min > $wert) $this->min = $wert;
        if ($this->max == null || $this->max < $wert) $this->max = $wert;
    }
    
}

/**
 * Moves sind temporär gespeicherte Verschiebungen von
 * Spalten im Output.
 * Sie werden grundsätzlich im Post-Request als Parameter übergeben.
 * 
 * Die hier angelegte Klasse kapselt die komplette Funktionalität
 * und entlastet so das Programm outputReport.php
 *
 */
class Moves{
    private $moves = array();
    private $movesFrom = array();
    private $movesTo = array();
    
    public function __construct(){
        if (array_key_exists('WSM_moves', $_REQUEST)){
            $mc = $_REQUEST['WSM_moves'];
            for ($i = 0; $i < $mc; $i++){
                $mpart = $_REQUEST['WSM_move'.$i];
                $mfrom = $_REQUEST['WSM_move'.$i.'From'];
                $mto = $_REQUEST['WSM_move'.$i.'To'];
                if ($mpart != null && $mfrom != null && $mto != null){
                    $this->moves[] = $mpart;
                    $this->movesFrom[] = $mfrom;
                    $this->movesTo[] = $mto;                    
                }
            }
        }
        if (array_key_exists('WSM_addMove', $_REQUEST) && $_REQUEST['WSM_addMove'] != ''){
            $this->moves[] = $_REQUEST['WSM_addMove'];
            $this->movesFrom[] = $_REQUEST['WSM_addMoveFrom'];
            $this->movesTo[] = $_REQUEST['WSM_addMoveTo'];
            
        }
            
    }
    
    public function addMove($mpart, $mfrom, $mto){
        if ($mpart != null && $mfrom != null && $mto != null){
            $this->moves[] = $mpart;
            $this->movesFrom[] = $mfrom;
            $this->movesTo[] = $mto;
        }        
    }
    
    public function apply(){
        global $query;
        for ($i=0; $i < count($this->moves); $i++){
            switch ($this->moves[$i]){
                case 'Zeilen': $query->repRows->movePos($this->movesFrom[$i], $this->movesTo[$i]);
                break;
                case 'Werte': $query->repValues->movePos($this->movesFrom[$i], $this->movesTo[$i]);
                break;
                case 'Spalten': $query->repColumns->movePos($this->movesFrom[$i], $this->movesTo[$i]);
                break;
            }
        }
    }
    
    public function outputParams(){
        for ($i=0; $i < count($this->moves); $i++){
?> 
<input type="hidden" name="WSM_move<?=$i ?>"  value="<?=$this->moves[$i] ?>" />
<input type="hidden" name="WSM_move<?=$i ?>From"  value="<?=$this->movesFrom[$i] ?>" />
<input type="hidden" name="WSM_move<?=$i ?>To"  value="<?=$this->movesTo[$i] ?>" />
<?php             
        }
        ?>
<input type="hidden" name="WSM_moves"  value="<?=count($this->moves) ?>" />
<input type="hidden" name="WSM_addMove"  value="" />
<input type="hidden" name="WSM_addMoveFrom"  value="" />
<input type="hidden" name="WSM_addMoveTo"  value="" />
<?php             
    }
    
}

class SqlAbfrage{
    public $sql='';
    public $param = array();
    public $firstWhere = true;
    private $dbsql = null;
    private $lastError = null;
    
    /**
     * @param String$dbHandle 'WSR' oder 'DWH'
     */
    function __construct($dbHandle){
        global $dbDWH, $dbh;
        require_once './phpsec/createDBH.inc';
        if ($dbHandle == 'WSR') $this->dbsql = $dbh;
        else if ($dbHandle == 'DWH') {
            if ($dbDWH == null) setdbDWH();
            $this->dbsql = $dbDWH;
        }
    }

    function test(){
        $stmt = $this->dbsql->prepare($this->sql);
        if ($stmt === false){
            http_response_code(425);
//            echo $this->dbsql->errorInfo()[2];
            echo $this->lastError[2];
            if (strlen($this->sql) <= 500){
                echo "\nTest SQL:\n".$this->sql;
            }
            else{
                echo "\nTest SQL:\n".substr($this->sql,0,500);
            }
            return false;
        }
        return true;
    }
    
    function prepare(){
        $stmt = $this->dbsql->prepare($this->sql);
        if ($stmt === false){
            $this->lastError = $this->dbsql->errorInfo();
            http_response_code(425);
            echo $this->dbsql->errorInfo()[2];
            if (strlen($this->sql) <= 500){
                echo "\nPrep. SQL:\n".$this->sql;
                error_log("SQL Fehler bei prepare: \n".$this->sql);
            }
            else{
                echo "\nPrep. SQL:\n".substr($this->sql,0,500);
                error_log("SQL Fehler bei prepare: \n".substr($this->sql,0,500));
            }
            error_log(print_r($this->dbsql->errorInfo(),true));
            $ex = new Exception();
            error_log($ex->getTraceAsString());
            return false;
        }
        return $stmt;
    }

    function execute(){
        return $this->executeE(false);
    }
    
    function executeE($showError){
        global $sqlErg, $testMode;
        $stmt = $this->dbsql->prepare($this->sql);
        if ($stmt === false){
            $this->lastError = $this->dbsql->errorInfo();
            http_response_code(425);            
            if ($showError) echo $this->dbsql->errorInfo()[2];
            if (!$testMode)
            if (strlen($this->sql) <= 500){
                if ($showError) echo "\nPrep. SQL:\n".$this->sql;                
                error_log("SQL Fehler bei prepare: \n".$this->sql);
            }
            else{
                echo "\nPrep. SQL:\n".substr($this->sql,0,500);
                error_log("SQL Fehler bei prepare: \n".substr($this->sql,0,500));
            }
            error_log(print_r($this->dbsql->errorInfo(),true));
            $ex = new Exception();
            error_log($ex->getTraceAsString());
            return false;
        }
        $sqlErg = $stmt->execute($this->param);
        if ($sqlErg === false){
            $this->lastError = $stmt->errorInfo();
            if ($showError) http_response_code(425);            
            if ($showError) echo $stmt->errorInfo()[2];
            if (strlen($this->sql) <= 500){
                error_log("SQL Fehler bei execute: \n".$this->sql);
                if ($showError) echo "\nExecute SQL:\n".$this->sql;
                $ex = new Exception();
                error_log($ex->getTraceAsString());
            }
            else {
                error_log("SQL Fehler bei execute: \n".substr($this->sql,0,500));
                if ($showError) echo "\nExecute SQL:\n".substr($this->sql,0,500);
                $ex = new Exception();
                error_log($ex->getTraceAsString());
            }
            error_log(print_r($stmt->errorInfo(),true));
            
            return false;
        }
        return $stmt;
    }
    
    function errorInfo(){
        return $this->lastError;
    }
    
    function addWhere($klausel){
        if (! isValid($klausel)) return;
        if ($this->firstWhere) $this->sql.="\nwhere (".$klausel.')';
        else $this->sql.="\n and (".$klausel.')';
        $this->firstWhere = false;
    }
    
    function klammerWeg(){
        if (substr($this->sql,strlen($this->sql)-1) == ')') 
            $this->sql = substr($this->sql, 0,strlen($this->sql)-1);
    }
    function unionWeg(){
        if (strlen(trim($this->sql))< 5) return;
        $this->sql = trim($this->sql);
        if (substr($this->sql,strlen($this->sql)-5) == 'UNION')
            $this->sql = substr($this->sql, 0,strlen($this->sql)-5);
    }

    function addWert($wert){
        global $sourceSQL;
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            else $this->sql.= ' '.$wert.' ';
        }else{
            $this->sql.= ' :filterPar'.$pn.' ';
        }
        $this->param['filterPar'.$pn] = $wert;
    }
    
    function addKlausel($klausel,$wert){
        global $sourceSQL;
        if (! isValid($klausel)) return;
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            else $this->sql.="\n ".$klausel.' '.$wert;
        }else{
            $this->sql.="\n".$klausel.' :filterPar'.$pn;
        }
        $this->param['filterPar'.$pn] = $wert;
    }
    
    
    
    function addSel($klausel,$wert){
        global $sourceSQL;
        if (! isValid($klausel)) return;
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            if ($this->firstWhere) $this->sql.="\nwhere (".$klausel.' '.$wert.')';
            else $this->sql.="\n and (".$klausel.' '.$wert.')';            
        }else{
            if ($this->firstWhere) $this->sql.="\nwhere (".$klausel.' :filterPar'.$pn.')';
            else $this->sql.="\n and (".$klausel.' :filterPar'.$pn.')';
        }
        $this->firstWhere = false;
        $this->param['filterPar'.$pn] = $wert;        
    }

    function startAddSel($klausel,$wert){
        global $sourceSQL;
        if (! isValid($klausel)) return;
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            if ($this->firstWhere) $this->sql.="\nwhere (".$klausel.$wert;
            else $this->sql.="\n and (".$klausel.$wert;
        }else{
            if ($this->firstWhere) $this->sql.="\nwhere (".$klausel.' :filterPar'.$pn;
            else $this->sql.="\n and (".$klausel.' :filterPar'.$pn;
        }
        $this->firstWhere = false;
        $this->param['filterPar'.$pn] = $wert;
    }
    
    function endAddSel($klausel,$wert){        
        global $sourceSQL;
        if ($this->firstWhere) return; // FALSCHE REIHENFOLGE
        if (! isValid($klausel)) {
            $this->sql.=')';
            return;
        }
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if ($wert == null){
                $this->sql.=" ".$klausel.')';
                return;
            }
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            $this->sql.=" ".$klausel.' '.$wert.')';
        }else{
            if ($wert == null){
                $this->sql.=" ".$klausel.')';
                return;
            }
            
            $this->sql.=" ".$klausel.' :filterPar'.$pn.')';
        }
        $this->param['filterPar'.$pn] = $wert;
    }
    
    function addAddSel($klausel,$wert){
        global $sourceSQL;
        if ($this->firstWhere) return; // FALSCHE REIHENFOLGE
        if (! isValid($klausel)) return;
        $pn = count($this->param);
        if (isset($sourceSQL) && $sourceSQL){
            if (!is_numeric($wert)) $wert = '"'.str_replace('"', '\"', $wert).'"';
            $this->sql.=" ".$klausel.' '.$wert;
        }else{
            $this->sql.=" ".$klausel.' :filterPar'.$pn;
        }
        $this->param['filterPar'.$pn] = $wert;
    }
    
    
    
    function union(){
        if (strlen($this->sql)<2) return;
        $this->sql = trim($this->sql);
        if (substr($this->sql,strlen($this->sql)-5) != 'UNION')
                $this->sql.="\n\nUNION\n";
        $this->firstWhere = true;
    }
    
/**
 * Ergänzt die SQL um einen parametriserten Where-Teil
 * Dabei wird die aktuelle globale $query verwendet
 * @param string $filterVar
 * @param RepFilter $filterVals
 * @param JoinedTable $joinTable
 */
    function addFilterSQL ($filterVar, $filterVals, $joinTable) {
        global $query;
        $this->addFilterSQLForQuery ($query, $filterVar, $filterVals, $joinTable) ;
    }
    
    /**
     * Ergänzt die SQL um einen parametriserten Where-Teil
     * @param Query $query
     * @param string $filterVar
     * @param RepFilter $filterVals
     * @param JoinedTable $joinTable
     */
    function addFilterSQLForQuery ($query, $filterVar, $filterVals, $joinTable) {
        global $meldungen, $testMode;
        ;
//        error_log("addFilterSQL ".$filterVar.' = '.print_r($filterVals,true));
        $this->sql.="\n-- Filter $filterVar:";
        $mr = new MetaRecord();
        $qu = $query;
        // Zum Testen wird die Where-Klausel auf FALSE gesetzt
        if ($filterVar == "false") {
            $this->addWhere(" false ");
            return;
        }
        $mr = $query->getReportMetaRecord($filterVar);
        if ($mr == null){
            error_log("FEHLER: Filter mit Name ".$filterVar.' nicht gefunden.');
            return;
        }
        
        // Eine Werte-Liste ist kein Filter
        if ($mr->filterType == MetaRecord::WERTELISTE) return;
        
        $selectVar = $mr->selectVar;
        /*
         * Sonderfilter gibt es nur bei createBasicTable. Deshalb ist hier der $table-Parameter
         * aus dem Aufruf irrelevant.
         */
        if ($mr->filterType == MetaRecord::SONDERFILTER) {
            $this->addWhere($selectVar);
            return;
        }

        if ($mr->filterType == MetaRecord::WINDOWFILTER) {
// nicht in Basis-Abfragen            $this->addWhere($selectVar);
            return;
        }
        
        
        if ($mr->filterType == MetaRecord::LIST) {
            $fVals = $filterVals->fs_list;
            if ($fVals == null) return;
            if (count($fVals) == 0) return;
            if ($filterVals->ausschlussListe) $this->startAddSel($mr->selectVar.' not in (', $fVals[0]);
            else $this->startAddSel($mr->selectVar.' in (', $fVals[0]);
            for ($i=1; $i < count($fVals);$i++){
                $this->addAddSel(' , ', $fVals[$i]);
            }
            $this->endAddSel(' )', null);
            return;
        }
        
        if (!$filterVals->isSet()) return ;
//        if (!isValid($filterVals->fs_von) && !isValid($filterVals->fs_bis) ) return ;
        
        
        if ($mr->filterType == MetaRecord::SONDERFILTERSEL) {
            if (! isValid($filterVals->fs_von)) return;
            $so = $mr->getSelectOptions();
            for ($s=0; $s < count($so); $s++){
                if ($testMode) $this->addWhere($so[$s][1]);
                else if ($so[$s][0] == $filterVals->fs_von) {
                    $this->addWhere($so[$s][1]);
                    return;
                }
            }
            if ($testMode) return;
            $meldungen.="\n<br>$filterVar : Filter-Option ".$filterVals->fs_von." nicht gefunden.";
            return;
        }
if (false) $mr = new MetaRecord();
$hsqd = $query->historyQueryDate;
if ($mr->dimNr > 0){
//  error_log('Filter '.$mr->getName().' auf Dimension '.$mr->dimNr);   
}
if (false) $joinTable = new JoinedTable('');
        if ($mr->filterType == MetaRecord::HISTDEL) {
//            error_log('HistDel '.$mr->getName().' mit '.$filterVals->fs_von);
            if ($testMode || $filterVals->fs_von == HistoryOptions::HSDELAKT){
                if ($joinTable != null){
                    if ($mr->dimNr > 0){
                        $this->addWhere('( ('.$mr->selectVar. ') is null or ('.$mr->selectVar. ') > '.$joinTable->dimHistoryQueryDate[$mr->dimNr].' )');
                    }else{
                        $this->addWhere('( ('.$mr->selectVar. ') is null or ('.$mr->selectVar. ') > '.$joinTable->dimHistoryQueryDate[0].' )');
                    }
                }else{
                    if ($mr->dimNr > 0){
                        $this->addWhere('( ('.$mr->selectVar. ') is null or ('.$mr->selectVar. ') > '.$query->dimensions[$mr->dimNr]->historyQueryDate. ' )');
                    }else{
                        $this->addWhere('( ('.$mr->selectVar. ') is null or ('.$mr->selectVar. ') > '.$query->historyQueryDate. ' )');                    
                    }
                }
            }
            if ($testMode || $filterVals->fs_von == HistoryOptions::HSDELBIS){
                if ($joinTable != null){
                    if ($mr->dimNr > 0){
                        $this->addWhere('('.$mr->selectVar. ') <= '.$joinTable->dimHistoryQueryDate[$mr->dimNr]);
                    }else{
                        $this->addWhere('('.$mr->selectVar. ') <= '.$joinTable->dimHistoryQueryDate[0]);
                    }
                }else{
                    if ($mr->dimNr > 0){
                        $this->addWhere('('.$mr->selectVar. ') <= '.$query->dimensions[$mr->dimNr]->historyQueryDate);
                    }else{
                        $this->addWhere('('.$mr->selectVar. ') <= '.$query->historyQueryDate);
                        
                    }
                }
            }
            if ($testMode || $filterVals->fs_von == HistoryOptions::HSDELAKTJB){
                if ($joinTable != null){
                    if ($mr->dimNr > 0){
                        $this->addWhere('( ('.$mr->selectVar. ') is null or year('.$mr->selectVar. ') >= year('.$joinTable->dimHistoryQueryDate[$mr->dimNr].') )');
                    }else{
                        $this->addWhere('( ('.$mr->selectVar. ') is null or year('.$mr->selectVar. ') >= year('.$joinTable->dimHistoryQueryDate[0].') )');
                    }
                }else{
                    if ($mr->dimNr > 0){
                        $this->addWhere('( ('.$mr->selectVar. ') is null or year('.$mr->selectVar. ') >= year('.$query->dimensions[$mr->dimNr]->historyQueryDate. ') )');
                    }else{
                        $this->addWhere('( ('.$mr->selectVar. ') is null or year('.$mr->selectVar. ') >= year('.$query->historyQueryDate. ') )');
                    }
                }
            }
            if ($testMode || $filterVals->fs_von == HistoryOptions::HSDELBISJB){
                if ($joinTable != null){
                    if ($mr->dimNr > 0){
                        $this->addWhere('year('.$mr->selectVar. ') < year('.$joinTable->dimHistoryQueryDate[$mr->dimNr].')');
                    }else{
                        $this->addWhere('year('.$mr->selectVar. ') < year('.$joinTable->dimHistoryQueryDate[0].')');
                    }
                }else{
                    if ($mr->dimNr > 0){
                        $this->addWhere('year('.$mr->selectVar. ') < year('.$query->dimensions[$mr->dimNr]->historyQueryDate.')');
                    }else{
                        $this->addWhere('year('.$mr->selectVar. ') < year('.$query->historyQueryDate.')');
                        
                    }
                }
            }
            return;
        }
        
        
        /**
         * Filter auf verknüpfte Tabellen
         */
        if (substr($filterVar, 1,1) == ':' || substr($filterVar, 1,1) == '='){
            $jt = $query->repTables->get(substr($filterVar, 0,1));
            if ($jt == null){
                $meldungen .="FEHLER (addFilterSQL): Join-Tabelle für $filterVar nicht gefunden.<br>";
                $this->addWhere(" false \n-- FEHLER (addFilterSQL): Join-Tabelle für $filterVar nicht gefunden.\n");
                return;
            }
            if ($jt->query == null){
                $meldungen .="FEHLER (addFilterSQL): Query der Join-Tabelle für $filterVar ist NULL.<br>";
                $this->addWhere(" false \n-- FEHLER (addFilterSQL): Query der Join-Tabelle für $filterVar ist NULL.\n");
                return;
            }
            $qu = $jt->query;
        
            /**
             * Filter auf verknüpfte Berechnung
             * Dazu muss die Berechnung aus repValues geholt werden, weil nur dort die Basis-Tabelle für die Berechnung steht
             */
            if (substr($filterVar, 1,1) == '='){
                $repValue = $query->repValues->get($filterVar);
                if ($repValue == null){
                    error_log ("SqlAbfrage->addFilterSql() verknüpfter Filter $filterVar nicht in repValues\n");
                    error_log (print_r($query->repValues,true));
                }
                $f = str_replace('join.',$jt->kennzeichen.'.', $mr->selectVar);
                if ($repValue->base != 'base') $f = str_replace('base.',$repValue->base.'.', $f);
                $selectVar = '('.$f.')';
            }else
                /**
                 * Value-Filter bei verknüpfter Tabelle
                 */
                if (array_search(substr($filterVar,2), $qu->valueVars) !== false){
                    $selectVar = $jt->kennzeichen.'.'.$mr->getName();
                 }
            
        }else 
        /**
         * Value-Filter bei base-table
         */
        if (array_search($filterVar, $query->valueVars) !== false){
            $selectVar = 'base.'.$mr->getName();         
        }
        
        //        $mr = $qu->metaData[$filterVar];
        if ($mr == null){
            $meldungen .="FEHLER (addFilterSQL): Keine Metadaten für $filterVar .<br>";
            $this->addWhere(" false \n-- FEHLER (addFilterSQL): Keine Metadaten für $filterVar .\n");
            return ;
        }
        
        
        $val1 = null;
        $val2 = null;
        if (isValid($filterVals->fs_von)) $val1 = $filterVals->fs_von;
        if (isValid($filterVals->fs_bis)) $val2 = $filterVals->fs_bis;
        
        /**
         * Werte für Filter in $val1 und $val2 aufbereiten
         */
        $notFilter = false;
        switch($mr->type) {
// TRANSLATION FEHLT
            case MetaRecord::INT:
            case MetaRecord::DECIMAL:
            case MetaRecord::GELD:
                    if (isValid($val1) && strpos($val1, '(') === false) $val1 = (float) $val1;
                    if (isValid($val2) && strpos($val2, '(') === false) $val2 = (float) $val2;
                    break;
            case MetaRecord::STRING:
            case MetaRecord::WEBLINK:
                if (isValid($val1)){
                    if (substr($val1,0,1) == '!'){
                        $val1 = substr($val1,1);
                        $notFilter = true;
                    }else
                        if (substr($val1,0,2) == '\!'){
                            $val1 = substr($val1,1);
                    }
                }
                if ($mr->filterType == MetaRecord::TEXT) $val1 = "%".$val1."%";
                else if ($mr->filterType == MetaRecord::TEXTEXAKT) ;
                else if ($mr->filterType == MetaRecord::TEXTLIKE) ;
                else {
                    if (isValid($val1)) $val1 = trim($val1) ;
                    if (isValid($val2)) $val2 = trim($val2) ;
                }
            default: 
                
                break;
        }
        
        
        /**
         * Filter anwenden
         * in $selectVar steht der Ausdruck, auf den der Filter angewendet wird.
         * Also je nach Situation
         * $mr->selectVar
         * $mr->getName()
         * $table.".".$mr->getName()
         * oder die Aufbereitung aus der verknüpften Berechnung
         */
        switch($mr->filterType) {
            case MetaRecord::VON_BIS:
            case MetaRecord::VON_BIS_SELECT:
                if ($mr->type == MetaRecord::HISTDEL){
                    if (isValid($val1)) {
                        if (isValid($val2)) {
                            $this->addSel($selectVar." <= ",$val2);
                            $this->addSel($selectVar." >= ",$val1);
                        }else{
                            $this->addSel($selectVar." is null or ".$selectVar." >= ",$val1);
                        }                        
                    }else{
                        if (isValid($val2)) $this->addSel($selectVar." is null or ".$selectVar." <= ",$val2);
                    }                    
                }else{
                    if (isValid($val1)) {
                        $this->addSel($selectVar." >= ",$val1);
                        if (isValid($val2)) $this->addSel($selectVar." <= ",$val2);
                    }else{
                        if (isValid($val2)) $this->addSel($selectVar." is null or ".$selectVar." <= ",$val2);
                    }
                }
                break;
            case MetaRecord::TEXT:
            case MetaRecord::TEXTLIKE:
                if (!isValid($val1)) return;
                if ($notFilter){
                    if ($mr->type == MetaRecord::STRING
                        || $mr->filterType == MetaRecord::WEBLINK)
                        $this->addSel($selectVar .' not like ',$val1);
                        else $this->addSel($selectVar . " != " ,$val1);
                }else{
                    if ($mr->type == MetaRecord::STRING
                        || $mr->filterType == MetaRecord::WEBLINK)
                        $this->addSel($selectVar .' like ',$val1);
                        else $this->addSel($selectVar . " = " ,$val1);
                }
                    break;
            case MetaRecord::TEXTEXAKT:
                if (!isValid($val1)) return;
                if ($notFilter){
                    $this->addSel($selectVar . " != " ,$val1);
                }else{
                    $this->addSel($selectVar . " = " ,$val1);
                }
                    break;
            case MetaRecord::TEXTMITLABEL:
                if (!isValid($val1)) return;
                if ($notFilter){
                    $this->addSel($selectVar . " != " ,$val1);
                }else{
                    $this->addSel($selectVar . " = " ,$val1);
                }
                break;
            case MetaRecord::SELECT:
                if (!isValid($val1)) return;
                    $this->addSel($selectVar . " = " ,$val1);
                break;
            default: break;
        }
        return ;
    }
}

/*
Class RepTable {
  // ersetzt durch JoinedTable  
}
*/

?>
