<?php
/**
 * Projekt: WS-Reporter
 *          Testen der Spezifikation einer Datenquelle
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    22.02.2023 MS
 *
 */

require_once './phpsec/createDBH.inc';
require_once './includes/createReport.inc';
require_once './includes/wsrTranslate.inc';
global $query,  $testMode, $sql;

function testFehler($wo){
    global $meldungen, $sql;
    if (strlen($meldungen) > 0){
        http_response_code(425);
        echo WSRTranslate::translate('Test nicht bestanden')."\n".$wo."\n".
            str_replace('<b>',"",str_replace('</b>',"",str_replace('<br>',"\n",str_replace('</br>',"\n", $meldungen))))."\n";
//        echo $sql->sql;
        die();
        return true;
    }
    return false;
}

function addFilter($v) {
    global $query, $meldungen;
    $mr = $query->getReportMetaRecord($v->getName());
    if ($mr == null || $mr->filterType == MetaRecord::NONE) return;
    $von = 99;
    switch ($mr->type){
        case MetaRecord::DATE: $von="2020-01-01";
        break;
        case MetaRecord::DATETIME: 
        case MetaRecord::DATETIMES:
            $von="2020-01-01 12:12";
        break;
        case MetaRecord::TIME: 
        case MetaRecord::TIMES:
            $von="12:12";
        break;
        case MetaRecord::STRING: 
        case MetaRecord::WEBLINK:
            $von="XX";
        break;
        case MetaRecord::HISTDEL: 
            if ($mr->filterType == MetaRecord::HISTDEL) $von = ''.HistoryOptions::HSDELAKT;
            else  $von="2020-01-01";
        break;
        default: $von = 1;
        break;
    }
    switch ($mr->filterType){
        case MetaRecord::VON_BIS:
            $v->fs_bis = $von;
        case MetaRecord::TEXT:
        case MetaRecord::TEXTEXAKT:
        case MetaRecord::TEXTMITLABEL:
            $v->fs_von = $von;
            break;
        case MetaRecord::SELECT:
        case MetaRecord::VON_BIS_SELECT:
        case MetaRecord::SONDERFILTERSEL:
            $so = $mr->getSelectOptions();
            if (count($so) < 1) $meldungen .= "\n".WSRTranslate::translate('Keine Auswahloptionen für Filter').' '.$v->getName();
            else if (count($so[0]) < 1) $meldungen .= "\n".WSRTranslate::translate('Keine Werte bei erster Auswahloption für Filter').' '.$v->getName();
            else {
                $v->fs_von = $mr->getSelectOptions()[0][0];
                if (!isValid($v->fs_von) && count($mr->getSelectOptions()) > 1)
                    $v->fs_von = $mr->getSelectOptions()[1][0];
                if ($mr->filterType == MetaRecord::VON_BIS_SELECT && count($mr->getSelectOptions()) > 2){
                    $v->fs_bis = $mr->getSelectOptions()[2][0];
                }
            }
            break;
        case MetaRecord::HISTDEL:
            $v->fs_von = $von;
            break;
            
    }
    $query->repFilter->put($v);
//    $meldungen .= "Filter ".$v->getName().' = '.$v->fs_von."\n";
}


$meldungen = '';
$testMode = true;
if ($query->histType == Query::VONBIS 
    && ! (isValid($query->histAb) && isValid($query->histBis))){
//        $t = Query::getHistText($query->histType)
    $meldungen = WSRTranslate::translate('Fehler bei Historisierung').' '.Query::getHistText($query->histType).":\n"
        .WSRTranslate::translate('Werte für Gültigkeitsintervalle fehlen.')."\n"
            .WSRTranslate::translate('Die Datenquelle kann nicht gespeichert werden.')."\n";
   http_response_code(425);   
}
if (($query->histType == Query::STICHTAG || $query->histType == Query::STICHTAGE)
    && ! isValid($query->histStichtag)){
        //        $t = Query::getHistText($query->histType)
        $meldungen = WSRTranslate::translate('Warnung für Historisierung').' '.Query::getHistText($query->histType).":\n"
            .WSRTranslate::translate('Wert für Stichtag fehlt.')."\n"
            . WSRTranslate::translate('Es können nur explizite Datumswerte verwendet werden.')."\n";
//        http_response_code(425);
}

$query->newReport();
for ($i=0; $i < count($query->rowVars); $i++){
    $v = new RepRow($query->rowVars[$i]);
    $v->summenZeilen = true;
    $query->repRows->put($v);
    addFilter(new RepFilter($query->rowVars[$i]));
}
for ($i=0; $i < count($query->valueVars); $i++){
    $v = new RepValue($query->valueVars[$i]);
    $query->repValues->put($v);
}

for ($i=0; $i < count($query->filterVars); $i++){
    $v = new RepFilter($query->filterVars[$i]);
    addFilter($v);
}
if (strlen($meldungen)>0){
    echo $meldungen."\n ".WSRTranslate::translate('bei Vorbereitung der Base-Table')."\n";
    $meldungen = '';
}


/** Test mit leerem Filter */
for ($a=0; $a < $query->addQueries->size(); $a++){
    $aq = $query->addQueries->getPos($a);
    $aq->clearActiveQuery();
    $jt = new JoinedTable($aq);
    $k = $query->getKennzeichen();
    $jt->kennzeichen = $k;
    $query->repTables->put($jt);

    $jQuery = $jt->query;
    if ($jQuery->phys_table == null){
        http_response_code(425);
        echo WSRTranslate::translate('Fehler beim Lesen der verknüpften Datenquelle')
        ."\n{$jQuery->class_table}\n".WSRTranslate::translate('Diese Datenquelle wurde nicht gefunden.');
        die();
    }
    
    for ($i=0; $i < count($aq->calcVars); $i++){
        $v = new RepValue($k.'='.$aq->calcVars[$i]);
        $query->repValues->put($v);
    }
    for ($i=0; $i < count($jQuery->rowVars); $i++){
        $vn = $k.':'.$jQuery->rowVars[$i];
        $mr = $query->getReportMetaRecord($vn);
        if ($mr != null && $mr->filterType != MetaRecord::NONE)
            $query->repFilter->put(new RepFilter($vn));
    }
    for ($i=0; $i < count($jQuery->filterVars); $i++){
        $v = new RepFilter($k.':'.$jQuery->filterVars[$i]);
        $query->repFilter->put($v);
    }
    /*
    for ($i=0; $i < count($jQuery->valueVars); $i++){
        $v = new RepValue($k.':'.$jQuery->valueVars[$i]);
        $query->repValues->put($v);
    }
    for ($i=0; $i < count($jQuery->filterVars); $i++){
        $v = new RepFilter($k.':'.$jQuery->filterVars[$i]);
        $query->repFilter->put($v);        
    }
    for ($i=0; $i < count($jQuery->rowVars); $i++){
        $vn = $k.':'.$jQuery->rowVars[$i];
        $mr = $query->getReportMetaRecord($vn);
        if ($mr != null && $mr->filterType != MetaRecord::NONE)
            $query->repFilter->put(new RepFilter($vn));
    }
    */
}

/** Test mit gesetztem Filter */
for ($a=0; $a < $query->addQueries->size(); $a++){
    $aq = $query->addQueries->getPos($a);
    $jt = new JoinedTable($aq);
    $k = $query->getKennzeichen();
    $jt->kennzeichen = $k;
    $query->repTables->put($jt);
    
    $jQuery = $jt->query;
error_log("Table $k hat hist-type {$jQuery->histType} und hist-option {$jQuery->historyOption}");
    for ($i=0; $i < count($jQuery->filterVars); $i++){
        $v = new RepFilter($k.':'.$jQuery->filterVars[$i]);
        addFilter($v);
    }
    for ($i=0; $i < count($jQuery->rowVars); $i++){
        $vn = $k.':'.$jQuery->rowVars[$i];
        $mr = $query->getReportMetaRecord($vn);
        if ($mr != null && $mr->filterType != MetaRecord::NONE)
            addFilter(new RepFilter($vn));
    }
    for ($i=0; $i < count($aq->calcVars); $i++){
        $v = new RepValue($k.'='.$aq->calcVars[$i]);
        $query->repValues->put($v);
    }
/*    
    for ($i=0; $i < count($jQuery->valueVars); $i++){
        $v = new RepValue($k.':'.$jQuery->valueVars[$i]);
        $query->repValues->put($v);
    }
    for ($i=0; $i < count($jQuery->filterVars); $i++){
        $v = new RepFilter($k.':'.$jQuery->filterVars[$i]);
        addFilter($v);
    }
    for ($i=0; $i < count($jQuery->rowVars); $i++){
        $vn = $k.':'.$jQuery->rowVars[$i];
        $mr = $query->getReportMetaRecord($vn);
        if ($mr != null && $mr->filterType != MetaRecord::NONE)
            addFilter(new RepFilter($vn));
    }
    */
    if (strlen($meldungen)>0){
        $meldungen = str_ireplace('<b>', '', $meldungen);
        $meldungen = str_ireplace('</b>', '', $meldungen);
        $meldungen = str_ireplace('<br>', "\n", $meldungen);
        echo $meldungen."\n ".WSRTranslate::translate('bei Vorbereitung der Tabelle').' '
            .$k.'('.$query->addQueries->getPos($a)->getQuery()->class_table.")\n";
        $meldungen = '';
    }
}

createHistory();
if (isValid($meldungen)){
    if (! isset($testSQL)) {        
        $meldungen = str_ireplace('<b>', '', $meldungen);
        $meldungen = str_ireplace('</b>', '', $meldungen);
        $meldungen = str_ireplace('<br>', "\n", $meldungen);
    }
    echo $meldungen;
    $meldungen = '';
}
    
    
$sql= new SqlAbfrage('DWH');
createBasicQuery($sql,'base', -1, -1);
testFehler(WSRTranslate::translate('Erstellen der SQL für Basic Query'));
if (isset($testSQL)) {
    echo "-- ".WSRTranslate::translate('Erstellen der SQL für Basic Query')."<pre>\n";
    echo $sql->sql;
    echo "\n</pre>";
}else{
if (! createTable('base', $sql)) $meldungen .="\n";
testFehler(WSRTranslate::translate('Erstellen der Basic Table'));
echo WSRTranslate::translate('Tabelle base ist fertig.')."\n";    
}

for ($i = 0; $i < $query->repTables->size(); $i++){
    $k = $query->repTables->getPos($i)->kennzeichen;
    echo $query->repTables->getPos($i)->query->class_table.' als '.$k."\n";
    $sql = new SqlAbfrage('DWH'); 
    createBasicQuery($sql,$k,-1, -1);  
    testFehler("-- Erstellen der SQL für $k ");
    if (isset($testSQL)) {
        echo WSRTranslate::translate('Erstellen der SQL für')." $k<pre>\n";
        echo $sql->sql;;
        echo "\n</pre>";
    }else{
        if (!createTable($k, $sql)) $meldungen .="\n";
        testFehler(WSRTranslate::translate('Erstellen der Joined Table')." $k ");
        echo WSRTranslate::translate('Tabelle').' '.$k.' '.WSRTranslate::translate('ist fertig.')."\n";
    }
}

$sql = new SqlAbfrage('DWH');
createLeadTable($sql);
testFehler(WSRTranslate::translate('Erstellen der SQL für Zeilen und Spalten'));
if (isset($testSQL)) {
    echo "-- ".WSRTranslate::translate('Erstellen der SQL für Zeilen und Spalten')."<pre>\n";
    echo $sql->sql;
    echo "\n</pre>";
}else{
    if (!createTable('lead', $sql)) $meldungen .="\n";
    testFehler(WSRTranslate::translate('Erstellen der  Zeilen und Spalten'));
    echo WSRTranslate::translate('Tabelle Zeilen und Spalten ist fertig.')."\n";
}


$sql = new SqlAbfrage('DWH');
createJoinedTable($sql);
testFehler(WSRTranslate::translate('Erstellen der SQL für die Ergebnis-Tabelle'));
if (isset($testSQL)) {
    echo "-- ".WSRTranslate::translate('Erstellen der SQL für die Ergebnis-Tabelle')."<pre>\n";
    echo $sql->sql;
    echo "\n</pre>";
}else{
    if (!createTable('ergebnis', $sql)) $meldungen .="\n"; 
    testFehler(WSRTranslate::translate('Erstellen der Ergebnis-Tabelle'));
    echo WSRTranslate::translate('Tabelle ergebnis ist fertig.')."\n";
   
// Und dasselbe nochmal mit Filter in den Joined Tables



    echo WSRTranslate::translate('Test bestanden');
}
    