<?php
/**
 * Projekt: WS-Reporter
 *          Speichern der Spezifikation einer Datenquelle
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    21.02.2023
 *
 */
global $dbh, $query;
if ($query->qLanguage != $query->currentLanguage){
    http_response_code(425);
    echo "FEHLER: Diese Datenquelle kann nur in ".Query::SPRACHEN[$query->qLanguage]." gespeichert werden.\n";
    echo "Aktuell: ".Query::SPRACHEN[$query->currentLanguage];
    die;
}

adminRechteVerlangen();

require_once './phpsec/createDBH.inc';

// Select-Options in MetaDaten einlesen
$class_table = $query->class_table;

if ($query->histType == Query::VONBIS
    && ! (isValid($query->histAb) && isValid($query->histBis))){
        //        $t = Query::getHistText($query->histType)
        http_response_code(425);
        echo "Fehler bei Historisierung {Query::getHistText($query->histType)}:\n"
        ."Werte für Gültigkeitsintervalle fehlen.\nDatenquelle wird nicht gespeichert.";
        die();
}
/* KEIN FEHLER
if ($query->histType == Query::STICHTAG  || $query->histType == Query::STICHTAGE
    && ! isValid($query->histStichtag)){
        //        $t = Query::getHistText($query->histType)
        http_response_code(425);
        echo "Fehler bei Historisierung {Query::getHistText($query->histType)}:\n"
        ."Wert für Stichtag fehlt.\nDatenquelle wird nicht gespeichert.";
        die();
}
*/


foreach ($query->metaData as $metaRecord){
    if ( !isValid($metaRecord->selectFilterTable)) $metaRecord->getSelectOptions();
    $metaRecord->setTable($class_table);
//    $class_table = "ET_".$class_table;   
}

$isOld = false;
$msg = "";       
        $sql = new SqlAbfrage('WSR');
        $sql->sql = "select * from queryAttribute";
        $sql->addSel('table_name = ', $class_table);
        $res1 = $sql->execute(); 
       
        if ($res1->fetchObject()) $isOld = true;
        $res1->closeCursor();
        if ($isOld) {
            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryAttribute";
            $sql->addSel('table_name = ', $class_table);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryAttribute';
                die();
            }
            else $msg.="Lösche queryAttribute where table_name=\"".$class_table."\"\n";
            
            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryColumnMetadata";
            $sql->addSel('table_name = ', $class_table);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryColumnMetadata';
                die();
            }
            else $msg.="Lösche queryColumnMetadata where table_name=\"".$class_table."\"\n";

            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryColumnMetadataSelectOptions";
            $sql->addSel('table_name = ', $class_table);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryColumnMetadataSelectOptions';
                die();
            }
            else $msg.="Lösche queryColumnMetadataSelectOptions where table_name=\"".$class_table."\"\n";

            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryDimensions";
            $sql->addSel('table_name = ', $class_table);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryDimensions';
                die();
            }
            else $msg.="Lösche queryDimensions where table_name=\"".$class_table."\"\n";
            
            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryDefaultFilter";
            $sql->addSel('table_name = ', $class_table);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryDefaultFilter';
                die();
            }
            else $msg.="Lösche queryDefaultFilter where table_name=\"".$class_table."\"\n";

            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryComment";
            $sql->addSel('table_name = ', $class_table);
            if (! $query->queryImported)
                $sql->addSel('qLanguage = ', $query->qLanguage);
            if (false === $sql->execute()){
                echo 'Fehler bei delete from queryComment';
                die();
            }
            else $msg.="Lösche queryComment where table_name=\"".$class_table."\"\n";

            $sql = new SqlAbfrage('WSR');
            $sql->sql = "delete from queryHints";
            $sql->addSel('table_name = ', $class_table);
            if (! $query->queryImported)
                $sql->addSel('qLanguage = ', $query->qLanguage);
                if (false === $sql->execute()){
                    echo 'Fehler bei delete from queryHints';
                    die();
                }
                else $msg.="Lösche queryHints where table_name=\"".$class_table."\"\n";
                
            
        }else{
            $msg.="Keine Löschungen";
            // Log für Neuen Eintrag
            $logSql = new SqlAbfrage('WSR');
            $logSql->sql = 'insert into wsrAdminQuery set timeCreated = sysdate()';
            $logSql->addKlausel(', userCreated = ', $_SESSION['Name']);
            $logSql->addKlausel(', table_name = ', $query->class_table);
            $erg = $logSql->executeE(false);
            if ($erg === false || $erg->rowCount() < 1 ){
                error_log('Fehler beim Insert wsrAdminQuery: '. $logSql->errorInfo()[2]);
            }
        }
        
        $p1 = $dbh->prepare("insert into queryAttribute set table_name=:class_table,"
            ."SatzTyp=:SatzTyp,attribute=:attribute");
        $a  = array('class_table' => $class_table,
            'SatzTyp' => 1, 'attribute' => wsrDecode($query->defaultTitle ) );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-1';
            echo $msg;
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        
        $a = array('class_table' => $class_table,
            'SatzTyp' => 2, 'attribute' => wsrDecode($query->outputTitlePrefix ) );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-2';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        $a = array('class_table' => $class_table,
            'SatzTyp' => 3, 'attribute' => $query->reportCSS  );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-3';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        $a = array('class_table' => $class_table,
            'SatzTyp' => 4, 'attribute' => $query->phys_table  );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-4';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        $a = array('class_table' => $class_table,
            'SatzTyp' => 7, 'attribute' => $query->dwh );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-7';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        $a = array('class_table' => $class_table,
            'SatzTyp' => 10, 'attribute' => ''.$query->queryType  );
        if (false == $p1->execute($a)){
            http_response_code(425);
            echo 'Fehler bei p1-10';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
                
        if ($query->histType > 0){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 11, 'attribute' => ''.$query->histType  );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-11';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }            
        }

        if ($query->histType > 0 && $query->historyOption > 0){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 12, 'attribute' => ''.$query->historyOption );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-12';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
            if ($query->historyOption == HistoryOptions::HSDATUM){
                $a = array('class_table' => $class_table,
                    'SatzTyp' => 17, 'attribute' => ''.$query->historyDate );
                if (false == $p1->execute($a)){
                    http_response_code(425);
                    echo 'Fehler bei p1-17';
                    echo var_dump($p1->errorInfo());
                    echo var_dump($a);
                    die();
                }
                
            }
        }
        if ($query->histType == Query::VONBIS && isValid($query->histAb)){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 13, 'attribute' => $query->histAb  );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-13';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        if ($query->histType == Query::VONBIS && isValid($query->histBis)){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 14, 'attribute' => $query->histBis  );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-14';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        /* Stichtage kann es auch bei von-bis-Historisierung geben
        if (($query->histType == Query::STICHTAG 
            || $query->histType == Query::STICHTAGE)
            && */ 
        if (isValid($query->histStichtag)){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 15, 'attribute' => $query->histStichtag  );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-15';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }

        if ($query->reportHoverMode > 0){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 16, 'attribute' => ''.$query->reportHoverMode  );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-16';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        
        // Satztyp 17 ist Datum für History-Option = HSDATUM

        if ($query->hatWerteAuswahl){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 18, 'attribute' => 'hatWerteAuswahl' );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-18';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        
        if (isValid($query->qLanguage)){
            $qt = $query->qLanguage;
            foreach ($query->qTranslations as $tr) $qt.=' '.$tr;
            $a = array('class_table' => $class_table,
                'SatzTyp' => 19, 'attribute' => $qt );
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-19';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        
        // Table-Description (Kommentar) in qLanguage speichern
        if (isValid($query->table_description)){
            $sortNr = 0;
            $stmt = $dbh->prepare('insert into queryComment set table_name = :table_name,'
                .' SortNr = :SortNr, qLanguage = :qLanguage, kommentar = :kommentar');
            if ($stmt === false){
                http_response_code(425);
                echo "\nFehler bei prepare Insert Kommentar\n";
                echo "\n".$dbh->errorInfo()[2];
                error_log("Fehler bei prepare von Insert Kommentar\n");
                error_log($dbh->errorInfo()[2]);
                die();
            }
    
            $k = explode("\n",str_replace("\r", '', $query->table_description));
            foreach ($k as $kom) {
                $sortNr ++;
                while (strlen($kom) > 248){
                    $pars = array(
                        'table_name' => $query->class_table,
                        'SortNr' => $sortNr,
                        'qLanguage' => $query->qLanguage,
                        'kommentar' => substr($kom,0,248)
                    );
                    $kom = substr($kom,248);
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert Kommentar\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert Kommentar\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                    $sortNr ++;
                }
                $pars = array(
                    'table_name' => $query->class_table,
                    'SortNr' => $sortNr,
                    'qLanguage' => $query->qLanguage,
                    'kommentar' => $kom."\n"
                );
                if (false === $stmt->execute($pars)){
                    http_response_code(425);
                    echo "\nFehler bei execute von Insert Kommentar\n";
                    echo "\n".$stmt->errorInfo()[2];
                    error_log("Fehler bei execute von Insert Kommentar\n");
                    error_log($stmt->errorInfo()[2]);
                    error_log(print_r($pars,true));
                    die();
                }
            }
        }
        // ENDE Speichern des Kommentars
        
        if ($query->queryImported){
            foreach ($query->description_imported as $lang => $kommentar)
                if (isValid($kommentar)){
                    $sortNr = 0;
                    $stmt = $dbh->prepare('insert into queryComment set table_name = :table_name,'
                        .' SortNr = :SortNr, qLanguage = :qLanguage, kommentar = :kommentar');
                    if ($stmt === false){
                        http_response_code(425);
                        echo "\nFehler bei prepare Insert Kommentar\n";
                        echo "\n".$dbh->errorInfo()[2];
                        error_log("Fehler bei prepare von Insert Kommentar\n");
                        error_log($dbh->errorInfo()[2]);
                        die();
                    }
                    
                    $k = explode("\n",str_replace("\r", '', $kommentar));
                    foreach ($k as $kom) {
                        $sortNr ++;
                        while (strlen($kom) > 248){
                            $pars = array(
                                'table_name' => $query->class_table,
                                'SortNr' => $sortNr,
                                'qLanguage' => $lang,
                                'kommentar' => substr($kom,0,248)
                            );
                            $kom = substr($kom,248);
                            if (false === $stmt->execute($pars)){
                                http_response_code(425);
                                echo "\nFehler bei execute von Insert Kommentar\n";
                                echo "\n".$stmt->errorInfo()[2];
                                error_log("Fehler bei execute von Insert Kommentar\n");
                                error_log($stmt->errorInfo()[2]);
                                error_log(print_r($pars,true));
                                die();
                            }
                            $sortNr ++;
                        }
                        $pars = array(
                            'table_name' => $query->class_table,
                            'SortNr' => $sortNr,
                            'qLanguage' => $lang,
                            'kommentar' => $kom."\n"
                        );
                        if (false === $stmt->execute($pars)){
                            http_response_code(425);
                            echo "\nFehler bei execute von Insert Kommentar\n";
                            echo "\n".$stmt->errorInfo()[2];
                            error_log("Fehler bei execute von Insert Kommentar\n");
                            error_log($stmt->errorInfo()[2]);
                            error_log(print_r($pars,true));
                            die();
                        }
                    }
            }            
        }
        
        // Technische Hinweise eintragen in qLanguage speichern
        error_log('Save table_hints '.$query->table_hints);
        if (isValid($query->table_hints)){
            $sortNr = 0;
            $stmt = $dbh->prepare('insert into queryHints set table_name = :table_name,'
                .' SortNr = :SortNr, qLanguage = :qLanguage, kommentar = :kommentar');
            if ($stmt === false){
                http_response_code(425);
                echo "\nFehler bei prepare Insert Table-Hints\n";
                echo "\n".$dbh->errorInfo()[2];
                error_log("Fehler bei prepare von Insert Table-Hints\n");
                error_log($dbh->errorInfo()[2]);
                die();
            }
            
            $k = explode("\n",str_replace("\r", '', $query->table_hints));
            foreach ($k as $kom) {
                $sortNr ++;
                while (strlen($kom) > 248){
                    $pars = array(
                        'table_name' => $query->class_table,
                        'SortNr' => $sortNr,
                        'qLanguage' => $query->qLanguage,
                        'kommentar' => substr($kom,0,248)
                    );
                    $kom = substr($kom,248);
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert Table-Hints\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert Table-Hints\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                    $sortNr ++;
                }
                $pars = array(
                    'table_name' => $query->class_table,
                    'SortNr' => $sortNr,
                    'qLanguage' => $query->qLanguage,
                    'kommentar' => $kom."\n"
                );
                if (false === $stmt->execute($pars)){
                    http_response_code(425);
                    echo "\nFehler bei execute von Insert Table-Hints\n";
                    echo "\n".$stmt->errorInfo()[2];
                    error_log("Fehler bei execute von Insert Table-Hints\n");
                    error_log($stmt->errorInfo()[2]);
                    error_log(print_r($pars,true));
                    die();
                }
            }
        }
        // ENDE Speichern der Table-Hints
        
        if ($query->queryImported){
            foreach ($query->hints_imported as $lang => $kommentar)
                if (isValid($kommentar)){
                    $sortNr = 0;
                    $stmt = $dbh->prepare('insert into queryHints set table_name = :table_name,'
                        .' SortNr = :SortNr, qLanguage = :qLanguage, kommentar = :kommentar');
                    if ($stmt === false){
                        http_response_code(425);
                        echo "\nFehler bei prepare Insert Table-Hints\n";
                        echo "\n".$dbh->errorInfo()[2];
                        error_log("Fehler bei prepare von Insert Table-Hints\n");
                        error_log($dbh->errorInfo()[2]);
                        die();
                    }
                    
                    $k = explode("\n",str_replace("\r", '', $kommentar));
                    foreach ($k as $kom) {
                        $sortNr ++;
                        while (strlen($kom) > 248){
                            $pars = array(
                                'table_name' => $query->class_table,
                                'SortNr' => $sortNr,
                                'qLanguage' => $lang,
                                'kommentar' => substr($kom,0,248)
                            );
                            $kom = substr($kom,248);
                            if (false === $stmt->execute($pars)){
                                http_response_code(425);
                                echo "\nFehler bei execute von Insert Table-Hints\n";
                                echo "\n".$stmt->errorInfo()[2];
                                error_log("Fehler bei execute von Insert Table-Hints\n");
                                error_log($stmt->errorInfo()[2]);
                                error_log(print_r($pars,true));
                                die();
                            }
                            $sortNr ++;
                        }
                        $pars = array(
                            'table_name' => $query->class_table,
                            'SortNr' => $sortNr,
                            'qLanguage' => $lang,
                            'kommentar' => $kom."\n"
                        );
                        if (false === $stmt->execute($pars)){
                            http_response_code(425);
                            echo "\nFehler bei execute von Insert Table-Hints\n";
                            echo "\n".$stmt->errorInfo()[2];
                            error_log("Fehler bei execute von Insert Table-Hints\n");
                            error_log($stmt->errorInfo()[2]);
                            error_log(print_r($pars,true));
                            die();
                        }
                    }
            }
        }
        // ENDE Technische Hinweise eintragen

        
        // Version eintragen
        
        $a = array('class_table' => $class_table,
            'SatzTyp' => 6, 'attribute' => '0.7'  );
        if (false == $p1->execute($a )){
            http_response_code(425);
            echo 'Fehler bei p1-6';
            echo var_dump($p1->errorInfo());
            echo var_dump($a);
            die();
        }
        
        if ($query->dimKey != null){
            $a = array('class_table' => $class_table,
                'SatzTyp' => 8, 'attribute' => $query->dimKey);
            if (false == $p1->execute($a)){
                http_response_code(425);
                echo 'Fehler bei p1-8';
                echo var_dump($p1->errorInfo());
                echo var_dump($a);
                die();
            }
        }
        for ($i = 0; $i < $query->addQueries->size(); $i++) {
            if ($query->addQueries->getPos($i)->class_table != null){
                $a = array('class_table' => $class_table,
                    'SatzTyp' => 100+$i, 'attribute' => $query->addQueries->getPos($i)->class_table);
                if (false == $p1->execute($a)){
                    http_response_code(425);
                    echo 'Fehler bei p1-'.(100+$i);
                    echo var_dump($a);
                    die();
                }
            }
        }
        $p1->closeCursor();
        
        
        if ($query->hatWerteAuswahl){
            $metaRecord = $query->metaData['WSR_WERTELISTE'];
            if ($metaRecord != null){
                $metaRecord->save(0,$i);
            }else{
                http_response_code(425);
                echo "FEHLER beim Speichern der Datenquelle\n";
                echo 'WSR_WERTELISTE hat keine Metadaten';
                die();
            }            
        }
        
        for ($i=0; $i < count($query->rowVars); $i++) {
            $metaRecord = $query->metaData[$query->rowVars[$i]];
            if ($metaRecord != null){
                $metaRecord->save(1,$i);
            }else{
                http_response_code(425);
                echo "FEHLER beim Speichern der Datenquelle\n";
                echo $query->rowVars[$i].' hat keine Metadaten';
                die();
            }
        }
        for ($i=0; $i < count($query->valueVars); $i++) {
            $metaRecord = $query->metaData[$query->valueVars[$i]];
            if ($metaRecord != null){
                $metaRecord->save(2,$i);
            }else{
                http_response_code(425);
                echo "FEHLER beim Speichern der Datenquelle\n";
                echo $query->valueVars[$i].' hat keine Metadaten';
                die();
            }
        }
                
        for ($i=0; $i < count($query->filterVars); $i++) {
            $metaRecord = $query->metaData[$query->filterVars[$i]];
            if ($metaRecord != null){
                $metaRecord->setTable($class_table);
                $metaRecord->save(3,$i);
            }else{
                http_response_code(425);
                echo "FEHLER beim Speichern der Datenquelle\n";
                echo $query->filterVars[$i].' hat keine Metadaten';
                die();
            }
        }
        
          
        for ($a=0; $a < $query->addQueries->size(); $a++) {
            $ad = $query->addQueries->getPos($a);
            for ($i=0; $i < count($ad->calcVars); $i++) {
                $vn = $ad->calcVars[$i];
                $metaRecord = $ad->getQuery()->metaData[$vn];
                $metaRecord->setTable($class_table);
                $metaRecord->save($a+100, $i);
            }
        }
        
        
        foreach ($query->dimensions as $dim){
            $dim->save();
        }
        
        
        if (count($query->defaultFilter) > 0){
            $pd = $dbh->prepare("insert into queryDefaultFilter set table_name=:class_table,"
                ."column_name=:column_name, qAddFilterVon=:fs_von,qAddFilterBis=:fs_bis");
            $a  = array('class_table' => $class_table);
            for ($i = 0; $i < count($query->defaultFilter); $i ++){
                if (array_key_exists($query->defaultFilter[$i]->column_name, $query->metaData)){
                    $a['column_name'] = $query->defaultFilter[$i]->column_name;
                    $a['fs_von'] = $query->defaultFilter[$i]->fs_von;
                    $a['fs_bis'] = $query->defaultFilter[$i]->fs_bis;
                    if (false === $pd->execute($a)){
                        http_response_code(425);
                        echo 'Fehler bei defaultFilter ';
                        var_dump($dbh->errorInfo());
                        var_dump($a);
                        die();
                    }
                }
            }
            $pd->closeCursor();
        }

        // Log für Speichern
        $logSql = new SqlAbfrage('WSR');
        $logSql->sql = 'update wsrAdminQuery set timeStored = sysdate()';
        $logSql->addKlausel(', userStored = ', $_SESSION['Name']);
        if ($query->queryImported){
            // Log für Importierten Eintrag
            if (isValid($query->importTimeCreated))
                $logSql->addKlausel(', timeCreated = ',$query->importTimeCreated);
            if (isValid($query->importUserCreated))
                $logSql->addKlausel(', userCreated = ',$query->importUserCreated);

            $logSql->addKlausel(', timeReleased = ',$query->importTimeReleased);
            $logSql->addKlausel(', userReleased = ',$query->importUserReleased);
        }
        $logSql->addSel('table_name = ', $query->class_table);
        $erg = $logSql->executeE(false);
        if ($erg === false || $erg->rowCount() < 1 ){
            error_log('Fehler beim Update wsrAdminQuery: '. $logSql->errorInfo()[2]);
        }
        
        
        $query->queryImported = false;
        
  echo "Die Datenquelle ".$class_table." wurde gespeichert.";
        
