<?php
/**
 * Projekt: WS-Reporter
 *          Sammlung von Funktionen für Übersetzungen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    20.9.2022
 *
 *
 */


require_once 'includes/classes.inc';
require_once 'includes/wsrTranslate.inc';

/**
 * Liste der Verwendungen eines Textes
 * @param $tSearch string
 * @return PDOStatement
 */
function  sqlSearch($tSearch){
    /* Liste der zu übersetzenden Texte */
    $tSearch = wsrDecode($tSearch);
    $sql = new SqlAbfrage('WSR');
    
    $sql->sql = "select 1 as rt, 'Default Report-Titel' as type , '' as column_name, '' as column_label, table_name  from queryAttribute";
    $sql->addsel('attribute = ',$tSearch);
    $sql->addWhere('SatzTyp = 1 ');

    $sql->sql .= "\nunion\nselect 1 as rt, 'Titel (Label) der Datenquelle' as type , '' as column_name, '' as column_label, table_name  from queryAttribute";
    $sql->firstWhere = true;
    $sql->addsel('attribute = ',$tSearch);
    $sql->addWhere('SatzTyp = 2 ');
    
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'physische Tabelle ' as type , '' as column_name, '' as column_label, table_name from queryAttribute";
    $sql->firstWhere = true;
    $sql->addsel('attribute = ',$tSearch);
    $sql->addWhere('SatzTyp = 4 ');
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'Label' as type, column_name, column_label, table_name from queryColumnMetadata";
    $sql->firstWhere = true;
    $sql->addsel('column_label = ',$tSearch);
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'Beschreibung' as type, column_name, column_label, table_name from queryColumnMetadata";
    $sql->firstWhere = true;
    $sql->addsel('column_desc = ',$tSearch);
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'Filter-SQL' as type, column_name, column_label, table_name  from queryColumnMetadata";
    $sql->sql .= "\n where (qSelectVar = ";
    $sql->addWert($tSearch);
    $sql->sql .= 'or (dimNr > 0 and qSelectVar = replace(';
    $sql->addWert($tSearch);
    $sql->sql .= ",'dim.',concat('d',dimNr,'.'))))";    
    $sql->sql .= ' and qFilterTyp = '.MetaRecord::SONDERFILTER;

    $sql->sql .= "\nunion\nselect 1 as rt, 'Web-Link' as type, column_name, column_label, table_name  from queryColumnMetadata";
    $sql->sql .= "\n where (qSelectVar = ";
    $sql->addWert($tSearch);
    $sql->sql .= 'or (dimNr > 0 and qSelectVar = replace(';
    $sql->addWert($tSearch);
    $sql->sql .= ",'dim.',concat('d',dimNr,'.'))))";
    $sql->sql .= ' and qFilterTyp = '.MetaRecord::SONDERFILTER;
    
    
    $sql->sql .= ' and qColType = '.MetaRecord::WEBLINK;
    
    $sql->sql .= "\nunion\nselect 1 as rt,  'Display-SQL' as type, column_name, column_label, table_name  from queryColumnMetadata";
    $sql->sql .= "\n where (qDisplayVar = ";
    $sql->addWert($tSearch);
    $sql->sql .= 'or (dimNr > 0 and qDisplayVar = replace(';
    $sql->addWert($tSearch);
    $sql->sql .= ",'dim.',concat('d',dimNr,'.'))))";
    
    
    
    $sql->sql .= "\nunion\nselect 1 as rt,  'Filter-SQL in ' as type, m.column_name, m.column_label as column_label, m.table_name "
        ." from queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
    $sql->firstWhere = true;
    $sql->addWhere("m.qFilterTyp = ".MetaRecord::SONDERFILTERSEL
        ." and m.column_name = s.column_name and m.table_name = s.table_name ");
    $sql->sql .= "\n and (s.label = ";
    $sql->addWert($tSearch);
    $sql->sql .= 'or (dimNr > 0 and s.label = replace(';
    $sql->addWert($tSearch);
    $sql->sql .= ",'dim.',concat('d',dimNr,'.'))))";
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'In Sonderfilter von' as type, m.column_name, m.column_label  as column_label, m.table_name "
        ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
    $sql->firstWhere = true;
    $sql->addWhere("m.qFilterTyp = ".MetaRecord::SONDERFILTERSEL
        ." and m.column_name = s.column_name and m.table_name = s.table_name ");
    $sql->addsel('s.value = ',$tSearch);
    
    $sql->sql .= "\nunion\nselect 1 as rt, 'In Sonderfilter von' as type, m.column_name, m.column_label, m.table_name "
        ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
    $sql->firstWhere = true;
    $sql->addWhere("m.qFilterTyp != ".MetaRecord::SONDERFILTERSEL
        ." and m.column_name = s.column_name and m.table_name = s.table_name ");
    $sql->addsel('s.value = ',$tSearch);
    
    // Suche in Report-Definitionen
    $sql->sql .= "\nunion\nselect 2 as rt, 'Table-Prefix' as type, ' in ' as column_name, '' as column_label, r.reportTitle_de as table_name "
        ." from reportJoinedTables s, reportTitle r ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.prefix = ',$tSearch);
    
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Titel' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from (reportRows s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
        
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_title = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Link-Titel' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportRows s, reportTitle r , reportAttributes a)"
            ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_link_title = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Link-Anzeige' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportRows s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_link_value = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Link' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportRows s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_link = ',$tSearch);
    $sql->addWhere('s.web_link like "#%"');
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Link in Spalte' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportColumns s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_link = ',$tSearch);
    $sql->addWhere('s.web_link like "#%"');
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Web-Titel' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportRows s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
        
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.web_title = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Werte-Label' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportValues s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.label = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql .= "\nunion\nselect 2 as rt, 'Spalten-Label' as type, s.column_name, m.column_label, r.reportTitle_de as table_name "
        ." from ( reportColumns s, reportTitle r , reportAttributes a)"
        ." left join queryColumnMetadata m on s.column_name = m.column_name and a.table_name = m.table_name ";
    $sql->firstWhere = true;
    $sql->addWhere("s.reportID = r.reportID ");
    $sql->addsel('s.column_label = ',$tSearch);
    $sql->addWhere("s.reportID = a.reportID ");
    
    $sql->sql.="\norder by rt, type, column_name, table_name";
    
//     echo '<pre>'.$sql->sql.'<pre>';
    // return;
    
    
    $stmt = $sql->execute();
    if ($stmt === false){
        var_dump($sql);
        var_dump($dbh->errorInfo());
        die();
    }
    return $stmt;
}

class TCROW {
    public $tRowspan = 0;
    public $cRowspan = 0;
    public $lRowspan = 0;
    public $type = null;
    public $name = null;
    public $label = null;
    public $table = null;
    public $rt = 0;
}

/**
 * 
 * @param PDOStatement $stmt
 * @param string $title
 * @param string $tableId
 * @param int $version
 * 
 * @return int anzahl Verwendungen
 */
function showUsage($stmt,$title,$tableId,$version){
    /**
     *
     * @var array TCROW[]
     */
    
    $twerte = array();
    $tcrow = new TCROW();
    $tcType = $tcrow;
    $tcCol = $tcrow;
    $tcLabel = $tcrow;
    $c = 0;
    // $twerte[] = $tcrow;
    
    while ($z = $stmt->fetchObject()){
        /*
         echo '<tr><th class="left">'.$z->type
         .'</th><td>'.$z->column_name.'</td><td>'.$z->column_label.'</th><td>'.$z->table_name."</td></tr>\n";
         */
        $c ++;
        if (count($twerte) == 0 ){
            $tcType->type = $z->type;
            $tcType->tRowspan ++;
            $tcCol->name = $z->column_name;
            $tcCol->cRowspan ++;
            if (!isValid($z->column_label)) $tcLabel->label = '&nbsp;';
            else $tcLabel->label = $z->column_label;
            $tcLabel->lRowspan ++;

            if ($version <= 2){
            ?>
<table id='<?=$tableId?>'>
<thead>
<tr><th><?=WSRTranslate::translate('Art')?></th><th><?=WSRTranslate::translate('Variable')?></th>
<th><?=WSRTranslate::translate('Label')?></th><th>
<?php
if ($z->rt == 1) echo WSRTranslate::translate('Datenquelle'); 
else echo WSRTranslate::translate('Report');
$rt = $z->rt;
if (isValid($title)) echo '</th></tr><tr><th colspan="4">'.$title;
?>
        </th></tr>
        </thead><tbody>
<?php
            } // $version <=2
            else{
                if (isValid($title)) echo '<tr><th colspan="4">'.$title.'</th></tr>';
            }
    }else{
        $tcrow = new TCROW();
        
        if ($tcType->type == $z->type) {
            $tcType->tRowspan ++;
        }else{
            $tcType = $tcrow;
            $tcrow->type = $z->type;
            $tcType->tRowspan = 1;
        }
        
        if ($tcCol->name == $z->column_name) {
            $tcCol->cRowspan ++;
        }else{
            $tcCol = $tcrow;
            $tcCol->name = $z->column_name;
            $tcCol->cRowspan = 1;
        }

        if ($tcLabel->label == $z->column_label) {
            $tcLabel->lRowspan ++;
        }else{
            $tcLabel = $tcrow;
            if (!isValid($z->column_label)) $tcLabel->label = '&nbsp;';
            else $tcLabel->label = $z->column_label;
            $tcLabel->lRowspan = 1;
        }
        
    }
    $tcrow->table = $z->table_name;
    $tcrow->rt = $z->rt;
    $twerte[] = $tcrow;
}
if (count($twerte) > 0) $rt = $twerte[0]->rt;
for ($i = 0; $i < count($twerte); $i++){
    $tcrow = $twerte[$i];
    if ($tcrow->rt > $rt){
        $rt = $tcrow->rt;
        echo '<tr><th colspan="3">&nbsp;</th><th>Report</th></tr>';
    }
    echo '<tr>';
    if ($tcrow->type != null) echo '<th class="left" rowspan="'.$tcrow->tRowspan.'">'.$tcrow->type.'</th>';
    else if ($i == 0) echo '<th></th>';
    if ($tcrow->name != null) echo '<td rowspan="'.$tcrow->cRowspan.'">'.$tcrow->name.'</td>';
    else if ($i == 0)  echo '<th></th>';
    if ($tcrow->label != null) echo '<td rowspan="'.$tcrow->lRowspan.'">'.$tcrow->label.'</td>';
    else  if ($i == 0) echo '<th></th>';
    echo '<td>'.$tcrow->table."</td></tr>\n";
}
if ($version < 2){
?>
</tbody>
</table>
<?php
}
return $c;
}
?>
