<?php 
/**
 * Projekt: WS-Reporter
 *          Übersetzungen zu den Datenquellen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    12.8.2022
 * 
 * Hinweise:
 * Es wird die gespeicherte Version einer Datenquelle übersetzt.
 * Das aktuelle Query-Objekt aus der Session wird ignoriert.  
 *
 */

/* ??
if (array_key_exists('class_table', $_REQUEST)){
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 
}
*/
include 'includes/defender.inc';

require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
require_once './includes/qTranslate.inc';
if (session_status() == PHP_SESSION_NONE)  session_start();
anmeldungPruefen();
uebersetzerRechteVerlangen();

global $dbh, $query;

/**
 * Frage: Soll hier das aktuelle Objekt übersetzt werden, indem 
 * man die einzelnen Texte hervorholt?
 * 
 * Oder wird die Übersetzungstabelle zur Bearbeitung angezeigt und
 * die nicht übersetzten Einträge folgen unten?
 * 
 * Können nur gespeicherte Datenquellen übersetzt werden?
 * Was ist beim Import?
 *  
 * Antworten:
 * * Übersetzte Einträge sollten von nicht übersetzten Einträgen separiert werden. 
 * * Gruppierung nach Art des Eintrags (qLanguageType)
 * 
 * Überlegungen:
 * * Wenn nur gespeicherte Datenquellen übersetzt werden können,
 *   kann die Aufbereitung der Tabelle in der Datenbank mit 
 *   Temporary Tables erfolgen.
 */

if (array_key_exists('table', $_REQUEST)){
    $table = $_REQUEST['table'];
}else{
    $table = $query->class_table;
}
if (!isValid($table)){
    http_response_code(400);
    echo 'Name der Datenquelle fehlt (Parameter: table)';
    die();    
}

/* Sprachen bestimmen */
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select attribute from queryAttribute';
$sql->addSel('table_name = ', $table);
$sql->addSel('SatzTyp = ', 19);
$stmt = $sql->execute();
if ($stmt === false) {
    var_dump($sql);
    var_dump($dbh->errorInfo());
    die();
}
$z = $stmt->fetchObject();
$sprachen = $z->attribute;
if (isValid($sprachen)){
    $qTranslations = explode(' ', $sprachen);
    $qLanguage = $qTranslations[0];
}else{
    $qLanguage = 'de';
    $qTranslations = array(0=>'de', 1=>'en');
}

/* Liste der zu übersetzenden Texte */
$sql = new SqlAbfrage('WSR');

$sql->sql = "select distinct case when t.qLanguageType is null then 2 else 1 end as gsort, q.type, q.text, t.* ";

$sql->sql .= "from (select 1 as sort, ".qTranslate::QT_LABEL." as type, attribute as text from queryAttribute";
$sql->addSel('table_name = ', $table);
$sql->addWhere('SatzTyp in (1,2) ');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQLTABLE." as type, attribute as text from queryAttribute";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);
$sql->addWhere('SatzTyp = 4 ');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, column_label as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_TEXT." as type, column_desc as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_TEXT." as type, column_hints as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);



$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQL." as type,"; 
$sql->sql .= 'case when dimNr > 0 then replace(qSelectVar,concat("d",dimNr,"."),"dim.") else qSelectVar end ';
$sql->sql .= " as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);
$sql->addWhere('qFilterTyp = '.MetaRecord::SONDERFILTER);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type,";
$sql->sql .= 'case when dimNr > 0 then replace(qSelectVar,concat("d",dimNr,"."),"dim.") else qSelectVar end ';
$sql->sql .= " as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);
$sql->addWhere('qColType = '.MetaRecord::WEBLINK);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQLDISP." as type, ";
$sql->sql .= 'case when dimNr > 0 then replace(qDisplayVar,concat("d",dimNr,"."),"dim.") else qDisplayVar end ';
$sql->sql .= " as text from queryColumnMetadata";
$sql->firstWhere = true;
$sql->addSel('table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_SQL." as type, s.label as text "
   ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
$sql->firstWhere = true;
$sql->addWhere("m.qFilterTyp = ".MetaRecord::SONDERFILTERSEL
    ." and m.column_name = s.column_name and m.table_name = s.table_name ");
$sql->addSel('m.table_name = ', $table);


$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, s.value as text "
    ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
$sql->firstWhere = true;
$sql->addWhere("m.qFilterTyp = ".MetaRecord::SONDERFILTERSEL
    ." and m.column_name = s.column_name and m.table_name = s.table_name ");
$sql->addSel('m.table_name = ', $table);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, s.label as text "
    ."from  queryColumnMetadataSelectOptions s, queryColumnMetadata m ";
$sql->firstWhere = true;
$sql->addWhere("m.qFilterTyp != ".MetaRecord::SONDERFILTERSEL
    ." and m.column_name = s.column_name and m.table_name = s.table_name ");
$sql->addSel('m.table_name = ', $table);
        
$sql->sql .="\n) q left join queryTranslations t on t.".$qLanguage." = trim(q.text) "; // and q.type = t.qLanguageType";        
$sql->sql .="\n and (q.type != ".qTranslate::QT_SQLTABLE.' or t.qLanguageType = '.qTranslate::QT_SQLTABLE.')'; 

$sql->sql .="\nwhere q.text is not null and q.text != ''";
$sql->sql .="\norder by gsort,  type, q.text";

$stmt = $sql->execute();
if ($stmt === false){
    echo '<pre>';
    var_dump($sql);
    var_dump($dbh->errorInfo());
    die();
}

?>
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<title>WS-Reporter: <?=WSRTranslate::translate('Übersetzungen für die Datenquelle')?> <?=$table ?></title>
<meta name="GENERATOR" content="WS-PHP: editQuery.php">
<link rel="stylesheet" href="./jscss/standard2.css">
<link rel="stylesheet" href="./jscss/WSReport.css" >
<script type="text/javascript" src="./jscss/translate.js">
	</script>
<script type="text/javascript" src="./jscss/tableHeader.js">
	</script>
<link rel="stylesheet" href="./jscss/tableFolder.css">
<script src="./jscss/tableFolder.js">
	</script>	
<script type="text/javascript">
 language = "<?=$qLanguage?>";
	</script>
</head>
<?php 
if ($_SERVER["HTTP_HOST"] == "localhost:8898"){
    ?>
<body class = "entwicklung">	
<?php
} else echo '<body>';
?>

<h1><?=WSRTranslate::translateH('Übersetzungen für Datenquelle')?> <?=$table ?></h1>

<div id="Editor" class = "joinWerteBox">
 <div class = "xButton" onclick="closeEditor()" title="<?=WSRTranslate::translate('schließen')?>">&otimes;</div>
 <h3><?=WSRTranslate::translateH('Verwendungen des Textes')?></h3>
<pre id="VerwText"></pre>
<?=WSRTranslate::translateH('wird verwendet von:')?><br>
<div id = "VerwAnzeige">
</div>
<br>  <button onclick="closeEditor()"><?=WSRTranslate::translate('Schließen')?></button> 
</div>

<table id='Tabelle'>
<thead>
<tr><th class="WSFHeader">&nbsp;</th><th class='fts'>Text (<?=$qTranslations[0] ?>)</th>
<?php 
for ($i = 1; $i < count($qTranslations); $i++){
    echo '<th>'.$qTranslations[$i].'</th>';
}
?>
</tr>
</thead>
<tbody>
<?php 
$type = -1;
$gsort = 1;
$c = 0;
$rowspans = array();
if (! array_key_exists('SQL', $_REQUEST))
while ($z = $stmt->fetch(PDO::FETCH_ASSOC)){
    $c++;
    if ($z['gsort'] != $gsort){
        $gsort = $z['gsort'];
        $type = -1;
        echo '<tr><th>&nbsp;</th><th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Ohne Übersetzungen').'</th></tr>';
    }
    if ($z['type'] != $type){
        $type = $z['type'];
        echo '<tr><th id="R'.$gsort.$type.'" class="WSFHeader">&nbsp;</th>';
        $rowspans['R'.$gsort.$type]  = 1;
        switch ($type) {
            case qTranslate::QT_LABEL:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Label').'</th></tr>';
            break;
            case qTranslate::QT_SQL:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Filter-SQL').'</th></tr>';
                break;
            case qTranslate::QT_SQLDISP:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Display-SQL').'</th></tr>';
                break;
            case qTranslate::QT_SQLTABLE:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Tabellen').'</th></tr>';
                break;
            case qTranslate::QT_TEXT:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Text').'</th></tr>';
                break;                
            case qTranslate::QT_WEBLINK:
                echo '<th colspan="'.(count($qTranslations)).'">'.WSRTranslate::translateH('Web-Link').'</th></tr>';
                break;
            default:
                ;
            break;
        }
    }
    $rowspans['R'.$gsort.$type]  ++;
    $rows = 0;
    $t = explode("\n",$z['text']);
    foreach ($t as $tz)
        if (strlen($tz) > 150) $rows += 4;
        elseif (strlen($tz) > 100) $rows += 3;
        elseif (strlen($tz) > 50) $rows += 2;
        else $rows++;
        ?>
        
<tr><td class='fts nowrap' ><textarea rows="<?=$rows?>" cols="50" id='<?='T'.$c.$qLanguage?>' readonly >
<?=htmlspecialchars($z['text'])?>
</textarea>
<span class="gButton"  title='<?=WSRTranslate::translate('Eintrag löschen')?>' onclick="deleteTranslationById('<?='T'.$c.$qLanguage?>','<?=$qLanguage?>')">DEL</span>
<span class="gButton" title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$c.$qLanguage?>')">V</span>
</td>  
<?php 
for ($l = 1; $l < count($qTranslations);$l++){
?>
<td class='nowrap'><textarea rows="<?=$rows?>" cols="50" id='<?='T'.$c.$qTranslations[$l]?>' 
     lang="<?=$qTranslations[$l] ?>" onchange='enter(this,<?=$type?>)' 
     ondblclick = 'showTranslation(this.value,"<?=$qTranslations[$l]?>")'
     ><?= htmlspecialchars($z[$qTranslations[$l]])?>
</textarea>
<span class="gButton"  title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$c.$qTranslations[$l]?>')">V</span>
<span class="gButton"  title='<?=WSRTranslate::translate('Übersetzungen anzeigen')?>' onclick="showTranslationById('<?='T'.$c.$qTranslations[$l]?>')">T</span>
</td>
<?php 
}
?>
</tr>
<?php 
}

?>
</tbody>
</table>

<h3><?=WSRTranslate::translateH('Beschreibungen')?></h3>
<div class='center top'>
<?php 
function showDescription($lang){
    global $table;
    echo '<br><b>'.qTranslate::sprache($lang).':</b><br>';
    ?>
 <textarea rows='20' cols='80'
 lang="<?=$lang?>" onchange='setQText(this,"<?=$table.'",'.qTranslate::QUERY_COMMENT ?>)' ><?php
  $sql = new SqlAbfrage('WSR');
  $sql->sql = 'select kommentar from queryComment';
  $sql->addSel('table_name = ',$table);
  $sql->addSel('qLanguage = ', $lang);
  $sql->sql .= ' order by SortNr';
  $stmt = $sql->executeE(true);
  while ($z = $stmt->fetchObject()) echo $z->kommentar;
?></textarea><br>
<?php
  $stmt->closeCursor();
}
foreach ($qTranslations as $lang){
    if (isValid($lang))
        showDescription($lang);
}
?>
</div>

<h3><?=WSRTranslate::translateH('Technische Hinweise')?></h3>
<div class='center top'>
<?php 
function showHints($lang){
    global $table;
    echo '<br><b>'.qTranslate::sprache($lang).':</b><br>';
    ?>
 <textarea rows='20' cols='80'
 lang="<?=$lang?>" onchange='setQText(this,"<?=$table.'",'.qTranslate::QUERY_HINTS ?>)' ><?php
  $sql = new SqlAbfrage('WSR');
  $sql->sql = 'select kommentar from queryHints';
  $sql->addSel('table_name = ',$table);
  $sql->addSel('qLanguage = ', $lang);
  $sql->sql .= ' order by SortNr';
  $stmt = $sql->executeE(true);
  while ($z = $stmt->fetchObject()) echo $z->kommentar;
?></textarea><br>
<?php
  $stmt->closeCursor();
}
foreach ($qTranslations as $lang){
    if (isValid($lang))
        showHints($lang);
}
?>
</div>


<?php
if ($c < 1 || array_key_exists('SQL', $_REQUEST)){
    $s = $sql->sql;
    for ($i = 0; $i <= 8; $i++){
        $s = str_replace(':filterPar'.$i, '"'.$table.'"', $s);
    }
    echo '<pre>'.$s.'</pre>';
}else{
?>
 <script>
    window.addEventListener('load', function() {
<?php    
    foreach ($rowspans as $id => $count){
        echo 'document.getElementById("'.$id.'").rowSpan='.$count.";\n";
    }
?>    
    var tf = new TableFolder('Tabelle',TableFolder.NachKlassen,TableFolder.Eingeklappt,true);
    }
   ); 
   </script>
<?php    
}
?>
</body>
</html>
